@@ -29,7 +29,7 @@ void Board::DestroyComponent(Component& comp)	// Destroys a component on the boa
 	m_compMgr.DestroyComp(comp);			// Destroy the component in the m_compMgr
 }
 
-int Board::CreateComponent(int iRow, int iCol, const COMP& eType, const Component* pComp)
+int Board::CreateComponent(int iRow, int iCol, COMP eType, const Component* pComp)
 {
 	assert( pComp == nullptr || pComp->GetType() == eType );	// Sanity check
 	assert( eType != COMP::INVALID );
@@ -164,21 +164,23 @@ bool Board::CanPutDown(Component& comp)	// Checks if its possible to place the (
 	const bool	bAllowHoleShare	= GetWireShare();
 	const bool	bDiagsOK		= GetDiagsMode() != DIAGSMODE::OFF;
 	const bool	bWire			= comp.GetType() == COMP::WIRE;	// Wire's only get NodeIDs while placed
-	const bool	bMark			= comp.GetType() == COMP::MARK;	// Marker can go anywhere without a pin
+	const bool	bSOIC			= comp.GetIsSOIC();
+	const bool	bMark			= comp.GetType() == COMP::MARK;	// Marker can go anywhere without a pin or SOIC pattern
 	const bool	bTrax			= comp.GetType() == COMP::TRACKS;
 	const int&	compCols		= comp.GetCompCols();
 	const int&	compRows		= comp.GetCompRows();
-	const int&	lyr				= comp.GetLyr();
+	const int&	compLyr			= comp.GetLyr();	assert(bTrax || compLyr == 0);
 	const int&	rowTL			= comp.GetRow();
 	const int&	colTL			= comp.GetCol();
 	const int&	boardCols		= GetCols();
 	const int&	boardRows		= GetRows();
+	if ( bSOIC && !GetHaveTopLyr() ) return false;	// SOIC parts need to use top layer
 
 	if ( bAllowHoleShare && bWire )
 	{
 		// pA and pB are the opposite ends of the wire
-		Element* pA = Get(lyr, rowTL, colTL);						assert( pA );
-		Element* pB = Get(lyr, rowTL+compRows-1, colTL+compCols-1);	assert( pB );
+		Element* pA = Get(compLyr, rowTL, colTL);						assert( pA );
+		Element* pB = Get(compLyr, rowTL+compRows-1, colTL+compCols-1);	assert( pB );
 		assert( !pA->GetCompExists(comp.GetId()) && !pB->GetCompExists(comp.GetId()) );
 		if ( pA->GetWireExists(pB) || pB->GetWireExists(pA) ) return false;	// No duplicates !!!
 	}
@@ -194,7 +196,7 @@ bool Board::CanPutDown(Component& comp)	// Checks if its possible to place the (
 		for (int i = 0; i < compCols && bOK; i++, iCol++)
 		{
 			const CompElement*	pComp = comp.GetCompElement(j, i);
-			const Element*		pGrid = Get(lyr, jRow, iCol);
+			const Element*		pGrid = Get(compLyr, jRow, iCol);
 			if ( bTrax )
 			{
 				const int traxNodeId = pComp->GetNodeId();
@@ -202,7 +204,7 @@ bool Board::CanPutDown(Component& comp)	// Checks if its possible to place the (
 
 				bOK =  ( traxNodeId == BAD_NODEID )
 					|| ( traxNodeId == pGrid->GetNodeId() )
-					|| ( !pGrid->GetHasPin() && pGrid->GetNodeId() == BAD_NODEID && !pGrid->GetIsHole() );
+					|| ( !pGrid->GetLyrHasPin() && pGrid->GetNodeId() == BAD_NODEID && !pGrid->GetIsHole() && !(pGrid->GetSoicProtected() && traxNodeId != BAD_NODEID) );
 
 				if ( !bOK ) // Special check for unpainted wires on the board
 				{
@@ -245,6 +247,8 @@ bool Board::CanPutDown(Component& comp)	// Checks if its possible to place the (
 				const uchar& compSurface	= pComp->GetSurface();
 				const uchar& boardHoleUse	= pGrid->GetHoleUse();
 				const uchar& compHoleUse	= pComp->GetHoleUse();
+				const uchar& boardSoicChar	= pGrid->GetSoicChar();
+				const uchar& compSoicChar	= pComp->GetSoicChar();
 
 				// Check surface and hole use.
 				// Need separate checks for SURFACE_FREE since that can be
@@ -253,20 +257,24 @@ bool Board::CanPutDown(Component& comp)	// Checks if its possible to place the (
 						( compSurface  == SURFACE_FREE ) ||
 						( boardSurface + compSurface <= SURFACE_FULL );
 				bOK &=	( boardHoleUse + compHoleUse <= HOLE_FULL );
+				bOK &=	( !GetHaveTopLyr() || !(compSoicChar & SOIC_TRACKS_TOP) || Get( LYR_TOP, jRow, iCol)->GetNodeId() == BAD_NODEID );	// Cannot place SOIC if board is painted in top SOIC tracks area
+				bOK &=	(                     !(compSoicChar & SOIC_TRACKS_BOT) || Get( LYR_BOT, jRow, iCol)->GetNodeId() == BAD_NODEID );	// Cannot place SOIC if board is painted in bottom SOIC tracks area
+				bOK &=	( boardSoicChar + compSoicChar <= SOIC_FULL );
+				bOK &=	( !pGrid->GetIsMark() || (compSoicChar & (SOIC_TRACKS_TOP | SOIC_TRACKS_BOT)) == 0 );
 				bOK &=	( !bWire || bAllowHoleShare || ( boardHoleUse + compHoleUse <= HOLE_WIRE ) );
 				bOK &=	( !bWire || bAllowWireCross || ( boardSurface <= ( bAllowHoleShare ? SURFACE_WIRE_END | SURFACE_GAP : SURFACE_GAP ) ) );
 				if ( !bOK ) continue;
 
 				// Check pins
-				if ( bMark )	// Marker can go anywhere except for pins, holes, (or other marker)
+				if ( bMark )	// Marker can go anywhere except for pins, holes, SOIC pattern areas (or other marker)
 				{
-					bOK = !pGrid->GetHasPin() && !pGrid->GetIsHole() && !pGrid->GetIsMark();
+					bOK = !pGrid->GetHasPin() && !pGrid->GetIsHole() && !pGrid->GetIsMark() && (boardSoicChar & (SOIC_TRACKS_TOP | SOIC_TRACKS_BOT)) == 0;
 				}
 				else if ( pComp->GetIsHole() )	// Check holes
 				{
 					// We've already checked the boardSurface is SURFACE_FREE
-					// Now test it is not a marker and not painted
-					bOK = !pGrid->GetIsMark() && pGrid->GetNodeId() == BAD_NODEID;
+					// Now test it is not a marker and not painted on any board layer
+					bOK = !pGrid->GetIsMark() && pGrid->GetNodeId() == BAD_NODEID && ( pGrid->GetNbr(NBR_X) == nullptr || pGrid->GetNbr(NBR_X)->GetNodeId() == BAD_NODEID );
 				}
 				else
 				{
@@ -276,21 +284,21 @@ bool Board::CanPutDown(Component& comp)	// Checks if its possible to place the (
 					bOK = !pGrid->GetIsMark(); // Pin can't go on marker
 					if ( !bOK ) continue;
 
-					// Check bottom grid
-					const int& nodeId		= pGrid->GetNodeId();		// Read nodeID on board
-					const int& iCompNodeId	= comp.GetNodeId(pinIndex);	// Read component nodeID
+					// Check relevant layer to get nodeID for pin
+					const Element*	p				= ( bSOIC || pGrid->GetHasPinSOIC() ) ? ( LYR_TOP == compLyr ? pGrid : pGrid->GetNbr(NBR_X) ) : pGrid;	assert(p);
+					const int&		nodeId			= p->GetNodeId();			// Read nodeID on board
+					const int&		iCompNodeId		= comp.GetNodeId(pinIndex);	// Read component nodeID
 
 					if ( bWire )	// Wires have no NodeId. Need matching IDs on both ends
 						assert( iCompNodeId == BAD_NODEID );	// Shouldn't have an ID yet
 					else			// Regular component ...
 					{
-						bOK = ( nodeId == BAD_NODEID || nodeId == iCompNodeId );	// Need no node ID or matching ID
-
-						if ( bOK )
+						bOK = p->GetHasPin() ? ( nodeId == iCompNodeId )							// Need matching node ID
+											 : ( nodeId == BAD_NODEID || nodeId == iCompNodeId );	// Need no node ID or matching node ID
+						if ( bOK && ( p->GetHasPinTH() || (pComp->GetSoicChar() & SOIC_THL_COMP)) )	// Check for short-circuit between layers
 						{
-							// Check for short-circuit between layers
-							Element* q = pGrid->GetNbr(NBR_X);
-							if ( q != nullptr && q != pGrid )
+							Element* q = p->GetNbr(NBR_X);
+							if ( q != nullptr && q != p )
 							{
 								const int& nodeId = q->GetNodeId();	// Read nodeId on other board layer
 								bOK = ( nodeId == BAD_NODEID || nodeId == iCompNodeId );	// Need no node ID or matching ID
@@ -304,9 +312,9 @@ bool Board::CanPutDown(Component& comp)	// Checks if its possible to place the (
 	if ( bOK && bWire )
 	{
 		// Check for short-circuit in this layer
-		assert( lyr == 0 );
-		Element* pW0 = Get(lyr, rowTL, colTL);							assert( pW0 );
-		Element* pW1 = Get(lyr, rowTL+compRows-1, colTL+compCols-1);	assert( pW1 );
+		assert( compLyr == 0 );
+		Element* pW0 = Get(compLyr, rowTL, colTL);							assert( pW0 );
+		Element* pW1 = Get(compLyr, rowTL+compRows-1, colTL+compCols-1);	assert( pW1 );
 		bOK = pW0->GetNodeId() == BAD_NODEID ||
 			  pW1->GetNodeId() == BAD_NODEID ||
 			  pW0->GetNodeId() == pW1->GetNodeId();
@@ -344,19 +352,18 @@ bool Board::PutDown(Component& comp)	// Tries to place the (floating) component
 
 	const bool	bDiagsOK	= GetDiagsMode() != DIAGSMODE::OFF;
 	const bool	bWire		= comp.GetType() == COMP::WIRE;	// Wire's only get NodeIDs while placed
+	const bool	bSOIC		= comp.GetIsSOIC();	assert( !bSOIC || GetHaveTopLyr() );
 	const bool	bTrax		= comp.GetType() == COMP::TRACKS;
 	const int&	compId		= comp.GetId();
 	const int&	compCols	= comp.GetCompCols();
 	const int&	compRows	= comp.GetCompRows();
-	const int&	lyr			= comp.GetLyr();
+	const int&	compLyr		= comp.GetLyr();	assert(bTrax || compLyr == 0);
 	const int&	rowTL		= comp.GetRow();
 	const int&	colTL		= comp.GetCol();
 
-	int wireNodeId(BAD_NODEID);	// If we're placing a wire, this will be it's nodeId
-
-	int iTraxNbrLT(NBR_LT);	// For handling diagonals on a rotated trax
 	if ( bTrax )
 	{
+		int iTraxNbrLT(NBR_LT);	// For handling diagonals on a rotated trax
 		switch( comp.GetDirection() )
 		{
 			case 'W': iTraxNbrLT = NBR_LT;	break;
@@ -364,11 +371,10 @@ bool Board::PutDown(Component& comp)	// Tries to place the (floating) component
 			case 'N': iTraxNbrLT = NBR_LB;	break;
 			case 'S': iTraxNbrLT = NBR_RT;	break;
 		}
-	}
 
-	std::set<int> blankWireIds;	// CompIds of unpainted wires in the area covered by trax
-	if ( bTrax )
-	{
+		std::set<int> blankWireIds;	// CompIds of unpainted wires in the area covered by trax
+
+		// First scan over comp
 		int jRow(rowTL);
 		for (int j = 0; j < compRows; j++, jRow++)
 		{
@@ -376,36 +382,35 @@ bool Board::PutDown(Component& comp)	// Tries to place the (floating) component
 			for (int i = 0; i < compCols; i++, iCol++)
 			{
 				const CompElement*	pComp = comp.GetCompElement(j, i);
-				Element*			pGrid = Get(lyr, jRow, iCol);
+				Element*			pGrid = Get(compLyr, jRow, iCol);
 				if ( !pComp->ReadFlagBits(RECTSET) ) continue;		// Skip non-rect points
 				if ( pGrid->GetNodeId() != BAD_NODEID ) continue;	// Skip painted points
 				// If have wire(s) ...
-				if ( pGrid->GetW(0) ) blankWireIds.insert( pGrid->GetCompId()  );	// ... store its compId
-				if ( pGrid->GetW(1) ) blankWireIds.insert( pGrid->GetCompId2() );	// ... store its compId
+				if ( pGrid->GetW(0) ) blankWireIds.insert( pGrid->GetSlotCompId(0)  );	// ... store its compId
+				if ( pGrid->GetW(1) ) blankWireIds.insert( pGrid->GetSlotCompId(1) );	// ... store its compId
 			}
 		}
-	}
 
-	int jRow(rowTL);
-	for (int j = 0; j < compRows; j++, jRow++)
-	{
-		int iCol(colTL);
-		for (int i = 0; i < compCols; i++, iCol++)
+		// Second scan over comp
+		jRow = rowTL;
+		for (int j = 0; j < compRows; j++, jRow++)
 		{
-			const CompElement*	pComp = comp.GetCompElement(j, i);
-			Element*			pGrid = Get(lyr, jRow, iCol);
-			if ( bTrax )
+			int iCol(colTL);
+			for (int i = 0; i < compCols; i++, iCol++)
 			{
+				const CompElement*	pComp = comp.GetCompElement(j, i);
+				Element*			pGrid = Get(compLyr, jRow, iCol);
+
 				const int traxNodeId = pComp->GetNodeId();
 				assert( traxNodeId == BAD_NODEID || pComp->ReadFlagBits(RECTSET) );	// Sanity check
 				if ( traxNodeId == BAD_NODEID ) continue;	// Skip blank trax points
 
 				const bool bBlankWire		= pGrid->GetHasWire() &&
-											( blankWireIds.find( pGrid->GetCompId()  ) != blankWireIds.end() ||
-											  blankWireIds.find( pGrid->GetCompId2() ) != blankWireIds.end() );
+											( blankWireIds.find( pGrid->GetSlotCompId(0) ) != blankWireIds.end() ||
+											  blankWireIds.find( pGrid->GetSlotCompId(1) ) != blankWireIds.end() );
 				const bool bExistingNodeId	= !bBlankWire && ( pGrid->GetNodeId() == traxNodeId );
 
-				SetNodeIdByUser(lyr, jRow, iCol, traxNodeId, false);	// false ==> don't paint pins
+				SetNodeIdByUser(compLyr, jRow, iCol, traxNodeId, false);	// false ==> don't paint pins
 
 				const bool bAllLyrs(false);
 				if ( bExistingNodeId )						// If the board already had the NodeId ...
@@ -417,29 +422,54 @@ bool Board::PutDown(Component& comp)	// Tries to place the (floating) component
 				if ( bDiagsOK && j > 0 && i > 0 && pComp->GetUsed(iTraxNbrLT) != pGrid->GetUsed(NBR_LT) )
 					pGrid->SwapDiagLinks();
 			}
-			else
+		}
+	}
+	else	// Regular component (not the "trax")
+	{
+		// First scan over comp.  We need to SetSoicChar() over the whole component area before we start painting NodeIds
+		int jRow(rowTL);
+		for (int j = 0; j < compRows; j++, jRow++)
+		{
+			int iCol(colTL);
+			for (int i = 0; i < compCols; i++, iCol++)
 			{
+				const CompElement*	pComp = comp.GetCompElement(j, i);
+				Element*			pGrid = Get(compLyr, jRow, iCol);
+
 				assert( !(pGrid->GetIsHole() && pComp->GetIsHole()) );	// Can't overlay holes
 
 				// Update surface and hole use
-				pGrid->SetSurface( pGrid->GetSurface() + pComp->GetSurface() );
-				pGrid->SetHoleUse( pGrid->GetHoleUse() + pComp->GetHoleUse() );
+				pGrid->SetSurface(  pGrid->GetSurface()  + pComp->GetSurface() );
+				pGrid->SetHoleUse(  pGrid->GetHoleUse()  + pComp->GetHoleUse() );
+				pGrid->SetSoicChar( pGrid->GetSoicChar() + pComp->GetSoicChar() );
+			}
+		}
+
+		int wireNodeId(BAD_NODEID);	// If we're placing a wire, this will be it's nodeId
+
+		// Second scan over comp		
+		jRow = rowTL;
+		for (int j = 0; j < compRows; j++, jRow++)
+		{
+			int iCol(colTL);
+			for (int i = 0; i < compCols; i++, iCol++)
+			{
+				const CompElement*	pComp = comp.GetCompElement(j, i);
+				Element*			pGrid = Get(compLyr, jRow, iCol);
 
 				// Update IDs at pin location (No pin ==> Leave existing pinIndexes and compIds)
 				const size_t pinIndex = pComp->GetPinIndex();
 				if ( pinIndex == BAD_PININDEX ) continue;
 
-				// Work out which slot to use if we have a wire
+				// Work out which slot to use
 				const int iSlot = pGrid->GetFreeSlot();
-				// Want GetIsPin() methods to be private so commented out following assert
-				// assert( ( iSlot == 0 && !pGrid->GetIsPin() ) || ( iSlot == 1 && !pGrid->GetIsPin2() ) );
 
 				// Store any user-painted nodeId's under the pin (i.e. "oridId") BEFORE placing
-				// If we are about to place a wire in the same hole as an existing wire, then
-				// inherit origId from the existing wire
-				if ( bWire && pGrid->GetNumWires() == 1 )
+				// If we are about to place a part/wire in the same hole as an existing part/wire,
+				// then inherit origId from the existing part/wire
+				if ( pGrid->GetNumUsedSlots() == 1 )
 				{
-					wireNodeId = std::max(wireNodeId, pGrid->GetNodeId());
+					if ( bWire ) wireNodeId = std::max(wireNodeId, pGrid->GetNodeId());
 
 					const int iSlotOther = ( iSlot == 0 ) ? 1 : 0;
 					size_t	iOtherPinIndex;
@@ -448,21 +478,25 @@ bool Board::PutDown(Component& comp)	// Tries to place the (floating) component
 					assert( iOtherPinIndex != BAD_PININDEX && iOtherCompId != BAD_COMPID );
 
 					const Component& otherComp = m_compMgr.GetComponentById( iOtherCompId );
-					assert( otherComp.GetType() == COMP::WIRE );
-					for (int lyr = 0; lyr < 2; lyr++)
+					assert( bWire == (otherComp.GetType() == COMP::WIRE) );
+					for (int iLyr = 0; iLyr < 2; iLyr++)
 					{
-						const int& origId = otherComp.GetOrigId(lyr, iOtherPinIndex);
-						comp.SetOrigId(lyr, pinIndex, origId);
+						const int& origId = otherComp.GetOrigId(iLyr, iOtherPinIndex);
+						comp.SetOrigId(iLyr, pinIndex, origId);
 					}
 				}
 				else
 				{
-					for (int lyr = 0; lyr < 2; lyr++)
+					// We've already done SetSoicChar() (thus setting up any TH pin flags).
+					// When a TH pin flag is set, Element::GetNodeId() will assume both layers
+					// have matching NodeIds, and so always return the base layer NodeId.
+					// So we must use p->TrackElement::GetNodeId() here to avoid tunneling between layers.
+					for (int iLyr = 0; iLyr < 2; iLyr++)
 					{
-						Element* p = ( lyr == 0 ) ? pGrid : pGrid->GetNbr(NBR_X);
-						if ( bWire && p ) wireNodeId = std::max(wireNodeId, p->GetNodeId());
-						const int origId = ( p && p->ReadFlagBits(USERSET) ) ? p->GetNodeId() : BAD_NODEID;
-						comp.SetOrigId(lyr, pinIndex, origId);
+						Element* p = ( iLyr == compLyr ) ? pGrid : pGrid->GetNbr(NBR_X);
+						if ( bWire && p ) wireNodeId = std::max(wireNodeId, p->TrackElement::GetNodeId());
+						const int origId = ( p && p->ReadFlagBits(USERSET) ) ? p->TrackElement::GetNodeId() : BAD_NODEID;
+						comp.SetOrigId(iLyr, pinIndex, origId);
 					}
 				}
 
@@ -472,54 +506,57 @@ bool Board::PutDown(Component& comp)	// Tries to place the (floating) component
 				assert( !bWire || iCompNodeId == BAD_NODEID ); // Wire shouldn't have a NodeId yet
 				if ( !bWire )	// Write nodeId & flag
 				{
-					const bool bAllLyrs = pGrid->GetHasPin();	assert( bAllLyrs );
-					SetNodeId(pGrid, iCompNodeId, bAllLyrs);
-					WipeFlagBits(pGrid, AUTOSET|VEROSET, bAllLyrs);
-					MarkFlagBits(pGrid, USERSET, bAllLyrs);
+					const bool bAllLyrs = pGrid->GetHasPinTH();
+					Element* p = ( bSOIC ) ? Get(LYR_TOP, jRow, iCol) : pGrid;	assert(p);
+					SetNodeId(p, iCompNodeId, bAllLyrs);
+					WipeFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
+					MarkFlagBits(p, USERSET, bAllLyrs);
 				}
 			}
 		}
-	}
-	if ( bWire )	// Handle wires setting the wire ends on the board to same value
-	{
-		// pA and pB are the opposite ends of the wire
-		Element*	pA		= Get(lyr, rowTL, colTL);						assert( pA );
-		Element*	pB		= Get(lyr, rowTL+compRows-1, colTL+compCols-1);	assert( pB );
-		const int	iSlotA	= pA->GetSlotFromCompId(compId);
-		const int	iSlotB	= pB->GetSlotFromCompId(compId);
-		pA->SetW(iSlotA, pB);	// Link wire ends
-		pB->SetW(iSlotB, pA);	// Link wire ends
 
-		// Take logical OR of flags on wire ends, ignoring the RECTSET bit
-		const char iWireFlag = ( pA->GetFlag() | pB->GetFlag() ) & (USERSET|AUTOSET|VEROSET);
-
-		WIRELIST wireList;	// Helper for chains of wires
-
-		// Handle all connected wires.  (We just need the wirelist on one end, so use pA)
-		pA->GetWireList(wireList);	// Get list containing pA and its wired points
-
-		for (const auto& o : wireList)
+		if ( bWire )	// Handle wires setting the wire ends on the board to same value
 		{
-			Element* pW = const_cast<Element*> (o.first);
-			// Set the nodeId's on the wire components ...
-			for (int iSlot = 0; iSlot < 2; iSlot++)
+			// pA and pB are the opposite ends of the wire
+			Element*	pA		= Get(compLyr, rowTL, colTL);						assert( pA );
+			Element*	pB		= Get(compLyr, rowTL+compRows-1, colTL+compCols-1);	assert( pB );
+			const int	iSlotA	= pA->GetSlotFromCompId(compId);	assert(iSlotA != -1);
+			const int	iSlotB	= pB->GetSlotFromCompId(compId);	assert(iSlotB != -1);
+			pA->SetW(iSlotA, pB);	// Link wire ends
+			pB->SetW(iSlotB, pA);	// Link wire ends
+
+			// Take logical OR of flags on wire ends, ignoring the RECTSET bit
+			const char iWireFlag = ( pA->GetFlag() | pB->GetFlag() ) & (USERSET|AUTOSET|VEROSET);
+
+			WIRELIST wireList;	// Helper for chains of wires
+
+			// Handle all connected wires.  (We just need the wirelist on one end, so use pA)
+			pA->GetWireList(wireList);	// Get list containing pA and its wired points
+
+			for (const auto& o : wireList)
 			{
-				size_t	iPinIndex;
-				int		tmpCompId;
-				pW->GetSlotInfo(iSlot, iPinIndex, tmpCompId);
-				if ( iPinIndex == BAD_PININDEX ) continue;
-				Component& comp = m_compMgr.GetComponentById( tmpCompId );
-				assert( comp.GetType() == COMP::WIRE );
-				comp.SetNodeId(iPinIndex, wireNodeId);
+				Element* pW = const_cast<Element*> (o.first);
+				// Set the nodeId's on the wire components ...
+				for (int iSlot = 0; iSlot < 2; iSlot++)
+				{
+					size_t	iPinIndex;
+					int		tmpCompId;
+					pW->GetSlotInfo(iSlot, iPinIndex, tmpCompId);
+					if ( iPinIndex == BAD_PININDEX ) continue;
+					Component& comp = m_compMgr.GetComponentById( tmpCompId );
+					assert( comp.GetType() == COMP::WIRE );
+					comp.SetNodeId(iPinIndex, wireNodeId);
+				}
+				// ... and on the corresponding board points
+				const bool bAllLyrs(true);
+				SetNodeId(pW, wireNodeId, bAllLyrs);
+				MarkFlagBits(pW, iWireFlag, bAllLyrs);
 			}
-			// ... and on the corresponding board points
-			const bool bAllLyrs(true);
-			SetNodeId(pW, wireNodeId, bAllLyrs);
-			MarkFlagBits(pW, iWireFlag, bAllLyrs);
 		}
 	}
-	comp.SetIsPlaced(true);	
-	if ( comp.GetType() == COMP::MARK ) Get(lyr, rowTL, colTL)->SetIsMark(true);	// Set marker flag
+
+	comp.SetIsPlaced(true);
+	if ( comp.GetType() == COMP::MARK ) Get(compLyr, rowTL, colTL)->SetIsMark(true);	// Set marker flag
 
 	m_colorMgr.ReAssignColors();	// Forces colors to be worked out again
 	return true;
@@ -531,17 +568,18 @@ bool Board::TakeOff(Component& comp)
 
 	const bool	bWire		= comp.GetType() == COMP::WIRE;	// Wire's only get NodeIDs while placed
 	const bool	bTrax		= comp.GetType() == COMP::TRACKS;
+	const bool	bSOIC		= comp.GetIsSOIC();
 	const int&	compId		= comp.GetId();
 	const int&	compCols	= comp.GetCompCols();
 	const int&	compRows	= comp.GetCompRows();
-	const int&	lyr			= comp.GetLyr();
+	const int&	compLyr		= comp.GetLyr();	assert(bTrax || compLyr == 0);
 	const int&	rowTL		= comp.GetRow();
 	const int&	colTL		= comp.GetCol();
 
 	// If we have a wire, then pA and pB are the opposite ends of the wire.
 	// Find out which wire slots are used before we take off the wire.
-	Element*	pA			= ( bWire ) ? Get(lyr, rowTL, colTL) : nullptr;
-	Element*	pB			= ( bWire ) ? Get(lyr, rowTL+compRows-1, colTL+compCols-1) : nullptr;
+	Element*	pA			= ( bWire ) ? Get(compLyr, rowTL, colTL) : nullptr;
+	Element*	pB			= ( bWire ) ? Get(compLyr, rowTL+compRows-1, colTL+compCols-1) : nullptr;
 	assert( !bWire || (pA != nullptr && pB != nullptr) );
 	int			iOrigIdA[2]	= {BAD_NODEID, BAD_NODEID};	// 1 per layer
 	int			iOrigIdB[2]	= {BAD_NODEID, BAD_NODEID};	// 1 per layer
@@ -550,13 +588,13 @@ bool Board::TakeOff(Component& comp)
 	size_t	iPinIndex;
 	if ( pA )
 	{
-		iSlotA = pA->GetSlotFromCompId(compId);
+		iSlotA = pA->GetSlotFromCompId(compId);	assert(iSlotA != -1);
 		pA->GetSlotInfo(iSlotA, iPinIndex, tmpCompId);	assert( tmpCompId == compId );
 		for (int iLyr = 0; iLyr < 2; iLyr++) iOrigIdA[iLyr] = comp.GetOrigId(iLyr, iPinIndex);
 	}
 	if ( pB )
 	{
-		iSlotB = pB->GetSlotFromCompId(compId) ;
+		iSlotB = pB->GetSlotFromCompId(compId);	assert(iSlotB != -1);
 		pB->GetSlotInfo(iSlotB, iPinIndex, tmpCompId);	assert( tmpCompId == compId );
 		for (int iLyr = 0; iLyr < 2; iLyr++) iOrigIdB[iLyr] = comp.GetOrigId(iLyr, iPinIndex);
 	}
@@ -568,14 +606,14 @@ bool Board::TakeOff(Component& comp)
 		for (int i = 0; i < compCols; i++, iCol++)
 		{
 			const CompElement*	pComp = comp.GetCompElement(j, i);
-			Element*			pGrid = Get(lyr, jRow, iCol);
+			Element*			pGrid = Get(compLyr, jRow, iCol);
 			if ( bTrax )
 			{
 				const bool bAllLyrs(false);
 				if ( !pComp->ReadFlagBits(RECTSET) ) continue;		// Skip non-rect points
 				if ( pComp->GetNodeId() == BAD_NODEID ) continue;	// Skip blank areas of the trax comp
-				if ( !pGrid->ReadFlagBits(RECTSET) && ( !pGrid->GetHasPin() || pGrid->GetHasWire() ) )
-					SetNodeIdByUser(lyr, jRow, iCol, BAD_NODEID, false);	// false ==> don't paint pins
+				if ( !pGrid->ReadFlagBits(RECTSET) && ( !pGrid->GetLyrHasPin() || pGrid->GetHasWire() ) )
+					SetNodeIdByUser(compLyr, jRow, iCol, BAD_NODEID, false);	// false ==> don't paint pins
 				WipeFlagBits(pGrid, RECTSET, bAllLyrs);
 			}
 			else
@@ -583,16 +621,21 @@ bool Board::TakeOff(Component& comp)
 				assert( !pComp->GetIsHole() || pGrid->GetIsHole() );	// Component hole can only be taken off a grid hole
 
 				// Update surface and hole use
-				pGrid->SetSurface( pGrid->GetSurface() - pComp->GetSurface() );
-				pGrid->SetHoleUse( pGrid->GetHoleUse() - pComp->GetHoleUse() );
-				if ( pGrid->GetHoleUse() == HOLE_FREE )
+				pGrid->SetSurface(  pGrid->GetSurface()  - pComp->GetSurface() );
+				pGrid->SetHoleUse(  pGrid->GetHoleUse()  - pComp->GetHoleUse() );
+				pGrid->SetSoicChar( pGrid->GetSoicChar() - pComp->GetSoicChar() );
+
+				// Update compId and pinIndex use
+				if ( pGrid->GetSoicChar() == SOIC_FREE )	// Should not really need this case.  The else should suffice.
 				{
 					pGrid->SetSlotInfo(0, BAD_PININDEX, BAD_COMPID);	// Slot 0
 					pGrid->SetSlotInfo(1, BAD_PININDEX, BAD_COMPID);	// Slot 1
 				}
-				else if ( bWire && ( pGrid == pA || pGrid == pB ) )	// Taking off a wire-end
+				else
 				{
-					pGrid->SetSlotInfo( ( pGrid == pA ) ? iSlotA : iSlotB, BAD_PININDEX, BAD_COMPID);
+					const int iSlot = pGrid->GetSlotFromCompId(compId);
+					if ( iSlot != -1 )
+						pGrid->SetSlotInfo(iSlot, BAD_PININDEX, BAD_COMPID);
 				}
 
 				// Update IDs at pin locations of component
@@ -603,20 +646,32 @@ bool Board::TakeOff(Component& comp)
 										comp.GetOrigId(1, pinIndex) };	// ... on both layers
 				comp.SetOrigId(0, pinIndex, BAD_NODEID);				// ... before wiping
 				comp.SetOrigId(1, pinIndex, BAD_NODEID);				// ... them
-				assert( origId[0] == BAD_NODEID || origId[0] == comp.GetNodeId(pinIndex) );
-				assert( origId[1] == BAD_NODEID || origId[1] == comp.GetNodeId(pinIndex) );
 
 				// Wire-ends need special treatment, so just handle non-wire pins here
 				if ( !bWire )
 				{
-					for (int lyr = 0, lyrs = GetLyrs(); lyr < lyrs; lyr++)
+					if ( !pGrid->GetHasPin() )	// If we're removing the last pin, revert nodeId on each board layer.
+					{
+						assert( origId[0] == BAD_NODEID || origId[0] == comp.GetNodeId(pinIndex) || bSOIC );	// Base layer check does not apply to SOIC
+						assert( origId[1] == BAD_NODEID || origId[1] == comp.GetNodeId(pinIndex) );
+
+						for (int iLyr = 0, lyrs = std::min(GetLyrs(), 2); iLyr < lyrs; iLyr++)
+						{
+							if ( bSOIC && iLyr != LYR_TOP ) continue;
+							Element* p = ( iLyr == compLyr ) ? pGrid : pGrid->GetNbr(NBR_X);
+							if ( p == nullptr ) continue;
+							const bool bAllLyrs = false;
+							SetNodeId(p, origId[iLyr], bAllLyrs);	// Restore grid element to original nodeId
+							WipeFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
+							MarkFlagBits(p, USERSET, bAllLyrs);
+						}
+					}
+					else if ( !bSOIC )	// Special case.  We're removing a TH pin from a TH/SOIC hole-share.  Only revert nodeId on bottom layer.
 					{
-						Element* p = ( lyr == 0 ) ? pGrid : pGrid->GetNbr(NBR_X);
-						if ( p == nullptr ) continue;
 						const bool bAllLyrs = false;
-						SetNodeId(p, origId[lyr], bAllLyrs);	// Restore grid element to original nodeId
-						WipeFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
-						MarkFlagBits(p, USERSET, bAllLyrs);
+						SetNodeId(pGrid, origId[0], bAllLyrs);	// Restore grid element on bottom layer (under the SOIC) to original nodeId
+						WipeFlagBits(pGrid, AUTOSET|VEROSET, bAllLyrs);
+						MarkFlagBits(pGrid, USERSET, bAllLyrs);
 					}
 				}
 			}
@@ -680,11 +735,11 @@ bool Board::TakeOff(Component& comp)
 		if ( !pA->GetHasWire() )	// If we've taken off the last wire at the location
 		{
 			const bool bAllLyrs = false;
-			for (int lyr = 0, lyrs = GetLyrs(); lyr < lyrs; lyr++)
+			for (int iLyr = 0, lyrs = std::min(GetLyrs(), 2); iLyr < lyrs; iLyr++)
 			{
-				Element* p = ( lyr == 0 ) ? pA : pA->GetNbr(NBR_X);
+				Element* p = ( iLyr == compLyr ) ? pA : pA->GetNbr(NBR_X);
 				if ( p == nullptr ) continue;
-				SetNodeId(p, iOrigIdA[lyr], bAllLyrs);	// Restore grid element to original nodeId
+				SetNodeId(p, iOrigIdA[iLyr], bAllLyrs);	// Restore grid element to original nodeId
 				WipeFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
 				MarkFlagBits(p, USERSET, bAllLyrs);
 			}
@@ -692,18 +747,18 @@ bool Board::TakeOff(Component& comp)
 		if ( !pB->GetHasWire() )	// If we've taken off the last wire at the location
 		{
 			const bool bAllLyrs = false;
-			for (int lyr = 0, lyrs = GetLyrs(); lyr < lyrs; lyr++)
+			for (int iLyr = 0, lyrs = std::min(GetLyrs(), 2); iLyr < lyrs; iLyr++)
 			{
-				Element* p = ( lyr == 0 ) ? pB : pB->GetNbr(NBR_X);
+				Element* p = ( iLyr == compLyr ) ? pB : pB->GetNbr(NBR_X);
 				if ( p == nullptr ) continue;
-				SetNodeId(p, iOrigIdB[lyr], bAllLyrs);	// Restore grid element to original nodeId
+				SetNodeId(p, iOrigIdB[iLyr], bAllLyrs);	// Restore grid element to original nodeId
 				WipeFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
 				MarkFlagBits(p, USERSET, bAllLyrs);
 			}
 		}
 	}
 	comp.SetIsPlaced(false);
-	if ( comp.GetType() == COMP::MARK ) Get(lyr, rowTL, colTL)->SetIsMark(false);	// Clear marker flag
+	if ( comp.GetType() == COMP::MARK ) Get(compLyr, rowTL, colTL)->SetIsMark(false);	// Clear marker flag
 	return true;
 }
 
@@ -712,6 +767,11 @@ void Board::FloatAllComps()	// Float all components (i.e. take them off the boar
 	for (auto& mapObj : m_compMgr.m_mapIdToComp) TakeOff( mapObj.second );
 }
 
+void Board::FloatAllCompsSOIC()	// Float all SOIC components (i.e. take them off the board)
+{
+	for (auto& mapObj : m_compMgr.m_mapIdToComp) if ( mapObj.second.GetIsSOIC() ) TakeOff( mapObj.second );
+}
+
 void Board::PlaceFloaters()	// Try to place down all the floating components
 {
 	// Do trax component first.
@@ -798,7 +858,7 @@ void Board::DestroyUserComps()	// Destroy components in the user-group
 	m_compMgr.ClearTrax();	// Clear the trax object
 }
 
-void Board::MoveUserCompText(const int& deltaRow, const int& deltaCol)	// Move text label
+void Board::MoveUserCompText(int deltaRow, int deltaCol)	// Move text label
 {
 	if ( m_groupMgr.GetNumUserComps() != 1 ) return;
 
@@ -809,7 +869,7 @@ void Board::MoveUserCompText(const int& deltaRow, const int& deltaCol)	// Move t
 		comp.MoveLabelOffsets(deltaRow, deltaCol);
 }
 
-void Board::StretchUserComp(const bool& bGrow)	// Stretch the selected component length
+void Board::StretchUserComp(bool bGrow)	// Stretch the selected component length
 {
 	if ( m_groupMgr.GetNumUserComps() != 1 ) return;
 
@@ -838,7 +898,7 @@ void Board::StretchUserComp(const bool& bGrow)	// Stretch the selected component
 	PlaceFloaters();	// See if we can now place floating components down
 }
 
-void Board::StretchWidthUserComp(const bool& bGrow)	// Stretch the selected component width (just for DIPs)
+void Board::StretchWidthUserComp(bool bGrow)	// Stretch the selected component width (just for DIPs)
 {
 	if ( m_groupMgr.GetNumUserComps() != 1 ) return;
 
@@ -867,7 +927,7 @@ void Board::StretchWidthUserComp(const bool& bGrow)	// Stretch the selected comp
 	PlaceFloaters();	// See if we can now place floating components down
 }
 
-void Board::ChangeTypeUserComp(const COMP& eType)
+void Board::ChangeTypeUserComp(COMP eType)
 {
 	if ( m_groupMgr.GetNumUserComps() != 1 ) return;
 
@@ -918,7 +978,7 @@ void Board::CopyUserComps()	// Make a blank copy of the user-group components an
 	CopyComps(userCompIds);
 }
 
-bool Board::MoveUserComps(const int& deltaRow, const int& deltaCol)	// Move user-group components, and return true if the grid was panned
+bool Board::MoveUserComps(int deltaRow, int deltaCol)	// Move user-group components, and return true if the grid was panned
 {
 	if ( deltaRow == 0 && deltaCol == 0 ) return false;
 	if ( GetDisableMove() ) return false;
@@ -929,7 +989,7 @@ bool Board::MoveUserComps(const int& deltaRow, const int& deltaCol)	// Move user
 	return MoveComps(userCompIds, deltaRow, deltaCol);
 }
 
-void Board::RotateUserComps(const bool& bCW)	// Rotate the selected components
+void Board::RotateUserComps(bool bCW)	// Rotate the selected components
 {
 	std::list<int> userCompIds;
 	m_groupMgr.GetUserCompIds(userCompIds);
@@ -954,7 +1014,7 @@ void Board::CopyComps(const std::list<int>& compIds)	// Make a blank copy of the
 	}
 }
 
-bool Board::MoveTextBox(const int& deltaRow, const int& deltaCol)	// Move text box, and return true if the grid was panned
+bool Board::MoveTextBox(int deltaRow, int deltaCol)	// Move text box, and return true if the grid was panned
 {
 	bool bPanned(false);	// Set true if we pan the grid
 
@@ -979,7 +1039,7 @@ bool Board::MoveTextBox(const int& deltaRow, const int& deltaCol)	// Move text b
 	return bPanned;
 }
 
-bool Board::MoveComps(const std::list<int>& compIds, const int& deltaRow, const int& deltaCol)	// Move components and return true if the grid was panned
+bool Board::MoveComps(const std::list<int>& compIds, int deltaRow, int deltaCol)	// Move components and return true if the grid was panned
 {
 	assert( deltaRow != 0 || deltaCol != 0 );
 	assert( !GetDisableMove() );
@@ -1045,7 +1105,7 @@ bool Board::MoveComps(const std::list<int>& compIds, const int& deltaRow, const
 	return bPanned;
 }
 
-void Board::RotateComps(const std::list<int>& compIds, const bool& bCW)	// Rotate components
+void Board::RotateComps(const std::list<int>& compIds, bool bCW)	// Rotate components
 {
 	// Treat the components as a single large footprint with LT at (minRow, minCol)
 	Rect rect = GetFootprintBounds(compIds);
@@ -1164,10 +1224,11 @@ void Board::FixCorruption()
 	for (int i = 0, iSize = GetSize(); i < iSize && !bBadGrid; i++)
 	{
 		Element* p = GetAt(i);
-		const int& compId	= p->GetCompId();
-		const int& compId2	= p->GetCompId2();
-		bBadGrid = ( compId  != BAD_COMPID && badCompIds.find(compId)  != badCompIds.end() )
-				|| ( compId2 != BAD_COMPID && badCompIds.find(compId2) != badCompIds.end() );
+		for (int iSlot = 0; iSlot < 2 && !bBadGrid; iSlot++)
+		{
+			const int compId = p->GetSlotCompId(iSlot);
+			bBadGrid = ( compId != BAD_COMPID && badCompIds.find(compId) != badCompIds.end() );
+		}
 	}
 	if ( !bBadGrid ) return;
 
@@ -1179,26 +1240,7 @@ void Board::FixCorruption()
 
 	// Ensure there are no component related effects on the board elements
 	for (int i = 0, iSize = GetSize(); i < iSize; i++)
-	{
-		Element* p = GetAt(i);
-		const bool bOK = p->GetCompId()		== BAD_COMPID	&&
-						 p->GetCompId2()	== BAD_COMPID	&&
-						 p->GetPinIndex()	== BAD_PININDEX	&&
-						 p->GetPinIndex2()	== BAD_PININDEX &&
-						 p->GetSurface()	== SURFACE_FREE &&
-						 p->GetHoleUse()	== HOLE_FREE	&&
-						 p->GetIsMark()		== false;
-		if ( !bOK )
-		{
-			p->SetCompId(BAD_COMPID);
-			p->SetCompId2(BAD_COMPID);
-			p->SetPinIndex(BAD_PININDEX);
-			p->SetPinIndex2(BAD_PININDEX);
-			p->SetSurface(SURFACE_FREE);
-			p->SetHoleUse(HOLE_FREE);
-			p->SetIsMark(false);
-		}
-	}
+		GetAt(i)->FixCorruption();
 
 	PlaceFloaters();	// Unfloat components
 }
