@@ -17,6 +17,8 @@
 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+//#define ROUTE_ALGORITHM_TIMER
+
 #include "Board.h"
 
 // MH_LRTB = Manhatten "distance" for horizontally/vertically adjacent grid points.
@@ -38,30 +40,30 @@
 
 // Routing methods
 
-void Board::WipeAutoSetPoints(int nodeId)
+void Board::WipeAutoSetPoints(int nodeId, bool bHavePlacedWires)
 {
 	WIRELIST wireList;	// Helper for chains of wires
 
 	const bool bWipeAll = ( nodeId == BAD_NODEID );
 	for (int i = 0, iSize = GetSize(); i < iSize; i++)
 	{
-		Element* p = GetAt(i);
+		Element* const p = GetAt(i);
 		if ( !bWipeAll && p->GetNodeId() != nodeId ) continue;	// Skip points with wrong nodeId
-		const bool bAllLyrs = p->GetHasPin();
+		const bool bAllLyrs = p->GetHasPinTH();
 		bool bWipe = p->ReadFlagBits(AUTOSET) && !p->ReadFlagBits(USERSET);
-		if ( p->GetHasWire() )
+		if ( bHavePlacedWires && p->GetHasWire() )
 		{
 			p->GetWireList(wireList);	// Get list containing p and its wired points ...
 			for (const auto& o : wireList)	// ... and disable wipe if any of them are USERSET
 			{
-				const Element* pW = o.first;
+				const Element* const pW = o.first;
 				if ( pW == p ) continue;	// Skip p
 				bWipe &= ( !pW->ReadFlagBits(USERSET) );
 				if ( !bWipe ) break;
 			}
 			for (const auto& o : wireList)
 			{
-				Element* pW = const_cast<Element*> (o.first);
+				Element* const pW = const_cast<Element*> (o.first);
 				if ( bWipe ) SetNodeId(pW, BAD_NODEID, bAllLyrs);
 				WipeFlagBits(pW, AUTOSET, bAllLyrs);
 				MarkFlagBits(pW, USERSET, bAllLyrs);
@@ -76,30 +78,40 @@ void Board::WipeAutoSetPoints(int nodeId)
 	}
 }
 
-void Board::BuildTargetPins(const int& nodeId)
+void Board::BuildTargetPins(int nodeId)
 {
 	// Populate m_targetPins with all (non-wire) component pins with the specified NodeId.
 	// These are the things on the board that the routing algorithm will try and connect together.
 
 	assert( nodeId != BAD_NODEID );
 	m_targetPins.clear();
-	for (int i = 0, iSize = GetSize(); i < iSize; i++)
+
+	// Can limit target pins (and loop) to layer-0 if we don't have SOIC parts
+	const int iSize	= ( m_compMgr.GetNumSOIC() > 0 && GetLyrs() == 2 ) ? GetSize() : ( GetSize() / GetLyrs() );
+	for (int i = 0; i < iSize; i++)	
 	{
-		Element* p = GetAt(i);
-		if ( p->IsLayer0() && p->GetHasPin() && p->GetNodeId() == nodeId && !p->GetHasWire() )
-			m_targetPins.push_back(p);
+		Element* const p = GetAt(i);
+		if ( p->GetHasPin() && p->GetNodeId() == nodeId && !(m_bHavePlacedWires && p->GetHasWire()) )
+		{
+			if ( p->GetHasPinTH() && !p->GetIsBotLyr() ) continue;	// Only put TH pins on bottom layer or we'll double count them
+			m_targetPins.push_back(p); 
+		}
 	}
 }
 
 void Board::Route(bool bMinimal)
 {
-	m_bRouteMinimal	= bMinimal;
+#ifdef ROUTE_ALGORITHM_TIMER
+	const auto start = std::chrono::steady_clock::now();
+#endif
 
-//	const auto start = std::chrono::steady_clock::now();
+	SetHavePlacedWires();	// Set up m_bHavePlacedWires at very start of routing
+
+	m_bRouteMinimal = bMinimal;
 
 	// When routing is enabled,  this method will build tracks and update the cost in each NodeInfo.
 	// When routing is disabled, this method will update each NodeInfo cost without building new tracks.
-	if ( GetRoutingEnabled() ) WipeAutoSetPoints();
+	if ( GetRoutingEnabled() ) WipeAutoSetPoints(BAD_NODEID, m_bHavePlacedWires);
 
 	m_nodeInfoMgr.SortByLowestDifficulty(m_compMgr);
 
@@ -122,7 +134,7 @@ void Board::Route(bool bMinimal)
 
 		for (size_t i = 0; i < numNodes; i++)	// Loop all nodeIds used by components
 		{
-			NodeInfo* pI = m_nodeInfoMgr.GetAt(i);
+			NodeInfo* const pI = m_nodeInfoMgr.GetAt(i);
 			if ( pI->GetCost() == 0 ) continue;	// Skip if fully routed
 
 			const int& nodeIdI = pI->GetNodeId();
@@ -141,18 +153,18 @@ void Board::Route(bool bMinimal)
 
 				CopyTo(Ibest);
 
-				WipeAutoSetPoints(nodeIdI);	// Rip-up I
+				WipeAutoSetPoints(nodeIdI, m_bHavePlacedWires);	// Rip-up I
 
 				CopyTo(Iripped);
 
 				size_t j(i-1);	// Loop j through previously routed nodeIds
 				while( pI->GetCost() > 0 )
 				{
-					NodeInfo* pJ = m_nodeInfoMgr.GetAt(j);
+					NodeInfo* const pJ = m_nodeInfoMgr.GetAt(j);
 					const int& nodeIdJ = pJ->GetNodeId();
 					if ( pJ->GetCost() == 0 )	// Only consider J if it is fully routed
 					{
-						WipeAutoSetPoints(nodeIdJ);	// Rip-up J
+						WipeAutoSetPoints(nodeIdJ, m_bHavePlacedWires);	// Rip-up J
 
 						const unsigned int costI = Flood(nodeIdI);	// Route I ...
 						if ( costI < pI->GetCost() )				// ... and if I improved
@@ -182,9 +194,11 @@ void Board::Route(bool bMinimal)
 
 	RebuildAdjacencies();
 
-//	const auto elapsed = std::chrono::steady_clock::now() - start;
-//	const auto duration_ms	= std::chrono::duration_cast<std::chrono::milliseconds>(elapsed).count();
-//	std::cout << "Time : " << duration_ms << std::endl;
+#ifdef ROUTE_ALGORITHM_TIMER
+	const auto elapsed = std::chrono::steady_clock::now() - start;
+	const auto duration_ms	= std::chrono::duration_cast<std::chrono::milliseconds>(elapsed).count();
+	std::cout << "Time : " << duration_ms << std::endl;
+#endif
 }
 
 void Board::UpdateVias()	// Sets the via flag to true on all candidate vias
@@ -192,71 +206,74 @@ void Board::UpdateVias()	// Sets the via flag to true on all candidate vias
 	m_bRouteMinimal	= true;
 	m_bHasVias = false;	// Reset the flag indicating if the board has routed vias (as opposed to "wires-as-tracks" vias)
 
-	// If the ends of a candidate via can be connected through a pin on the board then it is not a via.
-	const bool bRoutingEnabled	= GetRoutingEnabled();	// Log routing state
-	const bool bViasEnabled		= GetViasEnabled();		// Log vias state
-	SetRoutingEnabled(false);	// Don't build tracks
-	SetViasEnabled(false);		// Disable routing through vias to perform test
-
-	std::vector<unsigned int> iMH;
-	std::vector<unsigned int> iRID;
-	if ( bViasEnabled )
+	if ( GetViasEnabled() )
 	{
+		// If the ends of a candidate via can be connected through a pin on the board then it is not a via.
+		const bool bRoutingEnabled	= GetRoutingEnabled();	// Log routing state
+		SetRoutingEnabled(false);	// Don't build tracks while performing test
+		SetViasEnabled(false);		// Disable routing through vias to perform test
+
 		// Store existing MH values and route IDs (used for showing connected areas) because Flood() will wipe them
-		iMH.resize( GetSize() );
-		iRID.resize( GetSize() );
+		std::vector<unsigned int> iMH;	iMH.resize( GetSize() );
+		std::vector<unsigned int> iRID;	iRID.resize( GetSize() );
 		for (int i = 0, iSize = GetSize(); i < iSize; i++)
 		{
-			iMH[i]	= GetAt(i)->GetMH();
-			iRID[i]	= GetAt(i)->GetRouteId();
+			Element* const p = GetAt(i);
+			iMH[i]	= p->GetMH();
+			iRID[i]	= p->GetRouteId();
 		}
-	}
 
-	for (int i = 0, iSize = ( GetLyrs() == 1 ) ? GetSize() : ( GetSize() / 2 ); i < iSize; i++)	// Loop layer 0 only
-	{
-		Element* p = GetAt(i);
-		Element* q = p->GetNbr(NBR_X);
-		bool bIsVia = false;
-		if ( bViasEnabled && q && !p->GetHasPin() && p->GetNodeId() == q->GetNodeId() && p->GetNodeId() != BAD_NODEID )	// If candidate via ...
+		SetHavePlacedWires();	// Set up m_bHavePlacedWires before calling Flood() in loop below
+
+		for (int i = 0, iSize = ( GetLyrs() == 1 ) ? GetSize() : ( GetSize() / 2 ); i < iSize; i++)	// Loop layer 0 only
 		{
-			m_targetPins.clear();
-			m_targetPins.push_back(p);
-			m_targetPins.push_back(q);
-			bIsVia = Flood() > 0;
-			m_bHasVias |= bIsVia;	// Update m_bHasVias
+			Element* const p = GetAt(i);
+			Element* const q = p->GetNbr(NBR_X);
+			bool bIsVia(false);
+			if ( q && !p->GetHasPinTH() && p->GetNodeId() == q->GetNodeId() && p->GetNodeId() != BAD_NODEID )	// If candidate via ...
+			{
+				m_targetPins.clear();
+				m_targetPins.push_back(p);
+				m_targetPins.push_back(q);
+				bIsVia = Flood() > 0;
+				m_bHasVias |= bIsVia;	// Update m_bHasVias
+			}
+			p->SetIsVia(bIsVia);
 		}
-		p->SetIsVia(bIsVia);
-	}
 
-	if ( bViasEnabled )
-	{
 		// Restore MH values and route IDs
 		for (int i = 0, iSize = GetSize(); i < iSize; i++)
 		{
-			GetAt(i)->SetMH( iMH[i] );
-			GetAt(i)->SetRouteId( iRID[i] );
+			Element* const p = GetAt(i);
+			p->SetMH( iMH[i] );
+			p->SetRouteId( iRID[i] );
 		}
+
+		SetViasEnabled(true);				// Restore vias state
+		SetRoutingEnabled(bRoutingEnabled);	// Restore routing state
+	}
+	else
+	{
+		// No test needed.  Just wipe the via flag on the base elements
+		for (int i = 0, iSize = ( GetLyrs() == 1 ) ? GetSize() : ( GetSize() / 2 ); i < iSize; i++)	// Loop layer 0 only
+			GetAt(i)->SetIsVia(false);
 	}
-
-	SetViasEnabled(bViasEnabled);		// Restore vias state
-	SetRoutingEnabled(bRoutingEnabled);	// Restore routing state
-
 	CalcMIN_SEPARATION();
 }
 
-unsigned int Board::Flood(const int& iFloodNodeId)
+unsigned int Board::Flood(int iFloodNodeId)
 {
 	BuildTargetPins(iFloodNodeId);	// Populate m_targetPins
 	return Flood();
 }
 
-unsigned int Board::Flood(const bool bSingleRoute)
+unsigned int Board::Flood(bool bSingleRoute)
 {
 	// Flood the board with MH values, starting from the m_targetPins.
 	// The return value is a cost that shows how unconnected the pins are.
 	// Zero cost means the pins are all inter-connected.
 
-	const size_t N = m_targetPins.size();	assert( bSingleRoute == ( N == 1) );
+	const size_t N = m_targetPins.size();	assert( !bSingleRoute || ( N == 1 ) );
 
 	if ( !bSingleRoute && N < 2 ) return 0;	// Return cost of zero
 
@@ -310,7 +327,6 @@ void Board::Flood_Helper(const bool bBuildTracks)
 		UpdateMH(p, iRouteID, iMH, iMaxMH);				// Add p to set of visited points
 	}
 
-	const bool			bHaveWires	 = m_compMgr.GetHavePlacedWires();
 	const bool			bMultiLayer	 = GetLyrs() > 1;
 	const bool			bViasEnabled = bMultiLayer && GetViasEnabled();
 	const int&			iFloodNodeId = m_targetPins[0]->GetNodeId();
@@ -318,15 +334,15 @@ void Board::Flood_Helper(const bool bBuildTracks)
 	const unsigned int	iMaxDeltaMH	 = ( bViasEnabled ) ? MH_LVIA : bDiagsOK ? MH_DIAG : MH_LRTB;	// The max MH increment in single-layer mode depends on if diagonals are allowed
 
 	// If any of the target pins are wires, we have to handle those first
-	for (size_t n = 0; n < N && bHaveWires; n++)
+	for (size_t n = 0; n < N && m_bHavePlacedWires; n++)
 	{
-		Element* p = m_targetPins[n];
+		Element* const p = m_targetPins[n];
 		if ( !p->GetHasWire() ) continue;
 		const int iRID = p->GetRouteId();
 		p->GetWireList(wireList);	// Get list of p and its wired points
 		for (const auto& o : wireList)
 		{
-			Element* pW = const_cast<Element*> (o.first);
+			Element* const pW = const_cast<Element*> (o.first);
 			if ( pW == p ) continue;	// Skip p
 			assert( p->GetNodeId() == pW->GetNodeId() );			// Sanity check
 			if ( pW->GetMH() != BAD_MH ) continue;					// Don't overwrite visited points (even if MH is improved)
@@ -356,7 +372,7 @@ void Board::Flood_Helper(const bool bBuildTracks)
 		const size_t jjSize = m_tmpVecSize;	// m_tmpVecSize gets modified in loop so take a copy
 		for (size_t jj = jjStart; jj < jjSize && !bDone; jj++)	// Loop through visited points
 		{
-			Element* pJ = m_tmpVec[jj];
+			Element* const pJ = m_tmpVec[jj];
 
 			if ( pJ->GetMaxMH() + iMaxDeltaMH < iMH )	// If pJ (and all previous points) are too far from the flood boundary
 			{
@@ -370,7 +386,7 @@ void Board::Flood_Helper(const bool bBuildTracks)
 				switch( iType )
 				{
 					case 0:	// Type 0 ==> Change layer at a pin
-						if ( pJ->GetMH() + MH_LPIN == iMH && pJ->GetHasPin() )
+						if ( pJ->GetMH() + MH_LPIN == iMH && pJ->GetHasPinTH() )
 							Flood_Grow(iFloodNodeId, pJ, NBR_X, bBuildTracks, iMH, iMaxMH, bDone);
 						break;
 					case 1:	// Type 1 ==> Move within layer
@@ -384,7 +400,7 @@ void Board::Flood_Helper(const bool bBuildTracks)
 						}
 						break;
 					case 2:	// Type 2 ==> Change layer at a via
-						if ( pJ->GetMH() + MH_LVIA == iMH && !pJ->GetHasPin() )
+						if ( pJ->GetMH() + MH_LVIA == iMH && !pJ->GetHasPinTH() )
 							Flood_Grow(iFloodNodeId, pJ, NBR_X, bBuildTracks, iMH, iMaxMH, bDone);
 						break;
 				}
@@ -393,7 +409,7 @@ void Board::Flood_Helper(const bool bBuildTracks)
 
 		// Periodically (every sufficiently large MH increase) examine which routes have grown.
 		// If all growing routes are connected to each other then we're done.
-		if ( N > 1 && !bDone && iMH >= iMHlastGrowthCheck + iMaxDeltaMH + (bHaveWires ? MH_WIRE : 0) )
+		if ( N > 1 && !bDone && iMH >= iMHlastGrowthCheck + iMaxDeltaMH + (m_bHavePlacedWires ? MH_WIRE : 0) )
 		{
 			bDone = true;
 			for (size_t i = 0; i < N && bDone; i++)
@@ -410,11 +426,11 @@ void Board::Flood_Helper(const bool bBuildTracks)
 	}
 }
 
-void Board::Flood_Grow(const int& iFloodNodeId, Element* pJ, const int& iNbr, const bool& bBuildTracks, unsigned int& iMH, unsigned int& iMaxMH, bool& bDone)
+void Board::Flood_Grow(int iFloodNodeId, Element* const pJ, int iNbr, bool bBuildTracks, unsigned int& iMH, unsigned int& iMaxMH, bool& bDone)
 {
 	WIRELIST wireList;	// Helper for chains of wires
 
-	Element* pK = pJ->GetNbr(iNbr);	assert( pK );
+	Element* const pK = pJ->GetNbr(iNbr);	assert( pK );
 	if ( pK == nullptr ) return;
 
 	const bool			bOK	= pJ->GetNodeId() == iFloodNodeId;	// true ==> pJ already painted with correct NodeId
@@ -422,7 +438,7 @@ void Board::Flood_Grow(const int& iFloodNodeId, Element* pJ, const int& iNbr, co
 	const unsigned int&	k	= pK->GetRouteId();
 
 	const bool bDirOK = ( bOK && pJ->GetUsed(iNbr) ) ||	// i.e. if already painted with correct nodeId
-						( bBuildTracks && pJ->HaveNoBlankPins(iNbr) && !pJ->IsBlocked(iNbr, iFloodNodeId) && !pJ->IsUselessWire(iNbr, iFloodNodeId) );
+						( bBuildTracks && pJ->HaveNoBlankPins(iNbr) && !pJ->IsBlocked(iNbr, iFloodNodeId) && !(m_bHavePlacedWires && pJ->IsUselessWire(iNbr, iFloodNodeId)) );
 	if ( !bDirOK ) return;
 
 	if ( pK->GetMH() == BAD_MH ) // Grow route with ID j (from pJ to pK)
@@ -432,13 +448,13 @@ void Board::Flood_Grow(const int& iFloodNodeId, Element* pJ, const int& iNbr, co
 		{
 			UpdateMH(pK, j, iMH, iMaxMH);	// Add pK to set of visited points
 
-			const bool bWire = pK->IsLayer0() && pK->GetHasWire();	// Constrain wire-routing to layer 0
+			const bool bWire = m_bHavePlacedWires && pK->GetIsBotLyr() && pK->GetHasWire();	// Constrain wire-routing to bottom layer
 			if ( bWire )
 			{
 				pK->GetWireList(wireList);	// Get list of pK and its wired points
 				for (const auto& o : wireList)	// Ideally want these in order of increasing MH
 				{
-					Element* pW = const_cast<Element*> (o.first);
+					Element* const pW = const_cast<Element*> (o.first);
 					if ( pW == pK ) continue;	// Skip pK
 					assert( pK->GetNodeId() == pW->GetNodeId() );			// Sanity check
 					if ( pW->GetMH() != BAD_MH ) continue;					// Don't overwrite visited points (even if MH is improved)
@@ -454,8 +470,14 @@ void Board::Flood_Grow(const int& iFloodNodeId, Element* pJ, const int& iNbr, co
 	// Routes with IDs j and k have met and don't have a connection yet ...
 	if ( bBuildTracks )	// If building tracks ...
 	{
-		Backtrace(pJ, iFloodNodeId);	// ... trace pJ back to its source, painting iFloodNodeId along the way
-		Backtrace(pK, iFloodNodeId);	// ... trace pK back to its source, painting iFloodNodeId along the way
+		Backtrace(pJ, iFloodNodeId);					// ... trace pJ back to its source, painting iFloodNodeId along the way
+		Element* const pKprev = Backtrace(pK, iFloodNodeId);	// ... trace pK back to its source, painting iFloodNodeId along the way
+		// pKprev is the first backtraced element from pK.
+		// If diagonal connections are allowed, and if pJ and pKprev are diagonal neighbours,
+		// then remove pK from the backtrace.  This will give a direct diagonal connection between pJ and pKprev.
+		const bool bDiagsOK = ( GetDiagsMode() != DIAGSMODE::OFF );
+		if ( bDiagsOK && pKprev && pKprev->IsDiagNbr(pJ) )
+			BacktraceErase(pK);
 	}
 
 	m_connectionMatrix.Connect(j, k);	// Make j-k connection and enforce transitivity
@@ -464,114 +486,163 @@ void Board::Flood_Grow(const int& iFloodNodeId, Element* pJ, const int& iNbr, co
 		bDone = m_connectionMatrix.GetCost() == 0;	// Zero cost ==> done
 }
 
-void Board::Backtrace(Element* pEnd, const int& nodeId)
+Element* Board::Backtrace(Element* const pEnd, int nodeId)
 {
 	// Backtrace route from pEnd to point with MH = 0
+	Element* pOut = nullptr;	// We will return the first traced element from pEnd (if there is one)
 
 	Element* p = pEnd;
-	if ( p->GetMH() == BAD_MH ) return;
-
-	WIRELIST wireList;	// Helper for chains of wires
+	if ( p->GetMH() == BAD_MH ) return pOut;
 
 	const bool bMultiLayer	= GetLyrs() > 1;
 	const bool bViasEnabled	= bMultiLayer && GetViasEnabled();
 	const bool bDiagsOK		= ( GetDiagsMode() != DIAGSMODE::OFF );
 
+	int iLastDirection(-1);	// To avoid ping-pong between layers
+
 	unsigned int MH = p->GetMH();
 	while ( true )	// Backtrace
 	{
-		assert( !p->GetIsHole() );
+		assert( !p->GetIsHole() && !(p->GetSoicProtected() && nodeId != BAD_NODEID) );
 
-		const bool bHasPin = p->GetHasPin();
+		Element* const pW0 = m_bHavePlacedWires ? p->GetW(0) : nullptr;
+		Element* const pW1 = m_bHavePlacedWires ? p->GetW(1) : nullptr;
+		const bool bWire		= (pW0 || pW1) && p->GetIsBotLyr();	// Constrain wire-routing to bottom layer
+		const bool bHasPinTH	= bWire || p->GetHasPinTH();	// Checks all layers
+		const bool bLyrHasPin	= p->GetLyrHasPin();			// true ==> have a pin on this layer, so returns false for bottom layer under an SOIC pin (with no TH)
+		const bool bAllLyrs		= bHasPinTH;
+		if ( !bLyrHasPin || bWire ) // For non-pins and wires
+			BacktracePaint(p, nodeId, bAllLyrs, bWire);	// Paint element p
 
-		Element* pW0 = p->GetW(0);
-		Element* pW1 = p->GetW(1);
-		const bool bWire = p->IsLayer0() && p->GetHasWire();	// Constrain wire-routing to layer 0
-		if ( !bHasPin || bWire ) // For non-pins and wires
-		{
-			if ( p->GetNodeId() == BAD_NODEID )	// Set NodeId if not set yet.
-			{
-				SetNodeId(p, nodeId, bHasPin);
-				WipeFlagBits(p, USERSET, bHasPin);
-				MarkFlagBits(p, AUTOSET, bHasPin);
-				if ( bWire )
-				{
-					p->GetWireList(wireList);	// Get list of p and its wired points
-					for (const auto& o : wireList)
-					{
-						Element* pW = const_cast<Element*> (o.first);
-						if ( pW == p ) continue;	// Skip p
-						SetNodeId(pW, nodeId, bHasPin);
-						WipeFlagBits(pW, USERSET, bHasPin);
-						MarkFlagBits(pW, AUTOSET, bHasPin);
-					}
-				}
-			}
-			else if ( p->ReadFlagBits(USERSET) )
-			{
-				assert( p->GetNodeId() == nodeId );
-				MarkFlagBits(p, AUTOSET, bHasPin);
-				if ( bWire )
-				{
-					p->GetWireList(wireList);	// Get list of p and its wired points
-					for (const auto& o : wireList)
-					{
-						Element* pW = const_cast<Element*> (o.first);
-						if ( pW == p ) continue;	// Skip p
-						MarkFlagBits(pW, AUTOSET, bHasPin);
-					}
-				}
-			}
-		}
-
-		if ( MH == 0 ) break;
+		if ( !pOut && p != pEnd ) pOut = p;
+		if ( MH == 0 ) return pOut;	// We're done backtracing once MH == 0
 
 		bool bOK(false);	// Will be set true after a successful backtrace (with modified p and MH)
 		// Now decide where to back trace to.
 
 		// Check wires first...
-		bOK = ( pW0 && pW0->GetMH() == MH - MH_WIRE );
-		if ( bOK ) { p = pW0; MH -= MH_WIRE; continue; }
-		bOK = ( pW1 && pW1->GetMH() == MH - MH_WIRE );
-		if ( bOK ) { p = pW1; MH -= MH_WIRE; continue; }
+		if ( m_bHavePlacedWires )
+		{
+			bOK = ( pW0 && pW0->GetMH() == MH - MH_WIRE );
+			if ( bOK ) { p = pW0; MH -= MH_WIRE; continue; }
+			bOK = ( pW1 && pW1->GetMH() == MH - MH_WIRE );
+			if ( bOK ) { p = pW1; MH -= MH_WIRE; continue; }
+		}
 
 		for (int iLoop = 0; iLoop < 2 && !bOK; iLoop++)	// First pass to give preference to nbrs that are not wire ends
 		{
-			const bool bHasPin = ( bMultiLayer ) ? p->GetHasPin() : false;	// Only care if p has a pin if multilayer routing
 			const int iTypeMin(bMultiLayer ? 0 : 1), iTypeMax(bViasEnabled ? 2 : 1);
 			for (int iType = iTypeMin; iType <= iTypeMax && !bOK; iType++)
 			{
 				switch( iType )
 				{
-					case 0:	// Type 0 ==> Change layer at a pin
-						if ( bHasPin ) bOK = BacktraceHelper(p, MH, nodeId, MH_LPIN, NBR_X, iLoop);
+					case 0:	// Type 0 ==> Change layer at a pin if multi-layer routing
+						if ( bMultiLayer && bHasPinTH && iLastDirection != NBR_X )
+						{
+							bOK = BacktraceHelper(p, MH, nodeId, MH_LPIN, NBR_X, iLoop);
+							if ( bOK ) iLastDirection = NBR_X;
+						}
 						break;
 					case 1:	// Type 1 ==> Move within layer
-						for (int iDiag = 0, iDiagMax = ( bDiagsOK ) ? 2 : 1; iDiag < iDiagMax && !bOK; iDiag++)	// First pass ==> Non-diagonal nbrs.  Second pass diagonal nbrs
+						for (int iDiag = 0, iDiagMax = ( bDiagsOK ) ? 2 : 1; iDiag < iDiagMax && !bOK; iDiag++)	// Loop for diagonal/non-diagonal directions
 						{
-							const unsigned int iDeltaMH = ( iDiag ) ? MH_DIAG : MH_LRTB;
+							const unsigned int iDeltaMH = ( iDiag ) ? MH_DIAG : MH_LRTB;	// First pass ==> Non-diagonal nbrs.  Second pass diagonal nbrs
 							for (int iNbr = iDiag; iNbr < 8 && !bOK; iNbr += 2)	// Even/Odd iNbr ==> Non-diagonal/Diagonal
+							{
 								bOK = BacktraceHelper(p, MH, nodeId, iDeltaMH, iNbr, iLoop);
+								if ( bOK ) iLastDirection = iNbr;
+							}
 						}
 						break;
 					case 2:	// Type 2 ==> Change layer at a via
-						if ( !bHasPin ) bOK = BacktraceHelper(p, MH, nodeId, MH_LVIA, NBR_X, iLoop);
+						if ( !bHasPinTH && iLastDirection != NBR_X )
+						{
+							bOK = BacktraceHelper(p, MH, nodeId, MH_LVIA, NBR_X, iLoop);
+							if ( bOK ) iLastDirection = NBR_X;
+						}
 						break;
 				}
 			}
 		}
-		if ( bOK ) continue;
+		if ( !bOK ) break;
+	}
+	assert(0);	// Oh dear. Something went badly wrong !!!
+	return pOut;
+}
 
-		assert(0);	// Oh dear. Something went badly wrong !!!
-		break;
+void Board::BacktracePaint(Element* const p, int nodeId, bool bAllLyrs, bool bWire)
+{
+	const bool bPaintNodeId = ( p->GetNodeId() == BAD_NODEID );	// Set NodeId if not set yet.
+	if ( !bPaintNodeId && !p->ReadFlagBits(USERSET) ) return;
+
+	assert( bPaintNodeId || p->GetNodeId() == nodeId );	// If USERSET then nodeId must be OK
+
+	if ( bPaintNodeId )
+	{
+		SetNodeId(p, nodeId, bAllLyrs);
+		WipeFlagBits(p, USERSET, bAllLyrs);
+	}
+	MarkFlagBits(p, AUTOSET, bAllLyrs);
+
+	if ( bWire )
+	{
+		WIRELIST wireList;			// Helper for chains of wires
+		p->GetWireList(wireList);	// Get list of p and its wired points
+		for (const auto& o : wireList)
+		{
+			Element* const pW = const_cast<Element*> (o.first);
+			if ( pW == p ) continue;	// Skip p
+			if ( bPaintNodeId )
+			{
+				SetNodeId(pW, nodeId, bAllLyrs);
+				WipeFlagBits(pW, USERSET, bAllLyrs);
+			}
+			MarkFlagBits(pW, AUTOSET, bAllLyrs);
+		}
 	}
 }
 
-bool Board::BacktraceHelper(Element*& p, unsigned int& MH, const int& nodeId, const unsigned int& iDeltaMH, const int& iNbr, const int& iLoop)
+void Board::BacktraceErase(Element* const p)
 {
-	Element* pNbr = p->GetNbr(iNbr);
+	assert( !p->GetIsHole() && !p->GetSoicProtected() );
+	assert( p->GetNodeId() != BAD_NODEID );
+
+	Element* const pW0 = m_bHavePlacedWires ? p->GetW(0) : nullptr;
+	Element* const pW1 = m_bHavePlacedWires ? p->GetW(1) : nullptr;
+	const bool bWire		= (pW0 || pW1) && p->GetIsBotLyr();	// Constrain wire-routing to bottom layer
+	const bool bAllLyrs		= bWire || p->GetHasPinTH();
+	const bool bWipeNodeId	= !p->ReadFlagBits(USERSET);
+
+	if ( bWipeNodeId )
+	{
+		SetNodeId(p, BAD_NODEID, bAllLyrs);
+		WipeFlagBits(p, USERSET, bAllLyrs);
+	}
+	WipeFlagBits(p, AUTOSET, bAllLyrs);
+
+	if ( bWire )
+	{
+		WIRELIST wireList;			// Helper for chains of wires
+		p->GetWireList(wireList);	// Get list of p and its wired points
+		for (const auto& o : wireList)
+		{
+			Element* const pW = const_cast<Element*> (o.first);
+			if ( pW == p ) continue;	// Skip p
+			if ( bWipeNodeId )
+			{
+				SetNodeId(pW, BAD_NODEID, bAllLyrs);
+				WipeFlagBits(pW, USERSET, bAllLyrs);
+			}
+			WipeFlagBits(pW, AUTOSET, bAllLyrs);
+		}
+	}
+}
+
+bool Board::BacktraceHelper(Element*& p, unsigned int& MH, int nodeId, unsigned int iDeltaMH, int iNbr, int iLoop)
+{
+	Element* const pNbr = p->GetNbr(iNbr);
 	if ( pNbr->GetRouteId() != p->GetRouteId() ) return false;	// Skip if nbr has wrong routeId
-	const bool bWire = pNbr->IsLayer0() && pNbr->GetHasWire();	// Constrain wire-routing to layer 0
+	const bool bWire = m_bHavePlacedWires && pNbr->GetIsBotLyr() && pNbr->GetHasWire();	// Constrain wire-routing to bottom layer
 	if ( iLoop == 0 &&  bWire ) return false;					// Skip if nbr is a wire
 	if ( iLoop == 1 && !bWire ) return false;					// Skip if nbr is a non-wire
 	if ( p->IsBlocked(iNbr, nodeId) ) return false;				// Skip if blocked
@@ -580,7 +651,7 @@ bool Board::BacktraceHelper(Element*& p, unsigned int& MH, const int& nodeId, co
 	return true;	// We backtraced OK, and have modified p and MH
 }
 
-void Board::Manhatten(Element* p, const bool bSingleRoute)
+void Board::Manhatten(Element* p, bool bSingleRoute)
 {
 	const int iTraceNodeId = p->GetNodeId();	// The NodeID to trace
 	if ( iTraceNodeId == BAD_NODEID ) return;	// Don't trace invalid NodeID
@@ -588,23 +659,26 @@ void Board::Manhatten(Element* p, const bool bSingleRoute)
 	const bool bRouteMinimal	= m_bRouteMinimal;		// Log m_bRouteMinimal state
 	const bool bRoutingEnabled	= GetRoutingEnabled();	// Log routing state
 
+	SetHavePlacedWires();	// Set up m_bHavePlacedWires so we can skip calls to GetHasWire() here, and in Flood()
+
 	m_bRouteMinimal	= true;
 	SetRoutingEnabled(false);	// Don't build tracks
 
 	m_targetPins.clear();
 
-	if ( p->GetHasPin() )	// p could be a wire end
-		m_targetPins.push_back( p->IsLayer0() ? p : p->GetNbr(NBR_X) );	// Always want target pins on layer 0
-	else
-		m_targetPins.push_back(p);
+	Element* pFirst = ( p->GetHasPinTH() && !p->GetIsBotLyr() ) ? p->GetNbr(NBR_X) : p;	// Use bottom layer for TH pins
+	if ( pFirst )
+		m_targetPins.push_back(pFirst);
 
-	// If we haven't specified bSingleRoute, then build additional routes from all true component pins
-	for (int i = 0, iSize = ( GetLyrs() == 1 ) ? GetSize() : ( GetSize() / 2 ); i < iSize && !bSingleRoute; i++)	// Use layer 0 only for pins
+	// If we haven't specified bSingleRoute, then build additional routes from all true component pins (not wires)
+	for (int i = 0, iSize = ( GetLyrs() == 1 ) ? GetSize() : ( GetSize() / 2 ); i < iSize && !bSingleRoute; i++)	// Loop base layer only
 	{
-		Element* q = GetAt(i);
-		if ( q == p ) continue;	// Skip self
-		if ( q && q->GetHasPin() && !q->GetHasWire() && q->GetNodeId() == iTraceNodeId )	// Skip wires
-			m_targetPins.push_back(q);
+		Element* const q = GetAt(i);	// q is on layer 0
+		const bool bNonWirePin = q && q->GetHasPin() && !(m_bHavePlacedWires && q->GetHasWire());	// Skip wires
+		if ( !bNonWirePin ) continue;
+		Element* const r = q->GetHasPinTH() ? q : q->GetNbr(NBR_X);	assert(r);	// r is on layer 0 for TH pins, and layer 1 for SOIC pads
+		if ( r && r->GetNodeId() == iTraceNodeId && r != pFirst )	// Skip first
+			m_targetPins.push_back(r);
 	}
 
 	Flood(bSingleRoute);
@@ -615,8 +689,8 @@ void Board::Manhatten(Element* p, const bool bSingleRoute)
 	// Update m_iConnPin and m_iConnRID.
 	for (int i = 0, iSize = GetSize(); i < iSize && m_iConnPin == -1; i++)
 	{
-		Element* q = GetAt(i);
-		if ( q->GetMH() != BAD_MH && q->GetHasPin() && !q->GetHasWire() )
+		Element* const q = GetAt(i);
+		if ( q->GetMH() != BAD_MH && q->GetHasPin() && !(m_bHavePlacedWires && q->GetHasWire()) )
 		{
 			m_iConnPin = i;
 			m_iConnRID = q->GetRouteId();
@@ -643,7 +717,7 @@ void Board::CheckAllComplete()
 	Route(true);
 	for (size_t n = 0, nSize = m_nodeInfoMgr.GetSize(); n < nSize; n++)
 	{
-		NodeInfo* pNodeInfo = m_nodeInfoMgr.GetAt(n);
+		NodeInfo* const pNodeInfo = m_nodeInfoMgr.GetAt(n);
 		pNodeInfo->SetComplete( pNodeInfo->GetCost() == 0 );
 	}
 	m_nodeInfoMgr.SortByLowestDifficulty(m_compMgr);
@@ -669,23 +743,24 @@ void Board::PasteTracks(bool bTidy)
 	const int	rows		= bRestrict ? trax.GetCompRows() : GetRows();
 	const int	cols		= bRestrict ? trax.GetCompCols() : GetCols();
 
-	for (int k = bRestrict ? trax.GetLyr() : 0;			k < lyrs; k++)
+	for (int k = bRestrict ? trax.GetLyr() : 0;				k < lyrs; k++)
 	for (int j = 0, jRow = bRestrict ? trax.GetRow() : 0;	j < rows; j++, jRow++)
 	for (int i = 0, iCol = bRestrict ? trax.GetCol() : 0;	i < cols; i++, iCol++)
 	{
 		if ( bRestrict && !trax.GetCompElement(j,i)->ReadFlagBits(RECTSET) ) continue;	// Skip points outside grey area
 
-		Element*	p		= Get(k, jRow, iCol);
-		const bool	bHasPin	= p->GetHasPin();
+		Element* const	p		= Get(k, jRow, iCol);
+		const bool		bHasPin	= p->GetLyrHasPin();
 
 		// Tidy clears all non-pins and wires that are USER_SET ...
 		if ( bTidy && ( !bHasPin || p->GetHasWire() ) && p->ReadFlagBits(USERSET) && !p->ReadFlagBits(AUTOSET|VEROSET) )
 		{
-			SetNodeId(p, BAD_NODEID, bHasPin);
+			assert( p->GetHasWire() == p->GetHasPinTH() );	// Either we have a wire (and therefore a TH), or we have no TH part
+			SetNodeId(p, BAD_NODEID, p->GetHasPinTH());	// Last argument is "bAllLyrs" so use GetHasPinTH()
 			for (int iSlot = 0; iSlot < 2; iSlot++)
 			{
-				Element* pW = p->GetW(iSlot);
-				if ( pW ) SetNodeId(pW, BAD_NODEID, bHasPin);
+				Element* const pW = p->GetW(iSlot);
+				if ( pW ) SetNodeId(pW, BAD_NODEID, pW->GetHasPinTH());	// Last argument is "bAllLyrs" so use GetHasPinTH()
 			}
 		}
 
@@ -698,7 +773,7 @@ void Board::PasteTracks(bool bTidy)
 		const int& nodeId = p->GetNodeId();
 		for (int iSlot = 0; iSlot < 2; iSlot++)
 		{
-			Element* pW = p->GetW(iSlot);
+			Element* const pW = p->GetW(iSlot);
 			if ( pW == nullptr ) continue;
 
 			p->GetSlotInfo(iSlot, iPinIndex, tmpCompId);
@@ -731,16 +806,15 @@ void Board::WipeTracks()
 	const int	rows		= bRestrict ? trax.GetCompRows() : GetRows();
 	const int	cols		= bRestrict ? trax.GetCompCols() : GetCols();
 
-	for (int k = bRestrict ? trax.GetLyr() : 0;			k < lyrs; k++)
+	for (int k = bRestrict ? trax.GetLyr() : 0;				k < lyrs; k++)
 	for (int j = 0, jRow = bRestrict ? trax.GetRow() : 0;	j < rows; j++, jRow++)
 	for (int i = 0, iCol = bRestrict ? trax.GetCol() : 0;	i < cols; i++, iCol++)
 	{
 		if ( bRestrict && !trax.GetCompElement(j,i)->ReadFlagBits(RECTSET) ) continue;	// Skip points outside grey area
-		Element* p = Get(k, jRow, iCol);
-		assert( !p->GetHasPin() && !p->GetIsHole() && !p->GetHasComp() );	// Sanity check
-
+		Element* const p = Get(k, jRow, iCol);
+		assert( !p->GetHasPin() && !p->GetIsHole() && !p->GetHasComp() );	// Sanity check.  Components are floated, so board should have no pins, holes, or components
+		p->FixCorruption();	// As all components are floated, use this as an opportunity to fix any board corruption that may have occurred
 		SetNodeId(p, BAD_NODEID, !bRestrict);
-		p->SetSurface(SURFACE_FREE);
 		WipeFlagBits(p, bRestrict ? (AUTOSET|VEROSET|RECTSET) : (AUTOSET|VEROSET), !bRestrict);
 		MarkFlagBits(p, USERSET, !bRestrict);
 	}
