@@ -26,9 +26,10 @@
 #define MYNUMCOLORS 		12
 #define MY_GREY				(MYNUMCOLORS)
 #define MY_BLACK			(MYNUMCOLORS+1)
-#define NUM_PIXMAP_COLORS	(MYNUMCOLORS+2)
-#define MY_LYR_BOT			(MYNUMCOLORS+3)
-#define MY_LYR_TOP			(MYNUMCOLORS+4)
+#define MY_WHITE			(MYNUMCOLORS+2)
+#define NUM_PIXMAP_COLORS	(MYNUMCOLORS+3)
+#define MY_LYR_BOT			(MYNUMCOLORS+4)
+#define MY_LYR_TOP			(MYNUMCOLORS+5)
 
 // Manager class to handle assignment of colors to nodeIds
 
@@ -70,15 +71,15 @@ public:
 	void ReAssignColors()			{ m_bReAssign = true; }
 	void SetSaturation(int i)		{ m_iSaturation = i; }
 	void SetFillSaturation(int i)	{ m_iFillSaturation = i; }
-	void SetNodeColor(const int& nodeId, const QColor& color)
+	void SetNodeColor(int nodeId, const QColor& color)
 	{
 		if ( nodeId != BAD_NODEID ) m_mapNodeIdToCustomColor[nodeId] = MyRGB(color);
 	}
-	bool GetIsFixed(const int& nodeId) const
+	bool GetIsFixed(int nodeId) const
 	{
 		return m_mapNodeIdToCustomColor.find(nodeId) != m_mapNodeIdToCustomColor.end();
 	}
-	void Fix(const int& nodeId)
+	void Fix(int nodeId)
 	{
 		if ( nodeId == BAD_NODEID || GetIsFixed(nodeId) ) return;	// Already fixed
 		const int colorId = m_mapNodeIdToColorId[nodeId];
@@ -86,7 +87,7 @@ public:
 		if ( bOK )
 			m_mapNodeIdToCustomColor[nodeId] = GetPixmapRGB(colorId, false);
 	}
-	void Unfix(const int& nodeId)
+	void Unfix(int nodeId)
 	{
 		auto iter = m_mapNodeIdToCustomColor.find(nodeId);
 		if ( iter != m_mapNodeIdToCustomColor.end() )
@@ -158,13 +159,13 @@ public:
 			cnList[iUnusedColor].push_back(iNodeId);		// Add nodeId to list of iUnusedColor
 		}
 	}
-	int GetColorId(const int& nodeId) const
+	int GetColorId(int nodeId) const
 	{
 		if ( nodeId == BAD_NODEID ) return BAD_COLORID;
 		const auto iter = m_mapNodeIdToColorId.find(nodeId);
 		return ( iter != m_mapNodeIdToColorId.end() ) ? iter->second : BAD_COLORID;
 	}
-	QColor GetColorFromNodeId(const int& nodeId, bool bUseSaturation = true) const
+	QColor GetColorFromNodeId(int nodeId, bool bUseSaturation = true) const
 	{
 		// First check if the nodeId is in the custom list
 		const auto iter = m_mapNodeIdToCustomColor.find(nodeId);
@@ -179,11 +180,12 @@ public:
 		// Use auto-calculated colors
 		return GetPixmapColor(GetColorId(nodeId), bUseSaturation);
 	}
-	MyRGB GetPixmapRGB(const int& colorId, bool bUseSaturation = true) const
+	MyRGB GetPixmapRGB(int colorId, bool bUseSaturation = true) const
 	{
 		int R(0), G(0), B(0);
 		if		( colorId == BAD_COLORID )	{ R = G = B = 255; }
 		else if ( colorId == MY_GREY )		{ R = G = B = 96; }
+		else if ( colorId == MY_WHITE )		{ R = G = B = 255; }
 		else if ( colorId == MY_LYR_BOT )	{ G = 128; }
 		else if ( colorId == MY_LYR_TOP )	{ G = 96; B = 192; }
 		else if	( colorId >= 0 && colorId < MYNUMCOLORS )
@@ -200,7 +202,7 @@ public:
 		rgb.SetRGB(R, G, B);
 		return rgb;
 	}
-	QColor GetPixmapColor(const int& colorId, bool bUseSaturation = true) const
+	QColor GetPixmapColor(int colorId, bool bUseSaturation = true) const
 	{
 		return GetPixmapRGB(colorId, bUseSaturation).GetQColor();
 	}
