@@ -19,6 +19,7 @@
 
 #include "CompDefiner.h"
 #include "Component.h"
+#include "TemplateManager.h"
 
 void CompDefiner::Populate(const Component& o)
 {
@@ -35,6 +36,9 @@ void CompDefiner::Populate(const Component& o)
 	SetTypeStr( o.GetFullTypeStr() );
 	SetImportStr( o.GetFullImportStr() );
 
+	SetLabelOffsetRow( o.GetLabelOffsetRow() );
+	SetLabelOffsetCol( o.GetLabelOffsetCol() );
+
 	// Copy footprint to PinInfo map
 	m_grid.Allocate(o.GetLyrs(), o.GetRows(), o.GetCols());
 	for (int i = 0, iSize = o.GetSize(); i < iSize; i++)
@@ -75,10 +79,8 @@ void CompDefiner::Populate(const Component& o)
 	}
 }
 
-void CompDefiner::Build(Component& comp) const
+void CompDefiner::Build(const TemplateManager& templateMgr, Component& comp) const
 {
-	assert( GetIsValid() );
-
 	// Build component from definition
 	comp.SetPinFlags( GetPinFlags() );
 	comp.SetPadWidth( GetPadWidth() );
@@ -88,8 +90,15 @@ void CompDefiner::Build(Component& comp) const
 	comp.SetPrefixStr( GetPrefixStr() );
 	comp.SetTypeStr( GetTypeStr() );
 	comp.SetImportStr( GetImportStr() );
+	comp.SetLabelOffsetRow( GetLabelOffsetRow() );
+	comp.SetLabelOffsetCol( GetLabelOffsetCol() );
 
 	comp.SetType(COMP::CUSTOM);
+
+	const COMP eType = CompTypes::GetTypeFromImportStr( templateMgr.GetImportStrCut(m_importStr) );
+	if ( eType != COMP::INVALID && templateMgr.CheckAllowOverWrite(comp) )	// If import string is reserved (e.g. TO92, DIP) and a user-template has that string ...
+		comp.SetType(eType);												// ... change type from COMP::CUSTOM to eType
+
 	assert( m_grid.GetLyrs() == 1 );
 	comp.Allocate(m_grid.GetLyrs(), m_grid.GetRows(), m_grid.GetCols());
 	for (int i = 0, iSize = m_grid.GetSize(); i < iSize; i++)
@@ -107,9 +116,15 @@ void CompDefiner::Build(Component& comp) const
 	assert( comp.GetNumShapes() == 0 );
 	for (const auto& mapObj : m_mapShapes)
 		comp.AddOne(mapObj.second);
+
+#ifdef _TEST_SOIC
+	if ( comp.GetIsSOIC() )
+		return comp.SetupOccupanciesSOIC();	// Setup hole use, and SOIC info
+#endif
+	return comp.SetupOccupanciesTH();		// Setup hole use, and SOIC info
 }
 
-void CompDefiner::MoveCurrentShape(const double& dDown, const double& dRight)
+void CompDefiner::MoveCurrentShape(double dDown, double dRight)
 {
 	Shape& s = GetCurrentShape();
 	double dCentreCol(0), dCentreRow(0);
@@ -153,7 +168,7 @@ int CompDefiner::GetNewShapeId() const
 	}
 	return ( shapeId == INT_MAX ) ? BAD_ID : shapeId;
 }
-bool CompDefiner::SetWidth(const int& i)
+bool CompDefiner::SetWidth(int i)
 {
 	assert( i > 0 );
 	const bool bChanged = m_grid.GetCols() != i;
@@ -169,7 +184,7 @@ bool CompDefiner::SetWidth(const int& i)
 	}
 	return bChanged;
 }
-bool CompDefiner::SetHeight(const int& i)
+bool CompDefiner::SetHeight(int i)
 {
 	assert( i > 0 );
 	const bool bChanged = m_grid.GetRows() != i;
@@ -186,7 +201,7 @@ bool CompDefiner::SetHeight(const int& i)
 	return bChanged;
 }
 
-int CompDefiner::GetPinId(const int& row, const int& col) const	// Pick the most relevant pin at the location
+int CompDefiner::GetPinId(int row, int col) const	// Pick the most relevant pin at the location
 {
 	const int iRow = row - GetGridRowMin();
 	const int iCol = col - GetGridColMin();
@@ -195,7 +210,7 @@ int CompDefiner::GetPinId(const int& row, const int& col) const	// Pick the most
 	return ( bOK ) ? iCol + iRow * m_grid.GetCols() : BAD_ID;
 }
 
-int CompDefiner::GetShapeId(const double& dRowIn, const double& dColIn) const	// Pick the most relevant shape at the location
+int CompDefiner::GetShapeId(double dRowIn, double dColIn) const	// Pick the most relevant shape at the location
 {
 	double dCentreRow(0), dCentreCol(0);
 	GetGridCentre(dCentreRow, dCentreCol);	// Footprint centre w.r.t. screen
@@ -213,7 +228,7 @@ int CompDefiner::GetShapeId(const double& dRowIn, const double& dColIn) const	//
 		const double DY		= s.GetDY();
 		const double CX		= s.GetCX();
 		const double CY		= s.GetCY();
-		const double dA3	= s.GetA3() * M_PI / 180.0;	// Convert to radians
+		const double dA3	= s.GetA3() * RADIANS_PER_DEGREE;	// Convert to radians
 		const double dCos	= cos(dA3);
 		const double dSin	= sin(dA3);
 		const double dX		= dCol - CX;
@@ -235,7 +250,7 @@ int CompDefiner::GetShapeId(const double& dRowIn, const double& dColIn) const	//
 			case SHAPE::ARC:
 			case SHAPE::CHORD:
 			{
-				dArea	= M_PI * 0.25*DX*DY;	// Area of the ellipse
+				dArea	= DBL_PI_4*DX*DY;	// Area of the ellipse
 				double epsilon(dArea < 0.1 ? 0.1 : 0);
 				bOK		= rx*DY*rx*DY + ry*DX*ry*DX <= 0.25*DX*DX*DY*DY + epsilon;
 				break;
@@ -274,21 +289,28 @@ int CompDefiner::GetShapeId(const double& dRowIn, const double& dColIn) const	//
 	return iBestId;
 }
 
-bool CompDefiner::GetIsValid() const
+bool CompDefiner::GetIsValid(const TemplateManager& templateMgr) const
 {
 	if ( StringHelper::IsEmptyStr(m_typeStr) ) return false;
 	if ( StringHelper::IsEmptyStr(m_valueStr) ) return false;
 	if ( StringHelper::HasSpaces(m_importStr) ) return false;	// Import string must not have spaces
-	if ( CompTypes::GetTypeFromImportStr(m_importStr) != COMP::INVALID ) return false;	// Reserved string
-	// Following is copied from Board::Import() method.
-	// List of package identifiers for footprints with variable numbers of pins/lengths.
-	const int NUM_VARIABLE_PIN_PARTS = 13;
-	const std::string strVar[NUM_VARIABLE_PIN_PARTS] = {"SIP", "DIP", "PADS", "SWITCH_ST_DIP", "SWITCH_ST", "SWITCH_DT", "STRIP_100MIL", "BLOCK_100MIL", "BLOCK_200MIL", "RESISTOR", "DIODE", "CAP_CERAMIC", "CAP_FILM"};
-	for (int i = 0; i < NUM_VARIABLE_PIN_PARTS; i++)
+
+	Component comp;
+	Build(templateMgr, comp);
+
+	if ( comp.GetType() == COMP::CUSTOM )
 	{
-		const std::string&	strTmp	= strVar[i];	// e.g. "SIP", "DIP, etc
-		const auto			L		= strTmp.length();
-		if ( m_importStr.length() >= L && m_importStr.substr(0, L) == strTmp ) return false;
+		if ( CompTypes::GetTypeFromImportStr( templateMgr.GetImportStrCut(m_importStr) ) != COMP::INVALID ) return false;	// Reserved string (e.g. TO92)
+		// Following is copied from Board::Import() method.
+		// List of package identifiers for footprints with variable numbers of pins/lengths.
+		const int NUM_VARIABLE_PIN_PARTS = 13;
+		const std::string strVar[NUM_VARIABLE_PIN_PARTS] = {"SIP", "DIP", "PADS", "SWITCH_ST_DIP", "SWITCH_ST", "SWITCH_DT", "STRIP_100MIL", "BLOCK_100MIL", "BLOCK_200MIL", "RESISTOR", "DIODE", "CAP_CERAMIC", "CAP_FILM"};
+		for (int i = 0; i < NUM_VARIABLE_PIN_PARTS; i++)
+		{
+			const std::string&	strTmp	= strVar[i];	// e.g. "SIP", "DIP, etc
+			const auto			L		= strTmp.length();
+			if ( m_importStr.length() >= L && m_importStr.substr(0, L) == strTmp ) return false;
+		}
 	}
 	if ( m_grid.GetSize() == 0 ) return false;	// Should not be possible
 	bool bOK(false);
