@@ -27,16 +27,16 @@ class Component;
 // Class used to build a description of a custom component
 
 // The following flags are used to form a bitfield that determine how pins may be drawn
-static const uchar	PIN_RECT	= 1;	// Draw pin as a rectangle instead of circle (e.g. for switches/relays)
-static const uchar	PIN_LABELS	= 2;	// Allow pin labels to be drawn
-static const uchar	PIN_CUSTOM	= 4;	// Allow over-ride of pad and hole size
+Q_DECL_CONSTEXPR static const uchar	PIN_RECT	= 1;	// Draw pin as a rectangle instead of circle (e.g. for switches/relays)
+Q_DECL_CONSTEXPR static const uchar	PIN_LABELS	= 2;	// Allow pin labels to be drawn
+Q_DECL_CONSTEXPR static const uchar	PIN_CUSTOM	= 4;	// Allow over-ride of pad and hole size
 
-static const int	BAD_ID	= -1;
+Q_DECL_CONSTEXPR static const int	BAD_ID		= -1;
 
 // Quicker to use struct than a std::pair
 struct IntShape
 {
-	IntShape(const int& i, Shape s) : first(i), second(s) {}
+	IntShape(int i, const Shape& s) : first(i), second(s) {}
 	IntShape(const IntShape& o) { *this = o; }
 	IntShape& operator=(const IntShape& o)
 	{
@@ -61,6 +61,8 @@ struct IntShape
 	Shape	second;
 };
 
+class TemplateManager;
+
 class CompDefiner : public Persist
 {
 public:
@@ -74,6 +76,7 @@ public:
 		m_iPinFlags = 0; m_iPadWidth = 70; m_iHoleWidth = 35;
 		m_bAllowFlyWire = false;
 		m_valueStr = m_prefixStr = m_typeStr = m_importStr = "";
+		m_iLabelOffsetRow	= m_iLabelOffsetCol = 0;
 		m_grid.Allocate(1,4,4);
 		m_grid.Clear( Pin(BAD_PINCHAR, SURFACE_FULL, HOLE_FREE) );
 		m_pinLabels.clear();
@@ -93,6 +96,8 @@ public:
 		m_prefixStr			= o.m_prefixStr;
 		m_typeStr			= o.m_typeStr;
 		m_importStr			= o.m_importStr;
+		m_iLabelOffsetRow	= o.m_iLabelOffsetRow;
+		m_iLabelOffsetCol	= o.m_iLabelOffsetCol;
 		m_grid				= o.m_grid;
 		AllocatePins( o.GetNumPins() );
 		std::copy(o.m_pinLabels.begin(), o.m_pinLabels.end(), m_pinLabels.begin());
@@ -105,13 +110,13 @@ public:
 	{
 		return m_pinLabels.size();
 	}
-	void AllocatePins(const size_t numPins)
+	void AllocatePins(size_t numPins)
 	{
 		m_pinLabels.clear();	m_pinLabels.resize(numPins, "");
 		m_pinAligns.clear();	m_pinAligns.resize(numPins, Qt::AlignHCenter);
 		SetDefaultPinLabels();
 	}
-	void ReAllocatePins(const size_t maxPinNumber)
+	void ReAllocatePins(size_t maxPinNumber)
 	{
 		// Take copy of old array values
 		std::vector<std::string>	labels;	labels.resize(GetNumPins());
@@ -140,6 +145,8 @@ public:
 				&& m_prefixStr			== o.m_prefixStr
 				&& m_typeStr			== o.m_typeStr
 				&& m_importStr			== o.m_importStr
+				&& m_iLabelOffsetRow	== o.m_iLabelOffsetRow
+				&& m_iLabelOffsetCol	== o.m_iLabelOffsetCol
 				&& m_grid				== o.m_grid
 				&& m_pinLabels.size()	== o.m_pinLabels.size()
 				&& m_pinAligns.size()	== o.m_pinAligns.size()
@@ -157,32 +164,34 @@ public:
 	{
 		return !(*this == o);
 	}
-	bool SetCurrentPinId(const int& i)		{ const bool bChanged = ( m_currentPinId	!= i );	m_currentPinId		= i; return bChanged; }
-	bool SetCurrentShapeId(const int& i)	{ const bool bChanged = ( m_currentShapeId	!= i );	m_currentShapeId	= i; return bChanged; }
-	bool SetPinFlags(const uchar& i)		{ const bool bChanged = ( m_iPinFlags		!= i );	m_iPinFlags			= i; return bChanged; }
-	bool SetPadWidth(const int& i)			{ const bool bChanged = ( m_iPadWidth		!= i );	m_iPadWidth			= i;
+	bool SetCurrentPinId(int i)				{ const bool bChanged = ( m_currentPinId	!= i );	m_currentPinId		= i; return bChanged; }
+	bool SetCurrentShapeId(int i)			{ const bool bChanged = ( m_currentShapeId	!= i );	m_currentShapeId	= i; return bChanged; }
+	bool SetPinFlags(uchar i)				{ const bool bChanged = ( m_iPinFlags		!= i );	m_iPinFlags			= i; return bChanged; }
+	bool SetPadWidth(int i)					{ const bool bChanged = ( m_iPadWidth		!= i );	m_iPadWidth			= i;
 											  if ( bChanged && GetHoleWidth() > i-8 ) SetHoleWidth( i-8 );	// 8 ==> minimum annular ring = 4 mil
 											  return bChanged;
 											}
-	bool SetHoleWidth(const int& i)			{ const bool bChanged = ( m_iHoleWidth		!= i );	m_iHoleWidth		= i;
+	bool SetHoleWidth(int i)				{ const bool bChanged = ( m_iHoleWidth		!= i );	m_iHoleWidth		= i;
 											  if ( bChanged && GetPadWidth() < i+8 ) SetPadWidth( i+8 );	// 8 ==> minimum annular ring = 4 mil
 											  return bChanged;
 											}
-	bool SetAllowFlyWire(const bool& b)		{ const bool bChanged = ( m_bAllowFlyWire	!= b );	m_bAllowFlyWire		= b; return bChanged; }
+	bool SetAllowFlyWire(bool b)			{ const bool bChanged = ( m_bAllowFlyWire	!= b );	m_bAllowFlyWire		= b; return bChanged; }
 	bool SetValueStr(const std::string& s)	{ const bool bChanged = ( m_valueStr		!= s );	m_valueStr			= s; return bChanged; }
 	bool SetPrefixStr(const std::string& s)	{ const bool bChanged = ( m_prefixStr		!= s );	m_prefixStr			= s; return bChanged; }
 	bool SetTypeStr(const std::string& s)	{ const bool bChanged = ( m_typeStr			!= s );	m_typeStr			= s; return bChanged; }
 	bool SetImportStr(const std::string& s)	{ const bool bChanged = ( m_importStr		!= s );	m_importStr			= s; return bChanged; }
+	bool SetLabelOffsetRow(int i)			{ const bool bChanged = ( m_iLabelOffsetRow	!= i );	m_iLabelOffsetRow	= i; return bChanged; }
+	bool SetLabelOffsetCol(int i)			{ const bool bChanged = ( m_iLabelOffsetCol	!= i );	m_iLabelOffsetCol	= i; return bChanged; }
 	bool SetGrid(const PinGrid& o)			{ const bool bChanged = ( m_grid			!= o );	m_grid				= o; return bChanged; }
-	void SetPinLabel(const size_t& iPinIndex, const std::string& s)
+	void SetPinLabel(size_t iPinIndex, const std::string& s)
 	{
 		if ( iPinIndex < m_pinLabels.size() ) m_pinLabels[iPinIndex] = s;
 	}
-	void SetPinAlign(const size_t& iPinIndex, const int& i)
+	void SetPinAlign(size_t iPinIndex, int i)
 	{
 		if ( iPinIndex < m_pinAligns.size() ) m_pinAligns[iPinIndex] = i;
 	}
-	void AddShape(const int& id, const Shape& o)	{ assert( id != BAD_ID );	m_mapShapes.push_back( IntShape(id, o) ); }
+	void AddShape(int id, const Shape& o)	{ assert( id != BAD_ID );	m_mapShapes.push_back( IntShape(id, o) ); }
 	const int&				GetCurrentPinId() const		{ return m_currentPinId; }
 	const int&				GetCurrentShapeId() const	{ return m_currentShapeId; }
 	const uchar&			GetPinFlags() const			{ return m_iPinFlags; }
@@ -193,13 +202,15 @@ public:
 	const std::string&		GetPrefixStr() const		{ return m_prefixStr; }
 	const std::string&		GetTypeStr() const			{ return m_typeStr; }
 	const std::string&		GetImportStr() const		{ return m_importStr; }
+	const int&				GetLabelOffsetRow() const	{ return m_iLabelOffsetRow; }
+	const int&				GetLabelOffsetCol() const	{ return m_iLabelOffsetCol; }
 	const PinGrid&			GetGrid() const				{ return m_grid; }
-	const std::string&		GetPinLabel(const size_t& iPinIndex) const
+	const std::string&		GetPinLabel(size_t iPinIndex) const
 	{
 		static const std::string emptyStr("");
 		return ( iPinIndex < m_pinLabels.size() ) ? m_pinLabels[iPinIndex] : emptyStr;
 	}
-	const int&				GetPinAlign(const size_t& iPinIndex) const
+	const int&				GetPinAlign(size_t iPinIndex) const
 	{
 		static int defaultAlign(Qt::AlignHCenter);
 		return ( iPinIndex < m_pinAligns.size() ) ? m_pinAligns[iPinIndex] : defaultAlign;
@@ -231,7 +242,7 @@ public:
 		assert(0);
 		return m_mapShapes.begin()->second;
 	}
-	void	MoveCurrentShape(const double& dDown, const double& dRight);
+	void	MoveCurrentShape(double dDown, double dRight);
 	size_t	GetNumTruePins() const
 	{
 		size_t count(0);
@@ -250,8 +261,8 @@ public:
 		}
 		return maxPinNumber;
 	}
-	void Build(Component& comp) const;
-	bool SetPinNumber(const int& i)
+	void Build(const TemplateManager& templateMgr, Component& comp) const;
+	bool SetPinNumber(int i)
 	{
 		if ( GetCurrentPinId() == BAD_ID ) return false;
 		auto& o =  GetCurrentPin();
@@ -351,15 +362,15 @@ public:
 		else
 			return SetPinFlags( GetPinFlags() & ~PIN_RECT );	// Clear bit
 	}
-	bool SetCX(const double& d)			{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetCX(d); return true; }
-	bool SetCY(const double& d)			{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetCY(d); return true; }
-	bool SetDX(const double& d)			{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetDX(d); return true; }
-	bool SetDY(const double& d)			{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetDY(d); return true; }
-	bool SetA1(const double& d)			{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetA1(d); return true; }
-	bool SetA2(const double& d)			{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetA2(d); return true; }
-	bool SetA3(const double& d)			{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetA3(d); return true; }
-	bool SetLine(const bool& b)			{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetDrawLine(b); return true; }
-	bool SetFill(const bool& b)			{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetDrawFill(b); return true; }
+	bool SetCX(double d)				{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetCX(d); return true; }
+	bool SetCY(double d)				{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetCY(d); return true; }
+	bool SetDX(double d)				{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetDX(d); return true; }
+	bool SetDY(double d)				{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetDY(d); return true; }
+	bool SetA1(double d)				{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetA1(d); return true; }
+	bool SetA2(double d)				{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetA2(d); return true; }
+	bool SetA3(double d)				{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetA3(d); return true; }
+	bool SetLine(bool b)				{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetDrawLine(b); return true; }
+	bool SetFill(bool b)				{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetDrawFill(b); return true; }
 	bool SetFillColor(const MyRGB& r)	{ if ( GetCurrentShapeId() == BAD_ID ) return false; GetCurrentShape().SetFillColor(r); return true; }
 	bool GetCanLower() const
 	{
@@ -408,11 +419,11 @@ public:
 	int  CopyShape();
 	int  DestroyShape();
 	int  GetNewShapeId() const;
-	bool SetWidth(const int& i);
-	bool SetHeight(const int& i);
-	int  GetPinId(const int& row, const int& col) const;				// Pick the most relevant pin at the location
-	int  GetShapeId(const double& dRowIn, const double& dColIn) const;	// Pick the most relevant shape at the location
-	bool GetIsValid() const;
+	bool SetWidth(int i);
+	bool SetHeight(int i);
+	int  GetPinId(int row, int col) const;					// Pick the most relevant pin at the location
+	int  GetShapeId(double dRowIn, double dColIn) const;	// Pick the most relevant shape at the location
+	bool GetIsValid(const TemplateManager& templateMgr) const;
 	// Persist functions
 	virtual void Load(DataStream& inStream) override
 	{
@@ -433,6 +444,12 @@ public:
 		inStream.Load(m_prefixStr);
 		inStream.Load(m_typeStr);
 		inStream.Load(m_importStr);
+		m_iLabelOffsetRow = m_iLabelOffsetCol = 0;
+		if ( inStream.GetVersion() >= VRT_VERSION_54 )
+		{
+			inStream.Load(m_iLabelOffsetRow);	// Added in VRT_VERSION_54
+			inStream.Load(m_iLabelOffsetCol);	// Added in VRT_VERSION_54
+		}
 		m_grid.Load(inStream);
 
 		unsigned int numPins(0);
@@ -472,6 +489,8 @@ public:
 		outStream.Save(m_prefixStr);
 		outStream.Save(m_typeStr);
 		outStream.Save(m_importStr);
+		outStream.Save(m_iLabelOffsetRow);	// Added in VRT_VERSION_54
+		outStream.Save(m_iLabelOffsetCol);	// Added in VRT_VERSION_54
 		m_grid.Save(outStream);
 
 		const unsigned int numPins = static_cast<unsigned int>( GetNumPins() );
@@ -512,6 +531,8 @@ private:
 	std::string					m_prefixStr;		// Prefix string (e.g. "IC")
 	std::string					m_typeStr;			// Component type (e.g. "BBD")
 	std::string					m_importStr;		// For Planet/Tango import
+	int							m_iLabelOffsetRow;	// Label offset in units of 1/16 of a grid square
+	int							m_iLabelOffsetCol;	// Label offset in units of 1/16 of a grid square
 	PinGrid						m_grid;
 	std::vector<std::string>	m_pinLabels;		// Pin labels
 	std::vector<int>			m_pinAligns;		// Pin label alignments (Qt::AlignLeft,Qt::AlignRight,Qt::AlignHCenter)
