@@ -98,6 +98,14 @@ enum class COMP {	INVALID					=   -1,
 					RELAY_DIP_4PIN			= 4050,
 					RELAY_DIP_8PIN			= 4052,
 					FUSE_HOLDER				= 5000,
+					SOIC8					= 6000,
+					SOIC14					= 6001,
+					SOIC16					= 6002,
+					SOIC14W					= 6003,
+					SOIC16W					= 6004,
+					SOIC20W					= 6005,
+					SOIC24W					= 6006,
+					SOIC28W					= 6007,
 					VERO_NUMBER				= 10000,	// Not a real component. Used for labeling vero-boards
 					VERO_LETTER				= 10001,	// Not a real component. Used for labeling vero-boards
 					CUSTOM					= 100000,	// A user-defined component
@@ -107,7 +115,7 @@ enum class COMP {	INVALID					=   -1,
 
 struct CompTypes
 {
-	static void UpdateMaps(const COMP& eType, const std::string& typeStr, const std::string& importStr)
+	static void UpdateMaps(COMP eType, const std::string& typeStr, const std::string& importStr)
 	{
 		sm_mapCompTypeToTypeStr[eType]		= typeStr;
 		sm_mapCompTypeToImportStr[eType]	= importStr;
@@ -188,6 +196,14 @@ struct CompTypes
 		UpdateMaps(COMP::RELAY_DIP_4PIN,		"DIP 4-pin",					"RELAY_DIP_4PIN");
 		UpdateMaps(COMP::RELAY_DIP_8PIN,		"DIP 8-pin",					"RELAY_DIP_8PIN");
 		UpdateMaps(COMP::FUSE_HOLDER,			"Fuse Holder",					"FUSE_HOLDER");
+		UpdateMaps(COMP::SOIC8,					"SOIC-8",						"SOIC_8");
+		UpdateMaps(COMP::SOIC14,				"SOIC-14",						"SOIC_14");
+		UpdateMaps(COMP::SOIC16,				"SOIC-16",						"SOIC_16");
+		UpdateMaps(COMP::SOIC14W,				"SOIC-14W",						"SOIC_14W");
+		UpdateMaps(COMP::SOIC16W,				"SOIC-16W",						"SOIC_16W");
+		UpdateMaps(COMP::SOIC20W,				"SOIC-20W",						"SOIC_20W");
+		UpdateMaps(COMP::SOIC24W,				"SOIC-24W",						"SOIC_24W");
+		UpdateMaps(COMP::SOIC28W,				"SOIC-28W",						"SOIC_28W");
 		UpdateMaps(COMP::VERO_NUMBER,			"Vero Numbers",					"");	// No import string
 		UpdateMaps(COMP::VERO_LETTER,			"Vero Letters",					"");	// No import string
 		UpdateMaps(COMP::CUSTOM,				"Custom",						"");	// No import string (user-defined parts have their own strings)
@@ -205,21 +221,21 @@ struct CompTypes
 			if ( mapObj.second == str ) return mapObj.first;
 		return COMP::INVALID;
 	}
-	static std::string GetDefaultImportStr(const COMP& eType)
+	static std::string GetDefaultImportStr(COMP eType)
 	{
 		InitMapsCompTypeToStr();
 		const auto iter = sm_mapCompTypeToImportStr.find(eType);
 		if ( iter != sm_mapCompTypeToImportStr.end() ) return iter->second;
 		return "INVALID";
 	}
-	static std::string GetDefaultTypeStr(const COMP& eType)
+	static std::string GetDefaultTypeStr(COMP eType)
 	{
 		InitMapsCompTypeToStr();
 		const auto iter = sm_mapCompTypeToTypeStr.find(eType);
 		if ( iter != sm_mapCompTypeToTypeStr.end() ) return iter->second;
 		return "INVALID";
 	}
-	static int GetListOrder(const COMP& eType)	// For dialogs/menus.  Lower number ==> higher up list
+	static int GetListOrder(COMP eType)	// For dialogs/menus.  Lower number ==> higher up list
 	{
 		switch( eType )
 		{
@@ -263,23 +279,31 @@ struct CompTypes
 			case COMP::TRIM_3362W:
 			case COMP::TRIM_3362X:
 			case COMP::TRIM_3362Z:			return 8;
+			case COMP::CRYSTAL:				return 9;
 			case COMP::DIODE:
 			case COMP::LED:					return 10;
 			case COMP::TO92:
 			case COMP::TO18:
 			case COMP::TO39:
 			case COMP::TO220:				return 11;
-			case COMP::DIP:					return 12;
-			case COMP::DIP_RECTIFIER:		return 13;
-			case COMP::SIP:					return 14;
+			case COMP::SOIC8:			
+			case COMP::SOIC14:			
+			case COMP::SOIC16:			
+			case COMP::SOIC14W:			
+			case COMP::SOIC16W:			
+			case COMP::SOIC20W:			
+			case COMP::SOIC24W:			
+			case COMP::SOIC28W:				return 12;
+			case COMP::DIP:					return 13;
+			case COMP::DIP_RECTIFIER:		return 14;
+			case COMP::SIP:					return 15;
 			case COMP::STRIP_100:
 			case COMP::BLOCK_100:
-			case COMP::BLOCK_200:			return 15;
+			case COMP::BLOCK_200:			return 16;
 			case COMP::SWITCH_ST:
 			case COMP::SWITCH_DT:
 			case COMP::SWITCH_ST_DIP:
-			case COMP::SWITCH_BUTTON_4PIN:	return 16;
-			case COMP::CRYSTAL:				return 9;
+			case COMP::SWITCH_BUTTON_4PIN:	return 17;
 			case COMP::RELAY_HK19F:
 			case COMP::RELAY_HJR_4102:
 			case COMP::RELAY_FTR_B3C:
@@ -292,15 +316,15 @@ struct CompTypes
 			case COMP::RELAY_S1A050000:
 			case COMP::RELAY_TRCD:
 			case COMP::RELAY_DIP_4PIN:
-			case COMP::RELAY_DIP_8PIN:		return 17;
-			case COMP::FUSE_HOLDER:			return 18;
+			case COMP::RELAY_DIP_8PIN:		return 18;
+			case COMP::FUSE_HOLDER:			return 19;
 			case COMP::MARK:				return 100;
 			case COMP::VERO_NUMBER:			return 200;
 			case COMP::VERO_LETTER:			return 201;
 			default:						return 1000;	// Unhandled eType
 		}
 	}
-	static bool IsPlug(const COMP& type)	// true ==> Can plug gap between rows of IC pins
+	static bool IsPlug(COMP type)	// true ==> Can plug gap between rows of IC pins
 	{
 		switch( type )
 		{
@@ -314,7 +338,7 @@ struct CompTypes
 			default:				return false;
 		}
 	}
-	static std::string GetFamilyStr(const COMP& eType)	// For grouping in the templates
+	static std::string GetFamilyStr(COMP eType)	// For grouping in the templates
 	{
 		switch( eType )
 		{
@@ -376,7 +400,7 @@ struct CompTypes
 			default:						return "";
 		}
 	}
-	static std::string GetDefaultPrefixStr(const COMP& eType)	// Prefix for name on creation
+	static std::string GetDefaultPrefixStr(COMP eType)	// Prefix for name on creation
 	{
 		switch( eType )
 		{
@@ -452,16 +476,25 @@ struct CompTypes
 			case COMP::RELAY_DIP_4PIN:
 			case COMP::RELAY_DIP_8PIN:		return "SW";
 			case COMP::FUSE_HOLDER:			return "F";
+			case COMP::SOIC8:			
+			case COMP::SOIC14:			
+			case COMP::SOIC16:			
+			case COMP::SOIC14W:			
+			case COMP::SOIC16W:			
+			case COMP::SOIC20W:			
+			case COMP::SOIC24W:			
+			case COMP::SOIC28W:				return "IC";
 			case COMP::CUSTOM:				return "";
 			case COMP::VERO_NUMBER:			return "Vero Numbers";
 			case COMP::VERO_LETTER:			return "Vero Letters";
 			default:						return "INVALID";	// Unhandled eType
 		}
 	}
-	static bool AllowCustomPCBshapes(const COMP& eType)
+	static bool AllowCustomPCBshapes(COMP eType)
 	{
 		switch(eType)
 		{
+			case COMP::PAD:
 			case COMP::RESISTOR:
 			case COMP::INDUCTOR:
 			case COMP::DIODE:
@@ -472,7 +505,7 @@ struct CompTypes
 			default:					return false;
 		}
 	}
-	static bool AllowTypeChange(const COMP& eTypeA, const COMP& eTypeB)
+	static bool AllowTypeChange(COMP eTypeA, COMP eTypeB)
 	{
 		const std::string prefixA = GetDefaultPrefixStr(eTypeA);
 		const std::string prefixB = GetDefaultPrefixStr(eTypeB);
@@ -484,7 +517,7 @@ struct CompTypes
 		if ( prefixA == std::string("Strip") )	return true;
 		return false;
 	}
-	static int GetPinSeparation(const COMP& eType)	// To handle change of component type for LEDs and electro-caps.
+	static int GetPinSeparation(COMP eType)	// To handle change of component type for LEDs and electro-caps.
 	{
 		switch( eType )
 		{
@@ -512,7 +545,7 @@ struct CompTypes
 			if ( mapObj.second == str ) return mapObj.first;
 		return COMP::INVALID;
 	}
-	static std::string GetMakeInstructions(const COMP& eType, int& rows, int& cols)
+	static std::string GetMakeInstructions(COMP eType, int& rows, int& cols)
 	{
 		switch( eType )
 		{
@@ -588,6 +621,14 @@ struct CompTypes
 			case COMP::RELAY_DIP_4PIN		: rows = 4; cols = 7;  return "4+++++3+++++++++++++++1+++2+";
 			case COMP::RELAY_DIP_8PIN		: rows = 4; cols = 7;  return "87+++65++++++++++++++12+++34";
 			case COMP::FUSE_HOLDER			: rows = 3; cols = 10; return "++++++++++1++++++++2++++++++++";
+			case COMP::SOIC8				: rows = 6; cols = 4;  return "8765.**..**..**..**.1234";
+			case COMP::SOIC14				: rows = 6; cols = 5;  return "DCBA9E***8+***++***+1***723456";
+			case COMP::SOIC16				: rows = 6; cols = 6;  return "FEDCBAG****9.****..****.1****8234567";
+			case COMP::SOIC14W				: rows = 7; cols = 5;  return "DCBA9E***8+++++++++++++++1***723456";
+			case COMP::SOIC16W				: rows = 7; cols = 6;  return "FEDCBAG****9.++++..++++..++++.1****8234567";
+			case COMP::SOIC20W				: rows = 9; cols = 8;  return "JIHGFEDCK******B.******..++++++..++++++..++++++..******.1******A23456789";
+			case COMP::SOIC24W				: rows = 9; cols = 8;  return "MLKJIHGFN******EO******D.++++++..++++++..++++++.1******C2******B3456789A";
+			case COMP::SOIC28W				: rows = 9; cols = 10; return "QPONMLKJIHR********GS********F.++++++++..++++++++..++++++++.1********E2********D3456789ABC";
 			case COMP::VERO_NUMBER			: rows = 1; cols = 10; return "..........";
 			case COMP::VERO_LETTER			: rows = 1; cols = 10; return "..........";
 			case COMP::CUSTOM				: rows = 1; cols = 1;  return ".";
@@ -595,11 +636,41 @@ struct CompTypes
 			default:	assert(0);			  rows = 0; cols = 0;  return "";	// Unhandled eType
 		}
 	}
+	static std::string GetAliasNotes(COMP eType)
+	{
+		switch( eType )
+		{
+			case COMP::SOIC8:	return "Body Size (3.9mm x 4.9mm)";
+			case COMP::SOIC14:	return "Body Size (3.9mm x 8.7mm)";
+			case COMP::SOIC16:	return "Body Size (3.9mm x 9.9mm)";
+			case COMP::SOIC14W:	return "Body Size (7.5mm x 9.0mm)";
+			case COMP::SOIC16W:	return "Body Size (7.5mm x 10.3mm)";
+			case COMP::SOIC20W:	return "Body Size (7.5mm x 12.8mm)";
+			case COMP::SOIC24W:	return "Body Size (7.5mm x 15.4mm)";
+			case COMP::SOIC28W:	return "Body Size (7.5mm x 17.9mm)";
+			default:			return "";
+		}
+	}
+	static bool	GetIsSOIC(COMP eType)
+	{
+		switch( eType )
+		{
+			case COMP::SOIC8:
+			case COMP::SOIC14:
+			case COMP::SOIC16:
+			case COMP::SOIC14W:
+			case COMP::SOIC16W:
+			case COMP::SOIC20W:
+			case COMP::SOIC24W:
+			case COMP::SOIC28W:	return true;
+			default:			return false;
+		}
+	}
 	static std::string GetDefaultPinLabel(size_t iPinIndex)
 	{
 		return std::to_string(iPinIndex + 1);	// Pin numbers on screen start at 1
 	}
-	static int GetDefaultPinAlign(size_t iPinIndex, size_t iNumPins, const COMP& eType)
+	static int GetDefaultPinAlign(size_t iPinIndex, size_t iNumPins, COMP eType)
 	{
 		switch( eType )
 		{
@@ -608,7 +679,7 @@ struct CompTypes
 			default:					return Qt::AlignHCenter;
 		}
 	}
-	static int GetDefaultNumPins(const COMP& eType)
+	static int GetDefaultNumPins(COMP eType)
 	{
 		switch( eType )
 		{
@@ -684,13 +755,21 @@ struct CompTypes
 			case COMP::RELAY_DIP_4PIN:		return 4;
 			case COMP::RELAY_DIP_8PIN:		return 8;
 			case COMP::FUSE_HOLDER:			return 2;
+			case COMP::SOIC8:				return 8;
+			case COMP::SOIC14:				return 14;
+			case COMP::SOIC16:				return 16;
+			case COMP::SOIC14W:				return 14;
+			case COMP::SOIC16W:				return 16;
+			case COMP::SOIC20W:				return 20;
+			case COMP::SOIC24W:				return 24;
+			case COMP::SOIC28W:				return 28;
 			case COMP::VERO_NUMBER:			return 0;
 			case COMP::VERO_LETTER:			return 0;
 			case COMP::CUSTOM:				return 0;
 			default:						return 0;	// Unhandled eType
 		}
 	}
-	static int GetMinNumPins(const COMP& eType)
+	static int GetMinNumPins(COMP eType)
 	{
 		switch( eType )
 		{
@@ -705,7 +784,7 @@ struct CompTypes
 			default:					return GetDefaultNumPins(eType);
 		}
 	}
-	static int GetMaxNumPins(const COMP& eType)
+	static int GetMaxNumPins(COMP eType)
 	{
 		switch( eType )
 		{
@@ -720,7 +799,18 @@ struct CompTypes
 			default:					return GetDefaultNumPins(eType);
 		}
 	}
-	static int GetStretchIncrement(const COMP& eType)	// For stretchable components
+	static int GetModuloNumPins(COMP eType)
+	{
+		switch( eType )
+		{
+			case COMP::DIP:
+			case COMP::SWITCH_ST:
+			case COMP::SWITCH_ST_DIP:	return 2;
+			case COMP::SWITCH_DT:		return 3;
+			default:					return 1;
+		}
+	}
+	static int GetStretchIncrement(COMP eType)	// For stretchable components
 	{
 		switch( eType )
 		{
@@ -730,7 +820,7 @@ struct CompTypes
 			default:				return 1;
 		}
 	}
-	static int GetMinLength(const COMP& eType)	// For stretchable components
+	static int GetMinLength(COMP eType)	// For stretchable components
 	{
 		switch( eType )
 		{
@@ -754,7 +844,7 @@ struct CompTypes
 			default:	assert(0);		return 1;	// Non-stretchable component
 		}
 	}
-	static int GetMaxLength(const COMP& eType)	// For stretchable components
+	static int GetMaxLength(COMP eType)	// For stretchable components
 	{
 		switch( eType )
 		{
