@@ -18,6 +18,14 @@
 */
 
 #include "Component.h"
+#include "TemplateManager.h"
+
+Component::Component(const TemplateManager& templateMgr, const CompDefiner& definer)	// This method is for building a custom component
+{
+	Clear();
+
+	definer.Build(templateMgr, *this);	// Use component definer to make the footprint and shapes
+}
 
 void Component::SetDefaultPinFlags()
 {
@@ -50,6 +58,14 @@ void Component::SetDefaultPinFlags()
 		case COMP::RELAY_S1A050000:
 		case COMP::RELAY_TRCD:
 		case COMP::FUSE_HOLDER:		return SetPinFlags(static_cast<uchar>(PIN_RECT | iCustomFlag));
+		case COMP::SOIC8:
+		case COMP::SOIC14:
+		case COMP::SOIC16:
+		case COMP::SOIC14W:
+		case COMP::SOIC16W:
+		case COMP::SOIC20W:
+		case COMP::SOIC24W:
+		case COMP::SOIC28W:			return SetPinFlags(static_cast<uchar>(PIN_LABELS));
 		case COMP::RELAY_DIP_4PIN:
 		case COMP::RELAY_DIP_8PIN:	return SetPinFlags(static_cast<uchar>(PIN_RECT | PIN_LABELS | iCustomFlag));
 		case COMP::MARK:
@@ -105,7 +121,7 @@ void Component::SetDefaultPinFlags()
 	}
 }
 
-void Component::SetDefaultShapes(const bool& bUsePCBshapes)
+void Component::SetDefaultShapes(bool bUsePCBshapes)
 {
 	switch( GetType() )
 	{
@@ -120,9 +136,9 @@ void Component::SetDefaultShapes(const bool& bUsePCBshapes)
 		case COMP::MARK:					AddOne( Shape(SHAPE::ELLIPSE,	true, false,	-0.30,  0.30, -0.30,  0.30) );
 											AddOne( Shape(SHAPE::LINE,		true, false,	-0.21,  0.21, -0.21,  0.21) );
 											AddOne( Shape(SHAPE::LINE,		true, false,	-0.21,  0.21,  0.21, -0.21) );	break;
-		case COMP::PAD:						AddTwo( Shape(SHAPE::ELLIPSE,	true, true,		-0.45,  0.45, -0.45,  0.45) );	break;
+		case COMP::PAD:						AddTwo( Shape(bUsePCBshapes ? SHAPE::RECT : SHAPE::ELLIPSE,	true, true,	-0.45,  0.45, -0.45,  0.45) );	break;
 		case COMP::PAD_FLYINGWIRE:			AddTwo( Shape(SHAPE::ROUNDED_RECT, true, true,	-0.45,  0.45, -0.45,  0.45) );	break;
-		case COMP::LED:						AddTwo( Shape(SHAPE::CHORD,		true, true,		-0.75,  0.75, -0.75,  0.75, 30, -30) );break;
+		case COMP::LED:						AddTwo( Shape(SHAPE::CHORD,		true, true,		-0.875, 0.875, -0.875, 0.875, 30, -30) );break;
 		case COMP::CAP_ELECTRO_200_NP:		AddTwo( Shape(SHAPE::ELLIPSE,	true, true,		-1.00,  1.00, -1.00,  1.00) );	break;
 		case COMP::CAP_ELECTRO_250_NP:		AddTwo( Shape(SHAPE::ELLIPSE,	true, true,		-1.25,  1.25, -1.25,  1.25) );	break;
 		case COMP::CAP_ELECTRO_300_NP:		AddTwo( Shape(SHAPE::ELLIPSE,	true, true,		-1.50,  1.50, -1.50,  1.50) );	break;
@@ -192,6 +208,32 @@ void Component::SetDefaultShapes(const bool& bUsePCBshapes)
 		case COMP::RELAY_S1A050000:			AddTwo( Shape(SHAPE::RECT,		true, true,		-3.74,  3.74, -1.00,  1.00) );	break;
 		case COMP::RELAY_TRCD:				AddTwo( Shape(SHAPE::RECT,		true, true,		-4.53,  4.53, -3.17,  3.17) );	break;
 		case COMP::FUSE_HOLDER:				AddTwo( Shape(SHAPE::RECT,		true, true,		-4.85,  4.85, -1.35,  1.35) );	break;
+		case COMP::SOIC8:
+		case COMP::SOIC14:
+		case COMP::SOIC16:
+		case COMP::SOIC14W:
+		case COMP::SOIC16W:
+		case COMP::SOIC20W:
+		case COMP::SOIC24W:
+		case COMP::SOIC28W:
+		{
+			double w(0), h(0);
+			switch( GetType() )
+			{
+				case COMP::SOIC8:	w = 1.00;	h = 0.38;	break;
+				case COMP::SOIC14:	w = 1.75;	h = 0.38;	break;
+				case COMP::SOIC16:	w = 2.00;	h = 0.38;	break;
+				case COMP::SOIC14W:	w = 1.75;	h = 1.05;	break;
+				case COMP::SOIC16W:	w = 2.00;	h = 1.05;	break;
+				case COMP::SOIC20W:	w = 2.50;	h = 1.05;	break;
+				case COMP::SOIC24W:	w = 3.00;	h = 1.05;	break;
+				case COMP::SOIC28W:	w = 3.50;	h = 1.05;	break;
+				default:			assert(0);				break;
+			}
+			AddOne( Shape(SHAPE::ELLIPSE,	true, false, -w+0.1, -w+0.35, h-0.35, h-0.1));
+			AddTwo( Shape(SHAPE::RECT,		true, true,	 -w, w, -h, h));
+			break;
+		}
 		// Following handle variable length components
 		case COMP::DIP:
 		case COMP::SIP:
@@ -378,6 +420,14 @@ void Component::SetDefaultColor()
 		case COMP::RELAY_S1A050000:
 		case COMP::RELAY_TRCD:				return SetFillColor(MyRGB(0x84C0D0));
 		case COMP::FUSE_HOLDER:				return SetFillColor(MyRGB(0x909090));
+		case COMP::SOIC8:
+		case COMP::SOIC14:
+		case COMP::SOIC16:
+		case COMP::SOIC14W:
+		case COMP::SOIC16W:
+		case COMP::SOIC20W:
+		case COMP::SOIC24W:
+		case COMP::SOIC28W:
 		case COMP::DIP:
 		case COMP::SIP:
 		case COMP::DIP_RECTIFIER:
@@ -439,7 +489,7 @@ void Component::GetLabelOffsets(int& offsetRow, int& offsetCol) const	// w.r.t.
 	}
 }
 
-void Component::MoveLabelOffsets(const int& deltaRow, const int& deltaCol)	// w.r.t. screen, not comp rotation
+void Component::MoveLabelOffsets(int deltaRow, int deltaCol)	// w.r.t. screen, not comp rotation
 {
 	switch( GetDirection() )
 	{
