@@ -27,13 +27,34 @@
 // This only affects rendered connections between adjacent pins..
 // It does not affect connectivity or routing.
 // The joint preference of adjacent pins determines which layers will show the connection.
-static const uchar	LAYER_X = 0;	// No preference
-static const uchar	LAYER_B	= 1;	// Prefer bottom layer
-static const uchar	LAYER_T	= 2;	// Prefer top layer
+Q_DECL_CONSTEXPR static const uchar	LAYER_X = 0;	// No preference
+Q_DECL_CONSTEXPR static const uchar	LAYER_B	= 1;	// Prefer bottom layer
+Q_DECL_CONSTEXPR static const uchar	LAYER_T	= 2;	// Prefer top layer
 
-static const int MAX_PAD_OFFSET_MIL = 50;
+Q_DECL_CONSTEXPR static const int	MAX_PAD_OFFSET_MIL = 50;
 
 class CompManager;
+class TemplateManager;
+
+struct CompStrings	// Mainly for handling netlist import
+{
+	CompStrings() {}
+	CompStrings(const std::string& nameStr, const std::string& valueStr, const std::string& importStr)
+	: m_nameStr(nameStr), m_valueStr(valueStr), m_importStr(importStr) {}
+	~CompStrings() {}
+	CompStrings(const CompStrings& o) { *this = o; }
+	CompStrings& operator=(const CompStrings& o)
+	{
+		m_nameStr	= o.m_nameStr;
+		m_valueStr	= o.m_valueStr;
+		m_importStr	= o.m_importStr;
+		return *this;
+	}
+	void Clear() { m_nameStr.clear(); m_valueStr.clear(); m_importStr.clear(); }
+	std::string	m_nameStr;		// e.q. "U1"
+	std::string	m_valueStr;		// e.g. "TL072"
+	std::string	m_importStr;	// e.g. "DIP8"
+};
 
 // Class to describe a component.
 // Wires (i.e. jumpers) and markers also use this class even though they are not true components.
@@ -67,13 +88,8 @@ public:
 		m_pinAligns.clear();
 		m_shapes.clear();
 	}
-	Component(const CompDefiner& definer)	// This method is for building a custom component
-	{
-		Clear();
-
-		definer.Build(*this);		// Use component definer to make the footprint and shapes
-	}
-	Component(CompManager* pCompMgr, const RectManager& rectMgr, const ElementGrid& grid, const int& nLyr, const int& nRowMin, const int& nRowMax, const int& nColMin, const int& nColMax)
+	Component(const TemplateManager& templateMgr, const CompDefiner& definer);	// This method is for building a custom component
+	Component(CompManager* pCompMgr, const RectManager& rectMgr, const ElementGrid& grid, int nLyr, int nRowMin, int nRowMax, int nColMin, int nColMax)
 	{
 		Clear();
 
@@ -86,7 +102,7 @@ public:
 		m_col = nColMin;
 		m_bIsPlaced = false;	// Trax are always created from the board
 	}
-	Component(const std::string& name, const std::string& value, const COMP& eType, std::vector<int>& nodeIdPins)
+	Component(const std::string& name, const std::string& value, COMP eType, std::vector<int>& nodeIdPins)
 	{
 		Clear();
 
@@ -204,17 +220,17 @@ public:
 	}
 	bool operator<(const Component& o) const	{ return m_id < o.m_id; }
 	bool operator==(const Component& o) const	{ return m_id == o.m_id; }
-	void SetId(const int& i)					{ m_id = i; }
+	void SetId(int i)							{ m_id = i; }
 	void SetNameStr(const std::string& s)		{ m_nameStr = s; }
 	void SetValueStr(const std::string& s)		{ m_valueStr = s; }
 	void SetPrefixStr(const std::string& s)		{ m_prefixStr = s; }
 	void SetTypeStr(const std::string& s)		{ m_typeStr = s; }
 	void SetImportStr(const std::string& s)		{ m_importStr = s; }
-	void SetNodeId(const size_t& iPinIndex, const int& i)
+	void SetNodeId(size_t iPinIndex, int i)
 	{
 		if ( iPinIndex < m_nodeIdPins.size() ) m_nodeIdPins[iPinIndex] = i;
 	}
-	void SetOrigId(const int& lyr, const size_t& iPinIndex, const int& i)
+	void SetOrigId(int lyr, size_t iPinIndex, int i)
 	{
 		assert( lyr == 0 || lyr == 1 );
 		if ( lyr == 0 )
@@ -226,27 +242,27 @@ public:
 			if ( iPinIndex < m_origIdPins2.size() ) m_origIdPins2[iPinIndex] = i;
 		}
 	}
-	void SetLayerPref(const size_t& iPinIndex, const uchar& iPref)
+	void SetLayerPref(size_t iPinIndex, uchar iPref)
 	{
 		if ( iPinIndex < m_layerPrefs.size() ) m_layerPrefs[iPinIndex] = iPref;
 	}
-	void SetPinOffsetRow(const size_t& iPinIndex, const int& i)
+	void SetPinOffsetRow(size_t iPinIndex, int i)
 	{
 		if ( iPinIndex < m_pinOffsetRow.size() ) m_pinOffsetRow[iPinIndex] = std::max(-MAX_PAD_OFFSET_MIL, std::min(MAX_PAD_OFFSET_MIL, i));
 	}
-	void SetPinOffsetCol(const size_t& iPinIndex, const int& i)
+	void SetPinOffsetCol(size_t iPinIndex, int i)
 	{
 		if ( iPinIndex < m_pinOffsetCol.size() ) m_pinOffsetCol[iPinIndex] = std::max(-MAX_PAD_OFFSET_MIL, std::min(MAX_PAD_OFFSET_MIL, i));
 	}
-	void SetPinLabel(const size_t& iPinIndex, const std::string& s)
+	void SetPinLabel(size_t iPinIndex, const std::string& s)
 	{
 		if ( iPinIndex < m_pinLabels.size() ) m_pinLabels[iPinIndex] = s;
 	}
-	void SetPinAlign(const size_t& iPinIndex, const int& i)
+	void SetPinAlign(size_t iPinIndex, int i)
 	{
 		if ( iPinIndex < m_pinAligns.size() ) m_pinAligns[iPinIndex] = i;
 	}
-	void SetShape(const size_t& iShapeIndex, const Shape& o)
+	void SetShape(size_t iShapeIndex, const Shape& o)
 	{
 		if ( iShapeIndex < m_shapes.size() ) m_shapes[iShapeIndex] = o;
 	}
@@ -262,7 +278,7 @@ public:
 		AllocateShapes( o.GetNumShapes() );
 		std::copy(o.m_shapes.begin(), o.m_shapes.end(), m_shapes.begin());
 	}
-	void AllocatePins(const size_t numPins)
+	void AllocatePins(size_t numPins)
 	{
 		m_nodeIdPins.clear();	m_nodeIdPins.resize(numPins, BAD_NODEID);
 		m_origIdPins1.clear();	m_origIdPins1.resize(numPins, BAD_NODEID);
@@ -274,22 +290,22 @@ public:
 		m_pinAligns.clear();	m_pinAligns.resize(numPins, Qt::AlignHCenter);
 		SetDefaultPinLabels();
 	}
-	void AllocateShapes(const size_t numShapes)
+	void AllocateShapes(size_t numShapes)
 	{
 		m_shapes.clear();	m_shapes.resize(numShapes, Shape());
 	}
-	void SetLyr(const int& i)					{ m_lyr = i; }
-	void SetRow(const int& i)					{ m_row = i; }
-	void SetCol(const int& i)					{ m_col = i; }
-//	void SetLabelOffsetRow(const int& i)		{ m_iLabelOffsetRow = i; }
-//	void SetLabelOffsetCol(const int& i)		{ m_iLabelOffsetCol = i; }
-	void SetDirection(const char& d)			{ m_direction = d; }
-	void SetIsPlaced(const bool& b)				{ m_bIsPlaced = b; }
-	void SetPinFlags(const uchar& i)			{ m_iPinFlags = i; }
-	void SetPadWidth(const int& i)				{ m_iPadWidth = i; }
-	void SetHoleWidth(const int& i)				{ m_iHoleWidth = i; }
-	void SetAllowFlyWire(const bool& b)			{ m_bAllowFlyWire = b; }
-	void AddOne(const Shape& s)					{ m_shapes.push_back(s); }
+	void SetLyr(int i)				{ m_lyr = i; }
+	void SetRow(int i)				{ m_row = i; }
+	void SetCol(int i)				{ m_col = i; }
+	void SetLabelOffsetRow(int i)	{ m_iLabelOffsetRow = i; }
+	void SetLabelOffsetCol(int i)	{ m_iLabelOffsetCol = i; }
+	void SetDirection(char d)		{ m_direction = d; }
+	void SetIsPlaced(bool b)		{ m_bIsPlaced = b; }
+	void SetPinFlags(uchar i)		{ m_iPinFlags = i; }
+	void SetPadWidth(int i)			{ m_iPadWidth = i; }
+	void SetHoleWidth(int i)		{ m_iHoleWidth = i; }
+	void SetAllowFlyWire(bool b)	{ m_bAllowFlyWire = b; }
+	void AddOne(const Shape& s)		{ m_shapes.push_back(s); }
 	void AddTwo(const Shape& s)	// Adds the shape twice.  Once with fill only, and once with line only
 	{
 		Shape tmp(s);
@@ -305,12 +321,12 @@ public:
 	const std::string&	GetImportStr() const	{ return m_importStr; }
 	size_t				GetNumPins() const		{ return m_nodeIdPins.size(); }
 	size_t				GetNumShapes() const	{ return m_shapes.size(); }
-	const int&			GetNodeId(const size_t& iPinIndex) const
+	const int&			GetNodeId(size_t iPinIndex) const
 	{
 		static int badNodeId(BAD_NODEID);
 		return ( iPinIndex < m_nodeIdPins.size() ) ? m_nodeIdPins[iPinIndex] : badNodeId;
 	}
-	const int&			GetOrigId(const int& lyr, const size_t& iPinIndex) const
+	const int&			GetOrigId(int lyr, size_t iPinIndex) const
 	{
 		static int badNodeId(BAD_NODEID);
 		if ( lyr == 0 )
@@ -318,32 +334,32 @@ public:
 		else
 			return ( iPinIndex < m_origIdPins2.size() ) ? m_origIdPins2[iPinIndex] : badNodeId;
 	}
-	const uchar&		GetLayerPref(const size_t& iPinIndex) const
+	const uchar&		GetLayerPref(size_t iPinIndex) const
 	{
 		static uchar noPref(LAYER_X);
 		return ( iPinIndex < m_layerPrefs.size() ) ? m_layerPrefs[iPinIndex] : noPref;
 	}
-	const int&			GetPinOffsetRow(const size_t& iPinIndex) const
+	const int&			GetPinOffsetRow(size_t iPinIndex) const
 	{
 		static int defaultOffset(0);
 		return ( iPinIndex < m_pinOffsetRow.size() ) ? m_pinOffsetRow[iPinIndex] : defaultOffset;
 	}
-	const int&			GetPinOffsetCol(const size_t& iPinIndex) const
+	const int&			GetPinOffsetCol(size_t iPinIndex) const
 	{
 		static int defaultOffset(0);
 		return ( iPinIndex < m_pinOffsetCol.size() ) ? m_pinOffsetCol[iPinIndex] : defaultOffset;
 	}
-	const std::string&	GetPinLabel(const size_t& iPinIndex) const
+	const std::string&	GetPinLabel(size_t iPinIndex) const
 	{
 		static const std::string emptyStr("");
 		return ( iPinIndex < m_pinLabels.size() ) ? m_pinLabels[iPinIndex] : emptyStr;
 	}
-	const int&			GetPinAlign(const size_t& iPinIndex) const
+	const int&			GetPinAlign(size_t iPinIndex) const
 	{
 		static int defaultAlign(Qt::AlignHCenter);
 		return ( iPinIndex < m_pinAligns.size() ) ? m_pinAligns[iPinIndex] : defaultAlign;
 	}
-	const Shape&		GetShape(const size_t& iShapeIndex) const
+	const Shape&		GetShape(size_t iShapeIndex) const
 	{
 		static Shape	defaultShape;
 		return ( iShapeIndex < m_shapes.size() ) ? m_shapes[iShapeIndex] : defaultShape;
@@ -351,8 +367,8 @@ public:
 	const int&			GetLyr() const				{ return m_lyr; }
 	const int&			GetRow() const				{ return m_row; }
 	const int&			GetCol() const				{ return m_col; }
-//	const int&			GetLabelOffsetRow() const	{ return m_iLabelOffsetRow; }
-//	const int&			GetLabelOffsetCol() const	{ return m_iLabelOffsetCol; }
+	const int&			GetLabelOffsetRow() const	{ return m_iLabelOffsetRow; }
+	const int&			GetLabelOffsetCol() const	{ return m_iLabelOffsetCol; }
 	const char&			GetDirection() const		{ return m_direction; }
 	const bool&			GetIsPlaced() const			{ return m_bIsPlaced; }
 	const uchar&		GetPinFlags() const			{ return m_iPinFlags; }
@@ -360,17 +376,23 @@ public:
 	const int&			GetHoleWidth() const		{ return m_iHoleWidth; }
 	const bool&			GetAllowFlyWire() const		{ return m_bAllowFlyWire; }
 	const std::vector<Shape>& GetShapes() const		{ return m_shapes; }
-
+	bool				GetIsSOIC() const
+	{
+#ifdef _TEST_SOIC		
+		if ( GetValueStr() == "SOIC28_TEST" ) return true;	// So we can make test SOICs using component editor
+#endif			
+		return CompTypes::GetIsSOIC( GetType() );
+	}
 	// Helpers for labels
 	void SetDefaultLabelOffsets();
-	void GetLabelOffsets(int& offsetRow, int& offsetCol) const;			// w.r.t. screen, not comp rotation
-	void MoveLabelOffsets(const int& deltaRow, const int& deltaCol);	// w.r.t. screen, not comp rotation
+	void GetLabelOffsets(int& offsetRow, int& offsetCol) const;	// w.r.t. screen, not comp rotation
+	void MoveLabelOffsets(int deltaRow, int deltaCol);			// w.r.t. screen, not comp rotation
 	void HandleLegacyLabelOffsets();	// For old VRT files
 
 	// Helpers for custom pads
-	void SetCustomPads(const bool& b)	{ if ( b )	SetPinFlags( m_iPinFlags |  PIN_CUSTOM );
-										  else		SetPinFlags( m_iPinFlags & ~PIN_CUSTOM ); }
-	bool GetCustomPads() const			{ return ( GetPinFlags() & PIN_CUSTOM ) != 0; }
+	void SetCustomPads(bool b)	{ if ( b )	SetPinFlags( m_iPinFlags |  PIN_CUSTOM );
+								  else		SetPinFlags( m_iPinFlags & ~PIN_CUSTOM ); }
+	bool GetCustomPads() const	{ return ( GetPinFlags() & PIN_CUSTOM ) != 0; }
 
 	void GetSafeBounds(double& L, double& R, double& T, double& B, bool bFill = true) const
 	{
@@ -398,28 +420,26 @@ public:
 	}
 	std::string GetFullTypeStr() const		// For SIP/DIP types, append the number of pins
 	{
-		std::string str = GetTypeStr();
 		if ( GetType() == COMP::DIP || GetType() == COMP::SIP )
-			str += std::to_string(GetNumPins());	// e.g. "DIP16"
+			return CompTypes::GetDefaultTypeStr( GetType() ) + std::to_string(GetNumPins());	// e.g. "DIP16"
 		if ( GetType() == COMP::STRIP_100 || GetType() == COMP::BLOCK_100 || GetType() == COMP::BLOCK_200 )
-			str += std::string(" (") + std::to_string(GetNumPins()) + std::string(" pins)");
-		return str;
+			return CompTypes::GetDefaultTypeStr( GetType() ) + std::string(" (") + std::to_string(GetNumPins()) + std::string(" pins)");
+		return GetTypeStr();
 	}
 	std::string GetFullImportStr() const	// For SIP/DIP/SWITCH/STRIP/BLOCK types, append the number of pins
 	{
-		std::string str = GetImportStr();
 		if ( GetType() == COMP::DIP || GetType() == COMP::SIP ||
 			 GetType() == COMP::SWITCH_DT || GetType() == COMP::SWITCH_ST || GetType() == COMP::SWITCH_ST_DIP ||
 			 GetType() == COMP::STRIP_100 || GetType() == COMP::BLOCK_100 || GetType() == COMP::BLOCK_200 )
-			str += std::to_string(GetNumPins());	// e.g. "DIP16"
-		return str;
+			return CompTypes::GetDefaultImportStr( GetType() ) + std::to_string(GetNumPins());	// e.g. "DIP16"
+		return GetImportStr();
 	}
 	// Helpers (account for component direction)
 	const int&	GetCompRows() const	{ return GetRows( GetDirection() ); }
 	const int&	GetCompCols() const	{ return GetCols( GetDirection() ); }
 	int			GetLastRow() const	{ return GetRow() + GetCompRows() - 1; }
 	int			GetLastCol() const	{ return GetCol() + GetCompCols() - 1; }
-	void GetCompPinOffsets(const size_t& iPinIndex, int& Xmil, int& Ymil) const
+	void GetCompPinOffsets(size_t iPinIndex, int& Xmil, int& Ymil) const
 	{
 		switch ( GetDirection() )
 		{
@@ -429,7 +449,7 @@ public:
 			default:	Xmil =  GetPinOffsetCol(iPinIndex);	Ymil =  GetPinOffsetRow(iPinIndex); return;
 		}
 	}
-	void SetCompPinOffsets(const size_t& iPinIndex, const int& Xmil, const int& Ymil)
+	void SetCompPinOffsets(size_t iPinIndex, int Xmil, int Ymil)
 	{
 		switch ( GetDirection() )
 		{
@@ -439,7 +459,7 @@ public:
 			default:	SetPinOffsetCol(iPinIndex,  Xmil);	SetPinOffsetRow(iPinIndex,  Ymil); return;
 		}
 	}
-	void IncCompPinOffsets(const size_t& iPinIndex, const int& dX, const int& dY)
+	void IncCompPinOffsets(size_t iPinIndex, int dX, int dY)
 	{
 		if ( dX == 0 && dY == 0 )
 			return SetCompPinOffsets(iPinIndex, 0, 0);	// Reset
@@ -470,7 +490,7 @@ public:
 		}
 		Xmil = Ymil = 0;	// Default to 0 offsets
 	}
-	const CompElement*	GetCompElement(const int& compRow, const int& compCol) const
+	const CompElement*	GetCompElement(int compRow, int compCol) const
 	{
 		return FootPrint::Get(0, compRow, compCol, GetDirection());
 	}
@@ -479,7 +499,7 @@ public:
 		for (const auto& i : m_nodeIdPins) if ( i == nodeId ) return true;
 		return false;
 	}
-	void Rotate(const bool& bClockWise)
+	void Rotate(bool bClockWise)
 	{
 		switch( GetDirection() )	// Component direction: 'W','E','N','S'
 		{
@@ -501,6 +521,7 @@ public:
 	}
 	bool GetAllowCustomPads() const	// true ==> allow custom pad and hole widths
 	{
+		if ( GetIsSOIC() ) return false;
 		switch( GetType()  )
 		{
 			case COMP::TRACKS:	return false;
@@ -508,7 +529,7 @@ public:
 			default:			return GetNumPins() > 0;
 		}
 	}
-	bool CanStretch(const bool& bGrow) const
+	bool CanStretch(bool bGrow) const
 	{
 		if ( !FootPrint::CanStretch(bGrow) ) return false;
 		switch( GetType() )
@@ -524,7 +545,7 @@ public:
 			default:					return true;
 		}
 	}
-	void Stretch(const bool& bGrow, const bool& bUsePCBshapes = false)
+	void Stretch(bool bGrow, bool bUsePCBshapes = false)
 	{
 		FootPrint::Stretch(bGrow);
 		SetDefaultShapes(bUsePCBshapes);	// Rebuild the shapes list
@@ -542,7 +563,7 @@ public:
 			default:					return;
 		}
 	}
-	void StretchWidth(const bool& bGrow, const bool& bUsePCBshapes = false)
+	void StretchWidth(bool bGrow, bool bUsePCBshapes = false)
 	{
 		FootPrint::StretchWidth(bGrow);
 		SetDefaultShapes(bUsePCBshapes);	// Rebuild the shapes list
@@ -630,7 +651,7 @@ public:
 		if ( GetImportStr().empty() || bForce || GetType() != COMP::CUSTOM )
 			SetImportStr( CompTypes::GetDefaultImportStr( GetType() ) );
 	}
-	void SetDefaultShapes(const bool& bUsePCBshapes = false);
+	void SetDefaultShapes(bool bUsePCBshapes = false);
 	void SetDefaultColor();
 	// Persist interface functions
 	virtual void Load(DataStream& inStream) override
@@ -704,6 +725,10 @@ public:
 		}
 		if ( inStream.GetVersion() < VRT_VERSION_31 )
 			HandleLegacyLabelOffsets();
+
+		if ( inStream.GetVersion() < VRT_VERSION_55 )
+			SetupOccupanciesTH();
+
 		// Try to fix any missing definitions
 		SetDefaultPinFlags();
 		SetDefaultStrings(false);	// false ==> only set empty (m_prefixStr, m_guiStr, m_importStr)
@@ -773,6 +798,6 @@ private:
 	int							m_col;				// Board col for top-left element of footprint
 	int							m_iLabelOffsetRow;	// Label offset in units of 1/16 of a grid square
 	int							m_iLabelOffsetCol;	// Label offset in units of 1/16 of a grid square
-	char						m_direction;		// Component orienation:  'W', 'E', 'N', 'S'
+	char						m_direction;		// Component orientation:  'W', 'E', 'N', 'S'
 	bool						m_bIsPlaced;		// true ==> placed on board,  false ==> floating
 };
