@@ -34,10 +34,10 @@
 // each Element knows what it can be connected to without having
 // to go through the parent Grid object.
 
-static const int			TRAX_COMPID	= -2;		// The component manager member m_trax has this ID
-static const int			BAD_COMPID	= -1;		// Invalid component ID
-static const unsigned int	BAD_ROUTEID	= UINT_MAX;	// Invalid route (i.e. track section) ID
-static const unsigned int	BAD_MH		= UINT_MAX;	// "Infinite" MH distance
+Q_DECL_CONSTEXPR static const int			TRAX_COMPID	= -2;		// The component manager member m_trax has this ID
+Q_DECL_CONSTEXPR static const int			BAD_COMPID	= -1;		// Invalid component ID
+Q_DECL_CONSTEXPR static const unsigned int	BAD_ROUTEID	= UINT_MAX;	// Invalid route (i.e. track section) ID
+Q_DECL_CONSTEXPR static const unsigned int	BAD_MH		= UINT_MAX;	// "Infinite" MH distance
 
 class Element;
 
@@ -55,20 +55,31 @@ typedef std::list<ElementInt> WIRELIST;	// Helper for chains of wires
 class Element : public Pin, public TrackElement
 {
 public:
-	virtual void		 SetPinIndex(const size_t& i)	{ auto pBase = GetBase();		return pBase == this ? Pin::SetPinIndex(i)		 : pBase->SetPinIndex(i); }
-	virtual void		 SetSurface(const uchar& c)		{ auto pBase = GetBase();		return pBase == this ? Pin::SetSurface(c)		 : pBase->SetSurface(c); }
-	virtual void		 SetHoleUse(const uchar& c)		{ auto pBase = GetBase();		return pBase == this ? Pin::SetHoleUse(c)		 : pBase->SetHoleUse(c); }
-	virtual void		 SetOccupancy(const bool& bWire){ auto pBase = GetBase();		return pBase == this ? Pin::SetOccupancy(bWire)	 : pBase->SetOccupancy(bWire); }
-	virtual size_t		 GetPinIndex() const			{ auto pBase = GetBaseConst();	return pBase == this ? Pin::GetPinIndex()		 : pBase->GetPinIndex(); }
-	virtual const uchar& GetSurface() const				{ auto pBase = GetBaseConst();	return pBase == this ? Pin::GetSurface()		 : pBase->GetSurface(); }
-	virtual const uchar& GetHoleUse() const				{ auto pBase = GetBaseConst();	return pBase == this ? Pin::GetHoleUse()		 : pBase->GetHoleUse(); }
-	virtual bool		 GetIsPin() const				{ auto pBase = GetBaseConst();	return pBase == this ? Pin::GetIsPin()			 : pBase->GetIsPin(); }
-	virtual bool		 GetIsHole() const				{ auto pBase = GetBaseConst();	return pBase == this ? Pin::GetIsHole()			 : pBase->GetIsHole(); }
-	virtual const int&	 GetNodeId() const
+	// Debug methods that avoid tunneling through layers
+	size_t	GetPinIndexRaw() const	{ return Pin::GetPinIndex(); }
+	size_t	GetPinIndex2Raw() const	{ return ( m_pinChar2 == BAD_PINCHAR ) ? BAD_PININDEX : m_pinChar2; }
+	int		GetCompIdRaw() const	{ return m_compId; }
+	int		GetCompId2Raw() const	{ return m_compId2; }
+	uchar	GetSurfaceRaw() const	{ return Pin::GetSurface(); }
+	uchar	GetHoleUseRaw() const	{ return Pin::GetHoleUse(); }
+	uchar	GetSoicCharRaw() const	{ return Pin::GetSoicChar(); }
+	int		GetNodeIdRaw() const	{ return TrackElement::GetNodeId(); }
+	void		 SetSurface(uchar c)		{ return GetBase()->Pin::SetSurface(c); }
+	void		 SetHoleUse(uchar c)		{ return GetBase()->Pin::SetHoleUse(c); }
+	void		 SetSoicChar(uchar c)		{ return GetBase()->Pin::SetSoicChar(c); }
+	void		 SetOccupancyTH(bool bWire)	{ return GetBase()->Pin::SetOccupancyTH(bWire); }
+	const uchar& GetSurface() const			{ return GetBaseConst()->Pin::GetSurface(); }
+	const uchar& GetHoleUse() const			{ return GetBaseConst()->Pin::GetHoleUse(); }
+	bool		 GetIsHole() const			{ return GetBaseConst()->Pin::GetIsHole(); }
+	uchar		 GetSoicChar() const		{ return GetBaseConst()->Pin::GetSoicChar(); }
+	bool		 GetIsBotLyr() const		{ const Element* const p = GetNbr(NBR_X);	return p == nullptr || p > this; }
+	bool		 GetIsTopLyr() const		{ const Element* const p = GetNbr(NBR_X);	return p != nullptr && p < this; }
+	bool		 GetSoicProtected() const	{ return GetSoicChar() & ( GetIsTopLyr() ? SOIC_TRACKS_TOP : SOIC_TRACKS_BOT ); }
+	const int&	 GetNodeId() const
 	{
-		auto pBase = GetBaseConst();	return ( pBase != this && GetHasPin() ) ? pBase->GetNodeId() : TrackElement::GetNodeId();
+		auto pBase = GetBaseConst();	return ( pBase != this && GetHasPinTH() ) ? pBase->TrackElement::GetNodeId() : TrackElement::GetNodeId();
 	}
-	virtual void		 SetNodeId(const int& i)	// Only called via the parent board method Board::SetNodeId()
+	void		 SetNodeId(int i)	// Only called via the parent board method Board::SetNodeId()
 	{
 		TrackElement::SetNodeId(i);
 
@@ -83,7 +94,7 @@ public:
 		GetNbr(NBR_L)->UpdateUsed(NBR_RT);	GetNbr(NBR_R)->UpdateUsed(NBR_LT);	// LTX, RTX
 		GetNbr(NBR_L)->UpdateUsed(NBR_RB);	GetNbr(NBR_R)->UpdateUsed(NBR_LB);	// LBX, RBX
 
-		auto pBase = GetBase();	if ( pBase != this && GetHasPin() ) pBase->SetNodeId(i);
+		auto pBase = GetBase();	if ( pBase != this && GetHasPinTH() ) pBase->SetNodeId(i);
 	}
 
 	Element() : Pin(), TrackElement() { ZeroConnectionPointers(); }
@@ -127,28 +138,18 @@ public:
 	{
 		return !(*this == o);
 	}
-	void SetIsMark(const bool& b)		{ auto pBase = GetBase(); if ( pBase == this ) m_bIsMark = b; else pBase->SetIsMark(b); }
-	void SetCompId(const int& i)		{ auto pBase = GetBase(); if ( pBase == this ) m_compId  = i; else pBase->SetCompId(i); }
-	void SetCompId2(const int& i)		{ auto pBase = GetBase(); if ( pBase == this ) m_compId2 = i; else pBase->SetCompId2(i); }
-	void SetPinIndex2(const size_t& i)
-	{
-		auto pBase = GetBase();
-		if ( pBase == this )
-			m_pinChar2 = ( i >= BAD_PINCHAR ) ? BAD_PINCHAR : static_cast<uchar> (i);
-		else
-			pBase->SetPinIndex2(i);
-	}
-	void SetSolderR(const bool& b)		{ auto pBase = GetBase(); if ( pBase == this ) m_bSolderR = b; else pBase->SetSolderR(b); }
-	void SetIsVia(const bool& b)		{ auto pBase = GetBase(); if ( pBase == this ) m_bIsVia	  = b; else pBase->SetIsVia(b); }
-	void SetRoutable(const int& i)		{ m_iRoutable	= i; }
-	void SetRouteId(const unsigned int& i)	{ m_routeId = i; }
+	void SetIsMark(bool b)			{ GetBase()->m_bIsMark = b; }
+	void SetSolderR(bool b)			{ GetBase()->m_bSolderR = b; }
+	void SetIsVia(bool b)			{ GetBase()->m_bIsVia	= b; }
+	void SetRoutable(int i)			{ m_iRoutable	= i; }
+	void SetRouteId(unsigned int i)	{ m_routeId = i; }
 	void ResetMH()
 	{
 		m_routeId	= BAD_ROUTEID;	// Wipe RouteId
 		m_MH		= BAD_MH;		// Set "infinite" MH distance.
 		m_maxMH		= 0;			// Zero max MH parameter
 	}
-	void UpdateMH(const unsigned int& iRouteID, const unsigned int& iMH, unsigned int& iMaxMH)
+	void UpdateMH(unsigned int iRouteID, unsigned int iMH, unsigned int& iMaxMH)
 	{
 		assert( m_MH == BAD_MH );	// Should only ever write the MH once
 		iMaxMH		= std::max(iMaxMH, iMH);	// Update iMaxMH for output before storing it
@@ -156,44 +157,63 @@ public:
 		m_MH		= iMH;
 		m_maxMH		= iMaxMH;
 	}
-	void SetNbr(const int& iNbr, Element* p)	{ m_pNbr[iNbr]	= p; }
+	void SetNbr(int iNbr, Element* p)	{ m_pNbr[iNbr]	= p; }
 	void ClearWires()			{ SetW(0, nullptr);	SetW(1, nullptr); }
 	bool GetHasWire() const		{ return GetW(0) != nullptr || GetW(1) != nullptr; }
-	int  GetNumWires() const
+	int GetNumWires() const
 	{
-		int i(0);
-		if ( GetW(0) != nullptr ) i++;
-		if ( GetW(1) != nullptr ) i++;
-		return i;
+		int nCount(0);
+		for (int iSlot = 0; iSlot < 2; iSlot++) if ( GetW(iSlot) != nullptr ) nCount++;
+		return nCount;
 	}
-	int  GetUsedSlot() const
+	int GetNumUsedSlots() const
 	{
-		return	( GetCompId()  != BAD_COMPID ) ? 0 :
-				( GetCompId2() != BAD_COMPID ) ? 1 : -1;
+		int nCount(0);
+		for (int iSlot = 0; iSlot < 2; iSlot++) if ( GetSlotCompId(iSlot) != BAD_COMPID ) nCount++;
+		return nCount;
 	}
-	int  GetFreeSlot() const
+	int GetFirstUsedSlot() const
 	{
-		return	( GetCompId()  == BAD_COMPID ) ? 0 :
-				( GetCompId2() == BAD_COMPID ) ? 1 : -1;
-	}
-	int  GetSlotFromCompId(const int& compId)
-	{
-		assert( GetCompId() != GetCompId2() || GetCompId() == BAD_COMPID );
-		if ( compId == GetCompId()  ) return 0;
-		if ( compId == GetCompId2() ) return 1;
-		assert(0);	// Error
+		for (int iSlot = 0; iSlot < 2; iSlot++) if ( GetSlotCompId(iSlot) != BAD_COMPID ) return iSlot;
+		assert(0);
 		return -1;
 	}
-	void SetSlotInfo(const int& iSlot, const size_t& pinIndex, const int& compId)
+	int GetFreeSlot() const
+	{
+		for (int iSlot = 0; iSlot < 2; iSlot++) if ( GetSlotCompId(iSlot) == BAD_COMPID ) return iSlot;
+		assert(0);
+		return -1;
+	}
+	int GetSlotFromCompId(int compId) const
+	{
+		assert( GetCompId() != GetCompId2() || GetCompId() == BAD_COMPID );
+		for (int iSlot = 0; iSlot < 2; iSlot++) if ( GetSlotCompId(iSlot) == compId ) return iSlot;
+		return -1;
+	}
+	void SetSlotInfo(int iSlot, size_t pinIndex, int compId)
+	{
+		SetSlotPinIndex(iSlot, pinIndex);
+		SetSlotCompId(iSlot, compId);
+	}
+	void SetSlotPinIndex(int iSlot, size_t pinIndex)
 	{
 		switch( iSlot )
 		{
-			case 0:		SetPinIndex(pinIndex);	SetCompId(compId);	return;
-			case 1:		SetPinIndex2(pinIndex);	SetCompId2(compId);	return;
+			case 0:		return SetPinIndex(pinIndex);
+			case 1:		return SetPinIndex2(pinIndex);
 			default:	assert(0);
 		}
 	}
-	void GetSlotInfo(const int& iSlot, size_t& pinIndex, int& compId) const
+	void SetSlotCompId(int iSlot, int compId)
+	{
+		switch( iSlot )
+		{
+			case 0:		return SetCompId(compId);
+			case 1:		return SetCompId2(compId);
+			default:	assert(0);
+		}
+	}
+	void GetSlotInfo(int iSlot, size_t& pinIndex, int& compId) const
 	{
 		switch( iSlot )
 		{
@@ -202,46 +222,64 @@ public:
 			default:	pinIndex = BAD_PININDEX;	compId = BAD_COMPID;	assert(0);
 		}
 	}
+	size_t GetSlotPinIndex(int iSlot) const
+	{
+		switch( iSlot )
+		{
+			case 0:		return GetPinIndex();
+			case 1:		return GetPinIndex2();
+			default:	assert(0);	return BAD_PININDEX;
+		}
+	}
+	int GetSlotCompId(int iSlot) const
+	{
+		switch( iSlot )
+		{
+			case 0:		return GetCompId();
+			case 1:		return GetCompId2();
+			default:	assert(0);	return BAD_COMPID;
+		}
+	}
 	bool GetWireExists(const Element* p) const
 	{
 		return p != nullptr && ( GetW(0) == p || GetW(1) == p );
 	}
-	bool GetCompExists(const int& compId) const
+	bool GetCompExists(int compId) const
 	{
 		return compId != BAD_COMPID && ( GetCompId() == compId || GetCompId2() == compId );
 	}
-	void SetW(const int& iSlot, Element* p)
+	void SetW(int iSlot, Element* p)
 	{
 		assert( !GetWireExists(p) );	// No duplicates allowed
 		assert( iSlot == 0 || iSlot == 1 );
-		auto pBase = GetBase();	if ( pBase == this ) m_pW[iSlot] = p; else pBase->SetW(iSlot, p);
+		GetBase()->m_pW[iSlot] = p;
 	}
-	void				SetMH(const unsigned int& iMH)	{ m_MH = iMH; }
-	const bool&			GetIsMark() const				{ auto pBase = GetBaseConst(); return pBase == this ? m_bIsMark		: pBase->GetIsMark(); }
-	const int&			GetCompId() const				{ auto pBase = GetBaseConst(); return pBase == this ? m_compId		: pBase->GetCompId(); }
-	const int&			GetCompId2() const				{ auto pBase = GetBaseConst(); return pBase == this ? m_compId2		: pBase->GetCompId2(); }
-	const uchar&		GetPinChar2() const				{ auto pBase = GetBaseConst(); return pBase == this ? m_pinChar2	: pBase->GetPinChar2(); }
-	int					GetNumCompIds() const			{ int i(0); if ( GetCompId() != BAD_COMPID ) i++; if ( GetCompId2() != BAD_COMPID ) i++; return i; }
-	bool				GetHasComp() const				{ return GetCompId() != BAD_COMPID || GetCompId2() != BAD_COMPID; }
-	bool				GetHasPin() const				{ return GetIsPin() || GetPinChar2() != BAD_PINCHAR; }
-	size_t				GetPinIndex2() const			{ return ( GetPinChar2() == BAD_PINCHAR ) ? BAD_PININDEX : GetPinChar2(); }
-	const bool&			GetSolderR() const				{ auto pBase = GetBaseConst(); return pBase == this ? m_bSolderR	: pBase->GetSolderR(); }
-	const bool&			GetIsVia() const				{ auto pBase = GetBaseConst(); return pBase == this ? m_bIsVia		: pBase->GetIsVia(); }
-	const int&			GetRoutable() const				{ return m_iRoutable; }
-	const unsigned int&	GetRouteId() const				{ return m_routeId; }
-	const unsigned int&	GetMH() const					{ return m_MH; }
-	const unsigned int&	GetMaxMH() const				{ return m_maxMH; }
-	Element*			GetNbr(const int& iNbr) const	{ return m_pNbr[iNbr]; }
-	Element*			GetW(const int& i) const		{ auto pBase = GetBaseConst(); return pBase == this ? m_pW[i]		: pBase->GetW(i); }
-	bool				IsLayer0() const 				{ return GetBaseConst() == this; }
-
+	void				SetMH(unsigned int iMH)	{ m_MH = iMH; }
+	const bool&			GetIsMark() const		{ return GetBaseConst()->m_bIsMark; }
+	int					GetNumCompIds() const	{ int i(0); if ( GetCompId() != BAD_COMPID ) i++; if ( GetCompId2() != BAD_COMPID ) i++; return i; }
+	bool				GetHasComp() const		{ return GetCompId() != BAD_COMPID || GetCompId2() != BAD_COMPID; }
+	bool				GetHasPinLegacy() const	{ return GetIsPin() || GetIsPin2(); }	// Only kept for legacy purposes (e.g. old VRTs don't have SOIC codes)
+	bool				GetHasPin() const		{ return GetSoicChar() & (SOIC_PAD|SOIC_THL); }	// true ==> Have TH pin/SOIC pad on either layer
+	bool				GetHasPinTH() const		{ return GetSoicChar() & SOIC_THL; }			// true ==> Have TH pin on either layer
+	bool				GetHasPinSOIC() const	{ return GetSoicChar() & SOIC_PAD; }			// true ==> Have SOIC pad on either layer
+	bool				GetLyrHasPin() const	{ return GetHasPinTH() || ( GetHasPinSOIC() && GetIsTopLyr() ); }
+	const bool&			GetSolderR() const		{ return GetBaseConst()->m_bSolderR; }
+	const bool&			GetIsVia() const		{ return GetBaseConst()->m_bIsVia; }
+	const int&			GetRoutable() const		{ return m_iRoutable; }
+	const unsigned int&	GetRouteId() const		{ return m_routeId; }
+	const unsigned int&	GetMH() const			{ return m_MH; }
+	const unsigned int&	GetMaxMH() const		{ return m_maxMH; }
+	Element*			GetNbr(int iNbr) const	{ return m_pNbr[iNbr]; }
+	Element*			GetW(int iSlot) const	{ return GetBaseConst()->m_pW[iSlot]; }
+	bool				GetPinSupportsOffsetPads() const	{ return GetHasPinTH() && !GetHasWire(); }
+	bool				GetPinSupportsLayerPref() const		{ return GetHasPinTH() && !GetHasWire(); }
 	// Helpers
-	bool HaveNoBlankPins(const int& iNbr) const
+	bool HaveNoBlankPins(int iNbr) const
 	{
-		const Element* pLyr	= GetBaseConst();	// Use layer 0 for checking pins
+		const Element* pLyr = this;
 		const Element* pNbr = pLyr->GetNbr(iNbr);
-		return	( !pLyr->GetHasPin() || pLyr->GetNodeId() != BAD_NODEID || pLyr->GetHasWire() ) &&	// Only allow routing FROM blank pins if they are on wires
-				( !pNbr->GetHasPin() || pNbr->GetNodeId() != BAD_NODEID || pNbr->GetHasWire() );	// Only allow routing  TO  blank pins if they are on wires
+		return	( !pLyr->GetLyrHasPin() || pLyr->GetNodeId() != BAD_NODEID || pLyr->GetHasWire() ) &&	// Only allow routing FROM blank pins if they are on wires
+				( !pNbr->GetLyrHasPin() || pNbr->GetNodeId() != BAD_NODEID || pNbr->GetHasWire() );	// Only allow routing  TO  blank pins if they are on wires
 	}
 	void GetWireList(WIRELIST& wireList) const
 	{
@@ -249,7 +287,7 @@ public:
 		return UpdateWireList(wireList, 0);
 	}
 	// Connectivity helpers
-	void UpdateUsed(const int& iNbr)
+	void UpdateUsed(int iNbr)
 	{
 		const bool bUsed = GetNodeId() != BAD_NODEID
 						&& GetNodeId() == GetNbr(iNbr)->GetNodeId()
@@ -257,7 +295,7 @@ public:
 		SetUsed(iNbr, bUsed);
 		m_pNbr[iNbr]->SetUsed(Opposite(iNbr), bUsed);	// Keep consistent with nbr
 	}
-	void ToggleUsed(const int& iNbr)
+	void ToggleUsed(int iNbr)
 	{
 		ToggleCodeBit(iNbr, m_iCode);
 		ToggleCodeBit(Opposite(iNbr), m_pNbr[iNbr]->m_iCode);	// Keep consistent with nbr
@@ -276,10 +314,9 @@ public:
 		// Take "this" to be the bottom right element in group of 4 squares
 		// "LT"  is the diagonal from "this" to m_pLT
 		// "LTX" is the diagonal that cuts across it (from m_pL to m_pT)
-		const bool bCanSwap = GetNodeId() == GetNbr(NBR_LT)->GetNodeId()				&&	// LT:  "this" and LT must have same nodeId
-							  GetNbr(NBR_L)->GetNodeId() == GetNbr(NBR_T)->GetNodeId()	&&	// LTX: L and T must have same nodeId
-							  GetNbr(NBR_L)->IsClash( GetNodeId() );						// L and "this" must have clashing nodeIds
-		return bCanSwap;
+		return	GetNodeId() == GetNbr(NBR_LT)->GetNodeId()					&&	// LT:  "this" and LT must have same nodeId
+				GetNbr(NBR_L)->GetNodeId() == GetNbr(NBR_T)->GetNodeId()	&&	// LTX: L and T must have same nodeId
+				GetNbr(NBR_L)->IsClash( GetNodeId() );							// L and "this" must have clashing nodeIds
 	}
 	bool SwapDiagLinks()
 	{
@@ -290,14 +327,21 @@ public:
 		GetNbr(NBR_L)->ToggleUsed(NBR_RT);
 		return true;
 	}
+	bool IsDiagNbr(const Element* p) const
+	{
+		assert( p != nullptr );	// Sanity check
+		for (int iNbr = 1; iNbr < 8; iNbr += 2)
+			if ( GetNbr(iNbr) == p ) return true;
+		return false;
+	}
 	bool IsNbr(const Element* p) const
 	{
 		assert( p != nullptr );	// Sanity check
 		for (int iNbr = 0; iNbr < NUM_NBRS; iNbr++)
 			if ( GetNbr(iNbr) == p ) return true;
 		return false;
-	}			
-	bool IsUselessWire(const int& iNbr, const int& nodeId) const	// Helper: true ==> painting nbr with nodeId is wasteful
+	}
+	bool IsUselessWire(int iNbr, int nodeId) const	// Helper: true ==> painting nbr with nodeId is wasteful
 	{
 		const Element* pWA = GetNbr(iNbr);
 		if ( pWA->GetHasWire() )
@@ -311,26 +355,36 @@ public:
 			for (int iNbr = 0; iNbr < NUM_NBRS; iNbr++)
 			{
 				const Element* p = pWA->GetNbr(iNbr);
-				if ( p == nullptr ) continue;
-				if ( p->GetNodeId() != nodeId ) continue;
+				if ( p == nullptr || p->GetNodeId() != nodeId ) continue;
 				if ( pWB0 != nullptr && pWB0->IsNbr(p) ) return true;
 				if ( pWB1 != nullptr && pWB1->IsNbr(p) ) return true;
 			}
 		}
 		return false;
 	}
-	bool IsBlocked(const int& iNbr, const int& nodeId) const	// Helper: true ==> assiging nodeId to "this" blocks the iNbr direction
+	bool IsBlocked(int iNbr, int nodeId) const	// Helper: true ==> assiging nodeId to "this" blocks the iNbr direction
 	{
+		assert(nodeId != BAD_NODEID);
 		if ( !ReadCodeBit(iNbr, GetRoutable() ) ) return true;	// Block toroidal connections at board edges
-		if ( GetNbr(iNbr)->IsClash(nodeId) ) return true;		// Check if nbr has a clashing nodeId assigned to it
-		if ( GetNbr(iNbr)->GetIsHole() ) return true;			// Block connections to holes
+		auto pNbr = GetNbr(iNbr);
+		if ( pNbr->IsClash(nodeId) ) return true;		// Check if nbr has a clashing nodeId assigned to it
+		if ( pNbr->GetIsHole() ) return true;			// Block connections to holes
+		if ( pNbr->GetSoicProtected() ) return true;	// Block connections to SOIC area
+
+		switch( iNbr )	// Block diagonals crossing the SOIC area
+		{
+			case NBR_LT: if ( GetNbr(NBR_L)->GetSoicProtected() || GetNbr(NBR_T)->GetSoicProtected() ) return true;	break;
+			case NBR_RT: if ( GetNbr(NBR_R)->GetSoicProtected() || GetNbr(NBR_T)->GetSoicProtected() ) return true;	break;
+			case NBR_LB: if ( GetNbr(NBR_L)->GetSoicProtected() || GetNbr(NBR_B)->GetSoicProtected() ) return true;	break;
+			case NBR_RB: if ( GetNbr(NBR_R)->GetSoicProtected() || GetNbr(NBR_B)->GetSoicProtected() ) return true;	break;
+		}
 
 		switch( iNbr )	// Then do additional checks for competing diagonals
 		{
 			case NBR_LT: return GetNbr(NBR_L)->IsClash(nodeId) && GetNbr(NBR_L)->GetUsed(NBR_RT);
 			case NBR_RT: return GetNbr(NBR_R)->IsClash(nodeId) && GetNbr(NBR_R)->GetUsed(NBR_LT);
-			case NBR_LB: return GetNbr(NBR_B)->IsClash(nodeId) && GetNbr(NBR_B)->GetUsed(NBR_LT);
-			case NBR_RB: return GetNbr(NBR_B)->IsClash(nodeId) && GetNbr(NBR_B)->GetUsed(NBR_RT);
+			case NBR_LB: return GetNbr(NBR_L)->IsClash(nodeId) && GetNbr(NBR_L)->GetUsed(NBR_RB);
+			case NBR_RB: return GetNbr(NBR_R)->IsClash(nodeId) && GetNbr(NBR_R)->GetUsed(NBR_LB);
 			default:	 return false;
 		}
 	}
@@ -345,6 +399,28 @@ public:
 		if ( m_compId2 != BAD_COMPID && m_compId2 != TRAX_COMPID )
 			o.deltaCompId = std::max(o.deltaCompId,  m_compId2 + 1);
 	}
+	void FixCorruption()
+	{
+		const bool bOK = GetCompId()	== BAD_COMPID	&&
+						 GetCompId2()	== BAD_COMPID	&&
+						 GetPinIndex()	== BAD_PININDEX	&&
+						 GetPinIndex2()	== BAD_PININDEX	&&
+						 GetSurface()	== SURFACE_FREE	&&
+						 GetHoleUse()	== HOLE_FREE	&&
+						 GetSoicChar()	== SOIC_FREE	&&
+						 GetIsMark()	== false;
+		if ( !bOK )
+		{
+			SetCompId(BAD_COMPID);
+			SetCompId2(BAD_COMPID);
+			SetPinIndex(BAD_PININDEX);
+			SetPinIndex2(BAD_PININDEX);
+			SetSurface(SURFACE_FREE);
+			SetHoleUse(HOLE_FREE);
+			SetSoicChar(SOIC_FREE);
+			SetIsMark(false);
+		}
+	}
 	virtual void ApplyMergeOffsets(const MergeOffsets& o) override
 	{
 		Pin::ApplyMergeOffsets(o);	// Does nothing
@@ -399,8 +475,19 @@ public:
 		outStream.Save(m_pinChar2);		// Added in VRT_VERSION_27
 	}
 private:
-	Element*		GetBase()				{ Element*		 pBase = GetNbr(NBR_X);	return pBase == nullptr || pBase > this ? this : pBase; }
-	const Element*	GetBaseConst() const	{ const Element* pBase = GetNbr(NBR_X);	return pBase == nullptr || pBase > this ? this : pBase; }
+	Element*		GetBase()				{ Element*		 p = GetNbr(NBR_X);	return p == nullptr || p > this ? this : p; }
+	const Element*	GetBaseConst() const	{ const Element* p = GetNbr(NBR_X);	return p == nullptr || p > this ? this : p; }
+	const int&		GetCompId() const		{ return GetBaseConst()->m_compId; }
+	const int&		GetCompId2() const		{ return GetBaseConst()->m_compId2; }
+	const uchar&	GetPinChar2() const		{ return GetBaseConst()->m_pinChar2; }
+	bool			GetIsPin() const		{ return GetBaseConst()->Pin::GetIsPin(); }
+	bool			GetIsPin2() const		{ return GetPinChar2() != BAD_PINCHAR; }
+	size_t			GetPinIndex() const		{ return GetBaseConst()->Pin::GetPinIndex(); }
+	size_t			GetPinIndex2() const	{ return ( GetPinChar2() == BAD_PINCHAR ) ? BAD_PININDEX : GetPinChar2(); }
+	void			SetCompId(int i)		{ GetBase()->m_compId  = i; }
+	void			SetCompId2(int i)		{ GetBase()->m_compId2 = i; }
+	void			SetPinIndex(size_t i)	{ return GetBase()->Pin::SetPinIndex(i); }
+	void			SetPinIndex2(size_t i)	{ GetBase()->m_pinChar2 = ( i >= BAD_PINCHAR ) ? BAD_PINCHAR : static_cast<uchar> (i); }
 	bool WireListHelper(WIRELIST& wireList, const Element* p, unsigned int iStep) const
 	{
 		for (auto& o : wireList)
@@ -414,8 +501,8 @@ private:
 	void UpdateWireList(WIRELIST& wireList, unsigned int iStep) const
 	{
 		WireListHelper(wireList, this, iStep);
-		bool bOK_0	= GetW(0) != nullptr && WireListHelper(wireList, GetW(0), iStep + 1);
-		bool bOK_1	= GetW(1) != nullptr && WireListHelper(wireList, GetW(1), iStep + 1);
+		const bool bOK_0 = GetW(0) != nullptr && WireListHelper(wireList, GetW(0), iStep + 1);
+		const bool bOK_1 = GetW(1) != nullptr && WireListHelper(wireList, GetW(1), iStep + 1);
 
 		if ( bOK_0 ) GetW(0)->UpdateWireList(wireList, iStep + 1);
 		if ( bOK_1 ) GetW(1)->UpdateWireList(wireList, iStep + 1);
