@@ -21,7 +21,7 @@
 #include "CompManager.h"
 #include "RectManager.h"
 
-void FootPrint::BuildDefault(const COMP& type)
+void FootPrint::BuildDefault(COMP type)
 {
 	assert(type != COMP::INVALID);
 
@@ -48,14 +48,19 @@ void FootPrint::BuildDefault(const COMP& type)
 		{
 			case '.':	p->SetSurface(SURFACE_FREE);	break;
 			case '-':	p->SetSurface(SURFACE_GAP);		break;
+			case '*':	p->SetSurface(SURFACE_FULL);	p->SetSoicChar(SOIC_TRACKS_TOP);	break;
 			default:	p->SetSurface(bPlug ? SURFACE_PLUG : SURFACE_FULL);
 		}
 		p->SetPinIndex( GetPinIndexFromLegacyPinChar(static_cast<uchar>(pinChar)) );
 	}
-	SetupOccupancies();	// Setup hole use (and for wires setup surface use too)
+	const bool bIsSOIC = CompTypes::GetIsSOIC(m_type);
+	if ( bIsSOIC )
+		SetupOccupanciesSOIC();	// Setup hole use, and SOIC info
+	else
+		SetupOccupanciesTH();	// Setup hole use (and for wires setup surface use too), and SOIC info
 }
 
-void FootPrint::BuildTrax(CompManager* pCompMgr, const RectManager& rectMgr, const ElementGrid& o, const int& nLyr, const int& nRowMin, const int& nRowMax, const int& nColMin, const int& nColMax)
+void FootPrint::BuildTrax(CompManager* pCompMgr, const RectManager& rectMgr, const ElementGrid& o, int nLyr, int nRowMin, int nRowMax, int nColMin, int nColMax)
 {
 	// This method is for building a "tracks" component
 	SetType(COMP::TRACKS);
@@ -81,7 +86,7 @@ void FootPrint::BuildTrax(CompManager* pCompMgr, const RectManager& rectMgr, con
 
 			int iNodeId = pSource->GetNodeId();
 
-			if ( pSource->GetHasPin() )	// For pins/wires, let the track contain the origId before the part was placed
+			if ( pSource->GetLyrHasPin() )	// For pins/wires, let the track contain the origId before the part was placed
 			{
 				for (int iSlot = 0; iSlot < 2; iSlot++)
 				{
