@@ -17,7 +17,6 @@
 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "VeroRouteAndroid.h"
 #include "Version.h"
 #include "Board.h"
 #include "GWriter.h"
@@ -26,8 +25,9 @@
 #include <QTimeZone>
 #include <QtGlobal>
 
-static const bool	FULL_LINE	= false;	// Set to true to force each Gerber line to be written in long format
-static const bool	XNC_FORMAT	= true;		// true ==> XNC Format / Excellon Format 2.		false ==> Excellon Format 1.
+Q_DECL_CONSTEXPR static const bool FULL_LINE		= false;	// Set to true to force each Gerber line to be written in long format
+Q_DECL_CONSTEXPR static const bool XNC_FORMAT		= true;		// true ==> XNC Format,  false ==> Excellon Format 1 or 2
+Q_DECL_CONSTEXPR static const bool EXCELLON2_FORMAT	= true;		// true ==> Excellon Format 2,  false ==> Excellon Format 1.  Ignored if XNC_FORMAT.
 
 // Wrapper for a stream to a Gerber file
 void GStream::Clear()
@@ -49,13 +49,15 @@ void GStream::Close()
 	if ( m_file.isOpen() )
 		m_file.close();
 }
-bool GStream::Open(const QString& fileName, const GFILE& eType, const bool& bMetric, const Board& board, const bool& bVias, const bool& bConfirmEachFile)
+bool GStream::Open(const QString& fileName, GFILE eType, bool bMetric, const Board& board, bool bVias, bool bSOIC, bool bSOIC16, bool bConfirmEachFile)
 {
 	Clear();
 	m_eType		= eType;
 	m_bMetric	= bMetric;
 	m_pBoard	= &board;
 	m_bVias		= bVias;
+	m_bSOIC		= bSOIC;
+	m_bSOIC16	= bSOIC16;
 
 	QString str(fileName);
 	QString suffix;
@@ -103,7 +105,7 @@ void GStream::WriteHeader(const QString& UTC)	// Write header for current stream
 	QString	strLayer	= "Layer: ";
 	QString	strProgram	= "VeroRoute V" + QString(szVEROROUTE_VERSION);
 	QString	strUTC		= QString::fromStdString( UTC.toStdString() );
-	QString	strGen		= QString("Gerber Generator version 0.9");
+	QString	strGen		= QString("Gerber Generator version 1.1");
 	switch(m_eType)
 	{
 		case GFILE::GKO: strLayer += "BoardOutline";			break;
@@ -125,16 +127,20 @@ void GStream::WriteHeader(const QString& UTC)	// Write header for current stream
 		m_os << "M48";	EndLine();	// M48 is start of header
 		if ( m_bMetric )
 		{
-			m_os << "METRIC,LZ,0000.000000";	EndLine();	// Millimetres.  Leading zeros INCLUDED.  4 integer and 6 decimal
+			m_os << "METRIC";		// Millimetres			// Excellon only lists 3.2, 4.2, or 3.3 metric formats (so 4.6 format can only work for XNC format) !!!
+			if ( !XNC_FORMAT ) m_os << ",LZ,0000.000000";	// Leading zeros INCLUDED.  4 integer and 6 decimal
+			EndLine();
 		}
 		else
 		{
-			m_os << "INCH,LZ,00.0000";			EndLine();	// Inches.  Leading zeros INCLUDED.  2 integer and 4 decimal
+			m_os << "INCH";			// Inches
+			if ( !XNC_FORMAT ) m_os << ",LZ,00.0000";		// Leading zeros INCLUDED.  2 integer and 4 decimal
+			EndLine();
 		}
 		MakeDrills();
-		m_os << "%";	EndLine();							// Rewind Stop.  Often used instead of M95.
-		m_os << ( XNC_FORMAT ? "G05" : "G81" );	EndLine();	// Turn on drill
-		m_os << "G90";	EndLine();							// Absolute mode
+		m_os << "%";	EndLine();							// Rewind Stop.  Often used instead of M95 for end of header.
+		m_os << ( XNC_FORMAT || EXCELLON2_FORMAT ? "G05" : "G81" );	EndLine();	// Turn on drill
+		if ( !XNC_FORMAT ) { m_os << "G90";	EndLine(); }	// Absolute mode
 	}
 	else
 	{
@@ -164,7 +170,7 @@ void GStream::WriteFooter()
 	if ( !m_file.isOpen() ) return;
 	switch( m_eType )
 	{
-		case GFILE::DRL:	m_os << "M30";	EndLine();	return;	// End of program
+		case GFILE::DRL:	m_os << "M30";	EndLine();	return;	// End of file
 		default:			m_os << "M02";	EndLine();	return;	// End of file
 	}
 }
@@ -184,7 +190,7 @@ void GStream::MakeDrills()
 		m_ePenList.push_back( GPenInfo(GPEN::VIA_HLE, viahole, code++, "Via Hole = ") );
 
 	// Write drill list to file
-	const bool bLZ(true);	// Include leading zeros
+	const bool bLZ( !XNC_FORMAT );	// true ==> Include leading zeros
 	for (const auto& o : m_ePenList)
 	{
 		QString codeStr("T");
@@ -216,6 +222,8 @@ void GStream::MakeApertures()	// Make "pens" for current stream
 	const int		msk		= m_pBoard->GetMASK_MIL();
 	const int		slk		= m_pBoard->GetSILK_MIL();
 	const int		gko		= 10;	// Draw border in 10 mil pen
+	const int		trkIC	= m_pBoard->GetTRACK_IC_MIL();
+	const int		minIC	= m_pBoard->GetMIN_IC_MIL();
 
 	// Build aperture list
 	int code = 10;	// Start with aperture D10
@@ -234,6 +242,13 @@ void GStream::MakeApertures()	// Make "pens" for current stream
 				m_ePenList.push_back( GPenInfo(GPEN::TRK, trk, code++, " is for tracks") );
 			if ( true )
 				m_ePenList.push_back( GPenInfo(GPEN::TAG, tag, code++, " is for tracks") );
+			if ( m_eType == GFILE::GTL && m_bSOIC )
+			{
+				m_ePenList.push_back( GPenInfo(GPEN::TRK_IC, trkIC, code++, " is for SOIC tracks") );
+				if ( m_bSOIC16 )
+					m_ePenList.push_back( GPenInfo(GPEN::MIN_IC, minIC, code++, " is for thin SOIC tracks") );
+			}
+			// Ground fill pens ...
 			if ( !m_pBoard->GetGroundFill() ) break;
 			for (const auto& pad : pads)
 				m_ePenList.push_back( GPenInfo(GPEN::PAD_GAP, pad + 2 * gap, code++, " is for separating pads from fill", pad != padDefault) );
@@ -241,6 +256,12 @@ void GStream::MakeApertures()	// Make "pens" for current stream
 				m_ePenList.push_back( GPenInfo(GPEN::VIA_GAP, via + 2 * gap, code++, " is for separating via-pads from fill") );
 			if ( true )
 				m_ePenList.push_back( GPenInfo(GPEN::TRK_GAP, trk + 2 * gap, code++, " is for separating tracks from fill") );
+			if ( m_eType == GFILE::GTL && m_bSOIC )
+			{
+				m_ePenList.push_back( GPenInfo(GPEN::TRK_IC_GAP, trkIC + 2 * gap, code++, " is for separating SOIC tracks from fill") );
+				if ( m_bSOIC16 )
+					m_ePenList.push_back( GPenInfo(GPEN::MIN_IC_GAP, minIC + 2 * gap, code++, " is for separating thin SOIC tracks from fill") );
+			}
 			break;
 		case GFILE::GBS:
 		case GFILE::GTS:
@@ -321,7 +342,7 @@ bool GStream::GetOK() const
 		default:		 return m_pBoard->GetCurrentLayer() == 0 && m_eType == GFILE::DRL;
 	}
 }
-void GStream::SetPolarity(const GPOLARITY& ePolarity, bool bCheckOK)
+void GStream::SetPolarity(GPOLARITY ePolarity, bool bCheckOK)
 {
 	if ( !m_file.isOpen() || m_ePolarity == ePolarity || m_eType == GFILE::DRL ) return;
 	if ( bCheckOK && !GetOK() ) return;
@@ -340,37 +361,37 @@ void GStream::Drill(const QPoint& p)
 	m_os << "Y";  WriteDrillOrdinate( p.y() );
 	QtEndline();;
 }
-void GStream::AddPad(const QPointF& pF, const GPEN& ePen, const int& w)		// Add to m_pads buffer for later writing to file
+void GStream::AddPad(const QPointF& pF, GPEN ePen, int w)	// Add to m_pads buffer for later writing to file
 {
 	if ( !GetOK() ) return;
 	QPoint p;
 	GetQPoint(pF, p);
 	m_pads.push_back( new Curve(p, ePen, w) );
 }
-void GStream::AddViaPad(const QPointF& pF, const GPEN& ePen)	// Add to m_viapads buffer for later writing to file
+void GStream::AddViaPad(const QPointF& pF, GPEN ePen)	// Add to m_viapads buffer for later writing to file
 {
 	if ( !GetOK() ) return;
 	QPoint p;
 	GetQPoint(pF, p);
 	m_viapads.push_back( new Curve(p, ePen) );
 }
-void GStream::AddTrack(const QPolygonF& pF, const GPEN& ePen)	// Add to m_tracks buffer for later writing to file
+void GStream::AddTrack(const QPolygonF& pF, GPEN ePen)	// Add to m_tracks buffer for later writing to file
 {
 	if ( !GetOK() ) return;
 	QPolygon p;
 	GetQPolygon(pF, p);
 	m_tracks.push_back( new Curve(p, ePen) );
 }
-void GStream::AddVariTrack(const QPolygonF& pF, const GPEN& ePenHV, const GPEN& ePen)	// Add to m_tracks buffer for later writing to file
+void GStream::AddVariTrack(const QPolygonF& pF, GPEN ePenHV, GPEN ePen)	// Add to m_tracks buffer for later writing to file
 {
 	if ( !GetOK() ) return;
 	QPolygon p;
 	GetQPolygon(pF, p);
 
 	QPolygon temp;
-	auto A = p.begin();
+	auto A = p.cbegin();
 	auto B = A; ++B;
-	for (; B != p.end(); A++, B++)
+	for (; B != p.cend(); A++, B++)
 	{
 		temp.clear();
 		temp << *A << *B;
@@ -378,7 +399,7 @@ void GStream::AddVariTrack(const QPolygonF& pF, const GPEN& ePenHV, const GPEN&
 		m_tracks.push_back( new Curve(temp, bHV ? ePenHV : ePen) );
 	}
 }
-void GStream::AddLoop(const QPolygonF& pF, const GPEN& ePen)	// Add to m_loops buffer for later writing to file
+void GStream::AddLoop(const QPolygonF& pF, GPEN ePen)	// Add to m_loops buffer for later writing to file
 {
 	if ( !GetOK() ) return;
 	assert( pF.size() >= 3 );
@@ -396,7 +417,7 @@ void GStream::AddRegion(const QPolygonF& pF)	// Add to m_regions buffer for late
 	GetQPolygon(pF, p);
 	m_regions.push_back( new Curve(p, GPEN::NONE) );
 }
-void GStream::AddHole(const QPointF& pF, const GPEN& ePen, const int& w)	// Add to m_holes buffer for later writing to file
+void GStream::AddHole(const QPointF& pF, GPEN ePen, int w)	// Add to m_holes buffer for later writing to file
 {
 	if ( !GetOK() ) return;
 	QPoint p;
@@ -453,7 +474,7 @@ void GStream::OutLine(const Curve& curve, bool bForceClose)	// Outline of a curv
 		Draw(*iter);	// Draw line to next point in curve
 	if ( bForceClose && front != curve.back() ) Draw( front );
 }
-void GStream::SetPen(const GPEN& ePen, const int& w)
+void GStream::SetPen(GPEN ePen, int w)
 {
 	if ( !m_file.isOpen() ) return;
 	assert( m_pBoard );
@@ -519,21 +540,29 @@ void GStream::Line(const QPoint& pA, const QPoint& pB)
 	Move(pA);
 	Draw(pB);
 }
-void GStream::WriteXY(const QPoint& p,  const bool& bFullLine)
+void GStream::WriteXY(const QPoint& p, bool bFullLine)
 {
 	// p has deciMil units (since GRIDPIXELS == 1000 for Gerber Export)
 	if ( !m_file.isOpen() ) return;
 	const int ix = ( m_bMetric ) ? ( 2540 * p.x() ) : p.x();	// metric ==> convert deciMil to nanometres
 	const int iy = ( m_bMetric ) ? ( 2540 * p.y() ) : p.y();	// metric ==> convert deciMil to nanometres
-	if ( bFullLine || m_iLastX != ix ) m_os << "X" << ix;
-	if ( bFullLine || m_iLastY != iy ) m_os << "Y" << iy;
+	if ( bFullLine || m_iLastX != p.x() ) m_os << "X" << ix;
+	if ( bFullLine || m_iLastY != p.y() ) m_os << "Y" << iy;
 	m_iLastX = p.x();
 	m_iLastY = p.y();
 }
-void GStream::WriteDrillOrdinate(const int& iDeciMils)
+void GStream::WriteDrillOrdinate(int iDeciMils)
 {
 	if ( !m_file.isOpen() || m_eType != GFILE::DRL ) return;
 	m_os << ( iDeciMils >= 0 ? "+" : "-" );	// Write sign
+	if ( XNC_FORMAT )
+	{
+		if ( m_bMetric )
+			m_os << MilToMM(iDeciMils/10, false);	// false ==> no leading zeros
+		else
+			m_os << MilToInch(iDeciMils/10, false);	// false ==> no leading zeros
+		return;
+	}
 	if ( m_bMetric )	// Writes mm in format AAAABBBBBB
 	{
 		// Millimetres in 4.6 format, 1 deciMil = 0000.002540 mm
@@ -561,7 +590,7 @@ void GStream::WriteDrillOrdinate(const int& iDeciMils)
 		m_os << iAbs;
 	}
 }
-QString GStream::MilToInch(const int& iMil, const bool& bLZ) const	// Get inches in format AA.BBBB (for defining drills and apertures)
+QString GStream::MilToInch(int iMil, bool bLZ) const	// Get inches in format AA.BBBB (for defining drills and apertures)
 {
 	QString str;
 	const int i		 = iMil * 10;				// deciMil
@@ -576,7 +605,7 @@ QString GStream::MilToInch(const int& iMil, const bool& bLZ) const	// Get inches
 	str += QString::fromStdString( std::to_string(remain) );
 	return str;
 }
-QString GStream::MilToMM(const int& iMil, const bool& bLZ) const	// Get mm in format AAAA.BBBBBB (for defining drills and apertures)
+QString GStream::MilToMM(int iMil, bool bLZ) const	// Get mm in format AAAA.BBBBBB (for defining drills and apertures)
 {
 	QString str;
 	const int i		 = iMil * 25400;			// nanometres
@@ -599,9 +628,12 @@ void GStream::GetQPoint(const QPointF& in, QPoint& out) const
 {
 	// GRIDPIXELS == 100 means each integer ordinate is 1 mil
 	assert( m_pBoard && m_pBoard->GetGRIDPIXELS() == 1000 );	// Confirm each integer ordinate == 0.0001 inches
-	const double dEdge = m_pBoard->GetEdgeWidth();	// Add/subtract this offset so bottom-left corner of board outline is at (0,0)
-	out.setX( static_cast<int>(in.x() + dEdge) );
-	out.setY( m_pBoard->GetGRIDPIXELS() * m_pBoard->GetRows() - static_cast<int>(in.y() - dEdge) ); // Gerber y-axis goes up screen
+	int gndL, gndR, gndT, gndB;
+	m_pBoard->GetGroundFillBounds(gndL, gndR, gndT, gndB);
+	const int iEdge = static_cast<int>( m_pBoard->GetEdgeWidth() );
+	const int iHeight(gndB - gndT + (iEdge<<1));
+ 	out.setX( static_cast<int>(in.x()) );
+	out.setY( iHeight - static_cast<int>(in.y()) ); // Gerber y-axis goes up screen
 }
 void GStream::GetQPolygon(const QPolygonF& in, QPolygon& out) const
 {
@@ -610,7 +642,7 @@ void GStream::GetQPolygon(const QPolygonF& in, QPolygon& out) const
 }
 
 // Wrapper for handling a set of Gerber files
-bool GWriter::Open(const QString& fileName, const Board& board, const bool& bVias, const bool& bTwoLayerGerber, const bool& bMetric, const bool& bConfirmEachFile)
+bool GWriter::Open(const QString& fileName, const Board& board, bool bVias, bool bSOIC, bool bSOIC16, bool bTwoLayerGerber, bool bMetric, bool bConfirmEachFile)
 {
 	QDateTime	local(QDateTime::currentDateTime());
 	QString		UTC = local.toTimeSpec(Qt::UTC).toString(Qt::ISODate);
@@ -631,7 +663,7 @@ bool GWriter::Open(const QString& fileName, const Board& board, const bool& bVia
 		if ( GFILE(i) == GFILE::GTL ) bDoFileClose = !bTwoLayerGerber;
 		if ( GFILE(i) == GFILE::GTS ) bDoFileClose = !bTwoLayerGerber;
 
-		bOK = m_os[i].Open(fileName, GFILE(i), bMetric, board, bVias, bConfirmEachFile);
+		bOK = m_os[i].Open(fileName, GFILE(i), bMetric, board, bVias, bSOIC, bSOIC16, bConfirmEachFile);
 
 		if ( bDoFileClose )
 			m_os[i].Close();
@@ -649,7 +681,7 @@ void GWriter::Close()	// Write footers, and close all file streams
 		m_os[i].Close();
 	}
 }
-GStream& GWriter::GetStream(const GFILE& eType)
+GStream& GWriter::GetStream(GFILE eType)
 {
 	return m_os[static_cast<size_t>(eType)];
 }
