@@ -96,6 +96,7 @@ public:
 		m_bCompEdit			= o.m_bCompEdit;
 		m_bXthermals		= o.m_bXthermals;
 		m_bInverseMono		= o.m_bInverseMono;
+		m_bColoredMono		= o.m_bColoredMono;
 		return *this;
 	}
 	bool operator==(const GuiControl& o) const	// Compare persisted info
@@ -151,7 +152,8 @@ public:
 			&&	m_bVerticalStrips	== o.m_bVerticalStrips
 			&&	m_bCompEdit			== o.m_bCompEdit
 			&&	m_bXthermals		== o.m_bXthermals
-			&&	m_bInverseMono		== o.m_bInverseMono;
+			&&	m_bInverseMono		== o.m_bInverseMono
+			&&	m_bColoredMono		== o.m_bColoredMono;
 	}
 	bool operator!=(const GuiControl& o) const
 	{
@@ -333,7 +335,9 @@ public:
 		m_bInverseMono = false;
 		if ( inStream.GetVersion() >= VRT_VERSION_57 )
 			inStream.Load(m_bInverseMono);		// Added in VRT_VERSION_57
-				
+		m_bColoredMono = false;
+		if ( inStream.GetVersion() >= VRT_VERSION_59 )
+			inStream.Load(m_bColoredMono);		// Added in VRT_VERSION_59
 	}
 	virtual void Save(DataStream& outStream) override
 	{
@@ -389,6 +393,7 @@ public:
 		outStream.Save(m_bCompEdit);		// Added in VRT_VERSION_19
 		outStream.Save(m_bXthermals);		// Added in VRT_VERSION_53
 		outStream.Save(m_bInverseMono);		// Added in VRT_VERSION_57
+		outStream.Save(m_bColoredMono);		// Added in VRT_VERSION_59
 	}
 	bool SetBackgroundColor(const MyRGB& o)	{ const bool bChanged = m_backgroundColor	!= o; m_backgroundColor	= o; return bChanged;}
 	bool SetCurrentLayer(int i)				{ const bool bChanged = m_currentLayer		!= i; m_currentLayer	= i; return bChanged; }
@@ -451,6 +456,7 @@ public:
 	bool SetCompEdit(bool b)				{ const bool bChanged = m_bCompEdit			!= b; m_bCompEdit		= b; return bChanged; }
 	bool SetXthermals(bool b)				{ const bool bChanged = m_bXthermals		!= b; m_bXthermals		= b; return bChanged; }
 	bool SetInverseMono(bool b)				{ const bool bChanged = m_bInverseMono		!= b; m_bInverseMono	= b; return bChanged; }
+	bool SetColoredMono(bool b)				{ const bool bChanged = m_bColoredMono		!= b; m_bColoredMono	= b; return bChanged; }
 	const MyRGB&		GetBackgroundColor() const	{ return m_backgroundColor; }
 	const int&			GetCurrentLayer() const		{ return m_currentLayer; }
 	const int&			GetCurrentNodeId() const	{ return m_currentNodeId; }
@@ -506,10 +512,11 @@ public:
 	const bool&			GetCompEdit() const			{ return m_bCompEdit; }
 	const bool&			GetXthermals() const		{ return m_bXthermals; }
 	const bool&			GetInverseMono() const		{ return m_bInverseMono; }
+	const bool&			GetColoredMono() const		{ return m_bColoredMono; }
 	// Helpers
 	bool		SetGroundNodeId()					{ return ( GetCurrentLayer() == 0 ) ? SetGroundNodeId0( GetCurrentNodeId() ) : SetGroundNodeId1( GetCurrentNodeId() ); }
 	const int&	GetGroundNodeId(int lyr) const		{ return ( lyr == 0 ) ? GetGroundNodeId0() : GetGroundNodeId1(); }
-	bool		GetUsePCBshapes()					{ return GetTrackMode() == TRACKMODE::PCB; }
+	bool		GetUsePCBshapes()					{ return GetTrackMode() == TRACKMODE::PCB || GetTrackMode() == TRACKMODE::MONO; }
 	bool		GetMirrored() const					{ return GetFlipH() || GetFlipV(); }
 	bool		SetTrackSliderValue(int i)			{ const bool bChanged = ( GetTrackSliderValue() != i ); SetTrackMode( static_cast<TRACKMODE>(i) ); return bChanged; }
 	bool		SetCompSliderValue(int i)			{ const bool bChanged = ( GetCompSliderValue()  != i ); SetCompMode(  static_cast<COMPSMODE>(i) ); return bChanged; }
@@ -562,6 +569,7 @@ public:
 		m_bCompEdit			= false;
 		m_bXthermals		= false;
 		m_bInverseMono		= false;
+		m_bColoredMono		= false;
 	}
 private:
 	MyRGB		m_backgroundColor	= MyRGB(0xFFFFFF);
@@ -616,4 +624,5 @@ private:
 	bool		m_bCompEdit			= false;			// true ==> component editor mode
 	bool		m_bXthermals		= false;			// true ==> force X thermal reliefs and hide tracks in ground-fill
 	bool		m_bInverseMono		= false;			// true ==> white tracks on black background
+	bool		m_bColoredMono		= false;			// true ==> colored tracks in mono mode
 };
