@@ -28,9 +28,9 @@ FindDialog::FindDialog(MainWindow* parent)
 {
 	ui->setupUi(this);
 	ui->radioName->setChecked(true);
-	QObject::connect(ui->radioName,	SIGNAL(toggled(bool)),			this, SLOT(ToggleName(bool)));
-	QObject::connect(ui->checkExact,SIGNAL(toggled(bool)),			this, SLOT(ToggleExact(bool)));
-	QObject::connect(ui->nameEdit,	SIGNAL(textChanged(QString)),	this, SLOT(TextChanged(QString)));
+	QObject::connect(ui->radioName,		SIGNAL(toggled(bool)),			this,	SLOT(ToggleName(bool)));
+	QObject::connect(ui->checkExact,	SIGNAL(toggled(bool)),			this,	SLOT(ToggleExact(bool)));
+	QObject::connect(ui->nameEdit,		SIGNAL(textChanged(QString)),	this,	SLOT(TextChanged(QString)));
 }
 
 FindDialog::~FindDialog()
@@ -38,44 +38,65 @@ FindDialog::~FindDialog()
 	delete ui;
 }
 
-void FindDialog::UpdateControls()
-{
-	if ( isHidden() && !m_str.isEmpty() )
-		ui->nameEdit->setText(QString(""));
-}
-
 void FindDialog::ToggleName(bool b)
 {
 	m_bName = b;
-	m_pMainWindow->Find(m_bName, m_bExact, m_str);
+	if ( m_bCanFind ) m_pMainWindow->Find(m_bName, m_bExact, m_str);
 }
 
 void FindDialog::ToggleExact(bool b)
 {
 	m_bExact = b;
-	m_pMainWindow->Find(m_bName, m_bExact, m_str);
+	if ( m_bCanFind ) m_pMainWindow->Find(m_bName, m_bExact, m_str);
 }
 
 void FindDialog::TextChanged(const QString& str)
 {
 	m_str = str;
-	m_pMainWindow->Find(m_bName, m_bExact, m_str);
+	if ( m_bCanFind ) m_pMainWindow->Find(m_bName, m_bExact, m_str);
+}
+
+void FindDialog::showEvent(QShowEvent* event)
+{
+	m_bCanFind = true;
+	if ( m_pMainWindow->GetNumFound() == 0 )
+		ui->nameEdit->setText(QString(""));
+	TextChanged(m_str);
+	QDialog::showEvent(event);
+}
+
+void FindDialog::hideEvent(QHideEvent* event)
+{
+	m_bCanFind = false;
+	QDialog::hideEvent(event);
 }
 
 void FindDialog::closeEvent(QCloseEvent* event)
 {
-	ui->nameEdit->setText(QString(""));
+	m_bCanFind = false;
 	QDialog::closeEvent(event);
 }
 
 void FindDialog::keyPressEvent(QKeyEvent* event)
 {
+#ifndef VEROROUTE_ANDROID
 	m_pMainWindow->specialKeyPressEvent(event);
+#endif
 	QDialog::keyPressEvent(event);
+	event->accept();
 }
 
 void FindDialog::keyReleaseEvent(QKeyEvent* event)
 {
+#ifdef VEROROUTE_ANDROID
+	if ( event->key() == Qt::Key_Back )
+	{
+		QTimer::singleShot(0, m_pMainWindow, SLOT(HideFindDialog()));
+		return event->accept();
+	}
+#else
 	m_pMainWindow->commonKeyReleaseEvent(event);
+#endif
 	QDialog::keyReleaseEvent(event);
+	event->accept();
 }
