@@ -21,17 +21,17 @@
 #include "ui_infodialog.h"
 #include "mainwindow.h"
 
-InfoDialog::InfoDialog(MainWindow* parent)
-: QDialog(parent)
-, ui(new Ui::InfoDialog)
-, m_pMainWindow(parent)
+InfoDialog::InfoDialog(QWidget* parent)
+: QWidget(parent)
+, ui(new Ui_InfoDialog)
+, m_pMainWindow(nullptr)
 {
-	ui->setupUi(this);
+	ui->setupUi( reinterpret_cast<QDialog*>(this) );
+#ifdef VEROROUTE_ANDROID
+	ui->textEdit->verticalScrollBar()->setStyleSheet( ANDROID_VSCROLL_WIDTH );
+	ui->textEdit->horizontalScrollBar()->setStyleSheet( ANDROID_HSCROLL_HEIGHT );
+#endif
 	ShowButtons(false);
-	QObject::connect(ui->plainTextEdit,	SIGNAL(textChanged()),	this,			SLOT(TextChanged()));
-	QObject::connect(ui->prev,			SIGNAL(clicked()),		m_pMainWindow,	SLOT(LoadPrevTutorial()));
-	QObject::connect(ui->reload,		SIGNAL(clicked()),		m_pMainWindow,	SLOT(LoadTutorial()));
-	QObject::connect(ui->next,			SIGNAL(clicked()),		m_pMainWindow,	SLOT(LoadNextTutorial()));
 }
 
 InfoDialog::~InfoDialog()
@@ -39,14 +39,23 @@ InfoDialog::~InfoDialog()
 	delete ui;
 }
 
+void InfoDialog::SetMainWindow(MainWindow* p)
+{
+	m_pMainWindow = p;
+	QObject::connect(ui->textEdit,	SIGNAL(textChanged()),	this,			SLOT(TextChanged()));
+	QObject::connect(ui->prev,		SIGNAL(clicked()),		m_pMainWindow,	SLOT(LoadPrevTutorial()));
+	QObject::connect(ui->reload,	SIGNAL(clicked()),		m_pMainWindow,	SLOT(LoadTutorial()));
+	QObject::connect(ui->next,		SIGNAL(clicked()),		m_pMainWindow,	SLOT(LoadNextTutorial()));
+}
+
 void InfoDialog::SetReadOnly(bool b)
 {
-	ui->plainTextEdit->setReadOnly(b);
+	ui->textEdit->setReadOnly(b);
 }
 
 void InfoDialog::TextChanged()
 {
-	m_pMainWindow->SetInfoStr( ui->plainTextEdit->toPlainText() );
+	m_pMainWindow->SetInfoStr( ui->textEdit->toPlainText() );
 }
 
 void InfoDialog::ShowButtons(bool b)
@@ -54,6 +63,10 @@ void InfoDialog::ShowButtons(bool b)
 	ui->prev->setVisible(b);
 	ui->reload->setVisible(b);
 	ui->next->setVisible(b);
+	// Set bigger text edit box if we hide the buttons
+	QRect rect = ui->textEdit->geometry();
+	rect.setHeight(b ? 593 : 632);
+	ui->textEdit->setGeometry(rect);
 }
 
 void InfoDialog::EnablePrev(bool b)
@@ -69,7 +82,14 @@ void InfoDialog::EnableNext(bool b)
 void InfoDialog::Update()
 {
 	m_initialStr = m_pMainWindow->m_board.GetInfoStr();
-	ui->plainTextEdit->setPlainText( QString::fromStdString(m_initialStr) );
+	ui->textEdit->clear();
+	QTextCursor			textCursor		= ui->textEdit->textCursor();
+	QTextBlockFormat	textBlockFormat	= textCursor.blockFormat();
+	//textBlockFormat.setAlignment(Qt::AlignJustify);	//TODO See if tutorials can be written so this is useable
+	textCursor.mergeBlockFormat(textBlockFormat);
+	ui->textEdit->setTextCursor(textCursor);
+	ui->textEdit->append( QString::fromStdString(m_initialStr) );
+	ui->textEdit->verticalScrollBar()->setSliderPosition(0);
 }
 
 bool InfoDialog::GetIsModified()
@@ -80,7 +100,7 @@ bool InfoDialog::GetIsModified()
 void InfoDialog::keyPressEvent(QKeyEvent* event)
 {
 	// In tutorial mode, forward event to main window (apart from special cases)
-	bool bForwardToMainWindow = ui->plainTextEdit->isReadOnly();
+	bool bForwardToMainWindow = ui->textEdit->isReadOnly();
 	switch( event->key() )
 	{
 		case Qt::Key_Tab:
@@ -92,15 +112,24 @@ void InfoDialog::keyPressEvent(QKeyEvent* event)
 	}
 	if ( bForwardToMainWindow )
 		return m_pMainWindow->keyPressEvent(event);
+#ifndef VEROROUTE_ANDROID
 	m_pMainWindow->specialKeyPressEvent(event);
-	QDialog::keyPressEvent(event);
+#endif
+	QWidget::keyPressEvent(event);
+	event->accept();
 }
 
 void InfoDialog::keyReleaseEvent(QKeyEvent* event)
 {
 	// In tutorial mode, forward event to main window
-	if ( ui->plainTextEdit->isReadOnly() )
+	if ( ui->textEdit->isReadOnly() )
 		return m_pMainWindow->keyReleaseEvent(event);
+#ifdef VEROROUTE_ANDROID
+	if ( event->key() == Qt::Key_Back )
+		return m_pMainWindow->keyReleaseEvent(event);	// Try Undo operation
+#else
 	m_pMainWindow->commonKeyReleaseEvent(event);
-	QDialog::keyReleaseEvent(event);
+#endif
+	QWidget::keyReleaseEvent(event);
+	event->accept();
 }
