@@ -18,7 +18,6 @@
 */
 
 #include "mainwindow.h"
-#include "ui_mainwindow.h"
 #include "padoffsetdialog.h"
 #include "GPainter.h"
 #include "SpanningTreeHelper.h"
@@ -585,7 +584,8 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 		painter.scale(1, -1);	// Mirror T-B
 	}
 
-	bool bInverseMono(bMono && board.GetInverseMono());
+	const bool bInverseMono(bMono && board.GetInverseMono());
+	const bool bColoredMono(bMono && board.GetColoredMono());
 
 	const QColor backgroundColor	= ( bInverseMono ) ? Qt::black : ( m_bWritePDF ) ? Qt::white : GetBackgroundColor();
 	m_backgroundPen.setColor(backgroundColor);
@@ -757,8 +757,9 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 
 				// Use GetPixmapRGB for pixmaps.  It can handle MY_GREY, MY_WHITE, MY_BLACK as special cases
 				const int	colorId				= colorMgr.GetColorId(nodeId);
-				const bool	bInvalidColor		= colorId == BAD_COLORID || ( bMonoPCB && nodeId != GetCurrentNodeId() );
-				
+				const bool	bInvalidColor		= colorId == BAD_COLORID || ( bPCB  && nodeId != GetCurrentNodeId() )
+																		 || ( bMono && nodeId != GetCurrentNodeId() && !bColoredMono )
+																		 || ( bMono && nodeId != GetCurrentNodeId() &&  bColoredMono && bGroundFill && nodeId == groundNodeId );
 				const int	iEffColorId			= ( bInvalidColor )	? groundFillColorId
 												: ( nodeId == GetCurrentNodeId() ) ? MY_GREY : ( colorId % MYNUMCOLORS );
 				const bool	 bAllowCustomColor	=	iEffColorId != MY_GREY		&&	iEffColorId != MY_WHITE		&&	iEffColorId != MY_BLACK
@@ -1188,7 +1189,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 					m_orangePen.setWidth(4);// Make found components stand out
 				else if ( bPlaced )
 				{
-					if ( bPCB )	// Use floating point pen width to better match Gerber output
+					if ( bMonoPCB )	// Use floating point pen width to better match Gerber output
 						penPlaced.setWidthF( bHighlightComp ? ( board.GetSilkWidth() * 1.5 )
 															: ( bMark ? ( board.GetSilkWidth() * 0.5 )
 																	  :   board.GetSilkWidth() ) );
@@ -1467,7 +1468,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 
 			painter.save();
 			double dCopyTextScale = dTextScale;
-			if ( bPCB )
+			if ( bMonoPCB )
 			{
 				dCopyTextScale *= m_board.GetTextSizeComp() * (20.0 / 243 );	// Scale to make the Gerber font size similar to regular component font size
 			}
@@ -1480,7 +1481,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 
 			// Use floating point pen width to better match Gerber output.
 			// Scale the pen width down to compensate for painter.scale() scaling things up in the loop below.
-			const double dPenWidth = ( bPCB ) ? board.GetSilkWidth() / dCopyTextScale : 0;
+			const double dPenWidth = ( bMonoPCB ) ? board.GetSilkWidth() / dCopyTextScale : 0;
 			m_orangePen.setWidthF(dPenWidth);	// Use colored text for found components
 			m_redPen.setWidthF(dPenWidth);		// Use red text for floating components
 			penPlaced.setWidthF(dPenWidth);		// Use this for placed components
@@ -1502,7 +1503,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 
 			painter.scale(dCopyTextScale, dCopyTextScale);
 			painter.setPen(bFound ? m_orangePen : bPlaced ? penPlaced : m_redPen);
-			painter.drawText(0,0,0,0, Qt::AlignCenter | Qt::TextDontClip, myStr.c_str(), bPCB);
+			painter.drawText(0,0,0,0, Qt::AlignCenter | Qt::TextDontClip, myStr.c_str(), bMonoPCB);
 			painter.restore();
 		}
 	}
@@ -1611,6 +1612,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 			const int textW = static_cast<int>((R-L)/dTextScale);
 			const int textH = static_cast<int>((B-T)/dTextScale);
 
+			painter.save();
 			font.setBold( rect.GetStyle() & TEXT_BOLD );
 			font.setItalic( rect.GetStyle() & TEXT_ITALIC );
 			font.setUnderline( rect.GetStyle() & TEXT_UNDERLINE );
@@ -1620,7 +1622,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 			m_varPen.setColor(!bMono ? rect.GetQColor() : ( bGroundFill == bInverseMono ) ? Qt::black : Qt::white);
 			painter.setPen(m_varPen);
 			painter.setBrush(Qt::NoBrush);
-			painter.save();
+
 			painter.translate((bMono && layer == 0) ? R : L, T);							// Mirror all text boxes in Mono mode for bottom layer
 			painter.scale((bMono && layer == 0) ? -dTextScale : dTextScale, dTextScale);	// Mirror all text boxes in Mono mode for bottom layer
 			painter.drawText(0, 0, textW, textH, Qt::TextWordWrap | rect.GetFlagsH() | rect.GetFlagsV(), QString::fromStdString(rect.GetStr()));
