@@ -231,11 +231,11 @@ void MainWindow::MousePressEvent(const QPoint& pos, bool bLeftClick, bool bRight
 	// Cursor modification
 	if ( GetSmartPan() )
 		centralWidget()->setCursor(Qt::ClosedHandCursor);
-	else if ( GetPaintPins() || GetErasePins() || GetPaintBoard() || GetEraseBoard() || GetPaintFlood() )
+	else if ( GetPaintPins() || GetErasePins() || GetPaintBoard() || GetEraseBoard() || GetPaintFlood() || GetEditLayerPref() )
 		centralWidget()->setCursor(Qt::CrossCursor);
 	else if ( GetResizingText() )
 		centralWidget()->setCursor(Qt::SizeFDiagCursor);
-	else if ( GetCurrentTextId() != BAD_TEXTID || GetCurrentCompId() != BAD_COMPID )
+	else if ( AllowCurrentTextId() || AllowCurrentCompId() )
 		centralWidget()->setCursor(Qt::ClosedHandCursor);
 	else
 		centralWidget()->setCursor(Qt::OpenHandCursor);
@@ -316,7 +316,7 @@ void MainWindow::MousePressEvent(const QPoint& pos, bool bLeftClick, bool bRight
 		if ( compMode != COMPSMODE::OFF )
 		{
 			// Component selection (if text is not selected)
-			const int compId = ( GetCurrentTextId() == BAD_TEXTID ) ? m_board.GetComponentId(m_gridRow, m_gridCol) : BAD_COMPID;
+			const int compId = !AllowCurrentTextId() ? m_board.GetComponentId(m_gridRow, m_gridCol) : BAD_COMPID;
 			if ( GetCurrentCompId() != compId ) SetCurrentCompId(compId);
 
 			// Group manipulation
@@ -325,7 +325,7 @@ void MainWindow::MousePressEvent(const QPoint& pos, bool bLeftClick, bool bRight
 			{
 				groupMgr.UpdateUserGroup( GetCurrentCompId() );	// Add/remove current comp (and its siblings) to user group
 				UpdateControls();
-				if ( GetCurrentTextId() == BAD_TEXTID )
+				if ( !AllowCurrentTextId() )
 					SetMouseActionString("(un)select part(s)");
 			}
 			else if ( !groupMgr.GetIsUserComp( GetCurrentCompId() ) )
@@ -334,7 +334,7 @@ void MainWindow::MousePressEvent(const QPoint& pos, bool bLeftClick, bool bRight
 				groupMgr.ResetUserGroup( GetCurrentCompId() );	// Reset the user group with the current comp (and its siblings)
 				compMgr.ClearTrax();
 				UpdateControls();
-				if ( GetCurrentTextId() == BAD_TEXTID )
+				if ( !AllowCurrentTextId() )
 					SetMouseActionString("(un)select part(s)");
 			}
 		}
@@ -343,7 +343,7 @@ void MainWindow::MousePressEvent(const QPoint& pos, bool bLeftClick, bool bRight
 	// Painting/Unpainting the component pins or board
 	if ( GetSmartPan() || GetShiftKeyDown() || trackMode == TRACKMODE::OFF ) return;
 
-	if ( GetCurrentTextId() != BAD_TEXTID )
+	if ( AllowCurrentTextId() )
 	{
 		if ( m_dockPinDlg->isVisible() ) m_dockPinDlg->hide();
 		HidePadOffsetDialog();
@@ -352,8 +352,9 @@ void MainWindow::MousePressEvent(const QPoint& pos, bool bLeftClick, bool bRight
 
 	const Element* pC = m_board.Get(layer, m_gridRow, m_gridCol);
 
-	const bool bColor = ( trackMode == TRACKMODE::COLOR );
-	if ( GetPaintFlood() && bColor )
+	const bool bColor		= ( trackMode == TRACKMODE::COLOR );
+	const bool bColoredMono	= ( trackMode == TRACKMODE::MONO && m_board.GetColoredMono() );
+	if ( GetPaintFlood() && (bColor || bColoredMono) )
 	{
 		if ( m_dockPinDlg->isVisible() ) m_dockPinDlg->hide();
 		HidePadOffsetDialog();
@@ -381,7 +382,7 @@ void MainWindow::MousePressEvent(const QPoint& pos, bool bLeftClick, bool bRight
 			m_bReRoute = m_bReListNodes = true;
 		}
 	}
-	else if ( ( (GetPaintPins() || GetErasePins()) && bColor ) || GetPaintBoard() || GetEraseBoard() )
+	else if ( ( (GetPaintPins() || GetErasePins()) && (bColor || bColoredMono) ) || GetPaintBoard() || GetEraseBoard() )
 	{
 		if ( m_dockPinDlg->isVisible() ) m_dockPinDlg->hide();
 		HidePadOffsetDialog();
@@ -473,6 +474,22 @@ void MainWindow::MousePressEvent(const QPoint& pos, bool bLeftClick, bool bRight
 			m_bReRoute = m_bReListNodes = true;
 		}
 	}
+	else if ( GetEditLayerPref() )
+	{
+		const bool bCloseToGridPoint = ( hypot(dRow - 0.5, dCol - 0.5) <= 0.5 );	// true ==> clicked close to grid point
+		if ( bCloseToGridPoint && pC->GetPinSupportsLayerPref() )
+		{
+			const bool bToggled = m_board.ToggleLyrPref(layer, m_gridRow, m_gridCol);	assert(bToggled);
+			if ( bToggled )
+			{
+				size_t	pinIndex;
+				int		compId;
+				m_board.GetSlotInfoForTH(pC, pinIndex, compId);
+	
+				SetMouseActionString("change pin layer preference", compId);
+			}
+		}
+	}
 	else
 	{
 		if ( ALLOW_DELAY_BASED_PAD_SHIFT && pC->GetPinSupportsOffsetPads() )
@@ -513,12 +530,11 @@ void MainWindow::MouseDoubleClickEvent(const QPoint& pos)
 		return;
 	}
 
-	if ( GetCurrentTextId() != BAD_TEXTID )
+	if ( AllowCurrentTextId() )
 		return ShowTextDialog();
 
 	const bool bCompsOn	= m_board.GetCompMode()  != COMPSMODE::OFF;
 	const bool bTrackOn	= m_board.GetTrackMode() != TRACKMODE::OFF;
-	const bool bPCB		= m_board.GetTrackMode() == TRACKMODE::PCB;
 
 	if ( !bTrackOn && !bCompsOn ) return;
 
@@ -540,28 +556,6 @@ void MainWindow::MouseDoubleClickEvent(const QPoint& pos)
 
 	const bool bCloseToGridPoint = ( hypot(dRow - 0.5, dCol - 0.5) <= 0.5 );	// true ==> clicked close to grid point
 
-	// Handle changing layer preference for PCBs via double-clicking on a component pin
-	if ( bPCB && pC->GetPinSupportsLayerPref() && !GetPaintAction() && m_board.GetLyrs() == 2 && bCloseToGridPoint )	// Only consider clicks that are close to the grid point
-	{
-		const bool bToggled = m_board.ToggleLyrPref(layer, m_gridRow, m_gridCol);	assert(bToggled);
-		if ( bToggled )
-		{
-			size_t	pinIndex;
-			int		compId;
-			m_board.GetSlotInfoForTH(pC, pinIndex, compId);
-
-			SetMouseActionString("change pin layer preference", compId);
-
-			// Also handle change of nodeID ...
-			if ( GetCurrentNodeId() != pC->GetNodeId() )
-			{
-				SetCurrentNodeId( pC->GetNodeId() );
-				m_bReRoute = true;	// Dont' need to set m_bReListNodes when choosing different nodeID
-			}
-			return;
-		}
-	}
-
 	// Handle competing diagonals
 	if ( bTrackOn && !GetPaintAction() )
 	{
@@ -644,7 +638,7 @@ void MainWindow::MouseMoveEvent(const QPoint& pos)
 
 	if ( ALLOW_DELAY_BASED_SMART_PAN
 		 && !m_board.GetCompEdit() && !GetDefiningRect() && !GetPaintBoard() && !GetEraseBoard() && !CanModifyRuler()
-		 && GetCurrentTextId() == BAD_TEXTID && GetCurrentCompId() == BAD_COMPID )
+		 && !AllowCurrentTextId() && !AllowCurrentCompId() )
 	{
 		const auto elapsed		= std::chrono::steady_clock::now() - g_lastMouseClickTime;
 		const auto duration_ms	= std::chrono::duration_cast<std::chrono::milliseconds>(elapsed).count();
@@ -658,11 +652,11 @@ void MainWindow::MouseMoveEvent(const QPoint& pos)
 		centralWidget()->setCursor(Qt::ClosedHandCursor);
 	else if ( GetDefiningRect() || GetResizingText() )
 		centralWidget()->setCursor(Qt::SizeFDiagCursor);
-	else if ( GetPaintBoard() || GetEraseBoard() )
+	else if ( GetPaintBoard() || GetEraseBoard() || GetPaintPins() || GetErasePins() || GetPaintFlood() || GetEditLayerPref() )
 		centralWidget()->setCursor(Qt::CrossCursor);
-	else if ( GetCurrentTextId() != BAD_TEXTID && m_bMouseClick )
+	else if ( AllowCurrentTextId() && m_bMouseClick )
 		centralWidget()->setCursor(Qt::ClosedHandCursor);
-	else if ( GetCurrentCompId() != BAD_COMPID && m_bMouseClick )
+	else if ( AllowCurrentCompId() && m_bMouseClick )
 		centralWidget()->setCursor(Qt::ClosedHandCursor);
 	else
 		centralWidget()->setCursor(Qt::OpenHandCursor);
@@ -777,7 +771,7 @@ void MainWindow::MouseMoveEvent(const QPoint& pos)
 		m_board.WipeAutoSetPoints();
 		m_board.PlaceFloaters();	// See if we can now place floating components down
 	}
-	else if ( !GetSmartPan() && GetCurrentTextId() != BAD_TEXTID )
+	else if ( !GetSmartPan() && AllowCurrentTextId() )
 	{
 		if ( HaveZeroDeltaRowCol(deltaRow, deltaCol) ) return;	// No change
 
@@ -804,7 +798,7 @@ void MainWindow::MouseMoveEvent(const QPoint& pos)
 		}
 		SetMouseActionString(GetResizingText() ? "resize text box" : "move text box", GetCurrentTextId());
 	}
-	else if ( !GetSmartPan() && GetCurrentCompId() != BAD_COMPID && compMode != COMPSMODE::OFF )	// Move user-group components
+	else if ( !GetSmartPan() && AllowCurrentCompId() && compMode != COMPSMODE::OFF )	// Move user-group components
 	{
 		if ( HaveZeroDeltaRowCol(deltaRow, deltaCol) ) return;	// No change
 
@@ -914,7 +908,7 @@ void MainWindow::MouseReleaseEvent(const QPoint& pos)
 		SelectAllInRects();
 		ShowCurrentRectSize();
 	}
-	else if ( GetPaintPins() || GetErasePins() || GetPaintBoard() || GetEraseBoard() || GetPaintFlood() )
+	else if ( GetPaintPins() || GetErasePins() || GetPaintBoard() || GetEraseBoard() || GetPaintFlood() || GetEditLayerPref() )
 		centralWidget()->setCursor(Qt::CrossCursor);
 	else
 		centralWidget()->setCursor(Qt::OpenHandCursor);
@@ -1014,7 +1008,7 @@ void MainWindow::keyPressEvent(QKeyEvent* event)
 		{
 			case Qt::Key_Z:			CompRotateCCW();	break;
 			case Qt::Key_X:			CompRotateCW();		break;
-			case Qt::Key_Delete:	if ( GetCurrentTextId() != BAD_TEXTID || ( nComps && compMode != COMPSMODE::OFF ) )
+			case Qt::Key_Delete:	if ( AllowCurrentTextId() || ( nComps && compMode != COMPSMODE::OFF ) )
 										Delete();	//	So delete works like the backspace keyboard shortcut
 									break;
 		}
@@ -1022,14 +1016,16 @@ void MainWindow::keyPressEvent(QKeyEvent* event)
 	// Painting/Unpainting
 	if ( !bIsAutoRepeat )
 	{
+		const bool bAllowPaintPins = ( compMode != COMPSMODE::OFF ) &&
+									 ( ( trackMode == TRACKMODE::COLOR ) || ( trackMode == TRACKMODE::MONO && m_board.GetColoredMono() ) );
 		// Only one flag for paint-board/paint-pins/paint-flood must be true
 		switch( event->key() )
 		{
-			case Qt::Key_P:		if ( trackMode != TRACKMODE::COLOR || compMode == COMPSMODE::OFF || GetPaintBoard() || GetEraseBoard() || GetPaintFlood() ) return;
+			case Qt::Key_P:		if ( !bAllowPaintPins || GetPaintBoard() || GetEraseBoard() || GetPaintFlood() ) return;
 								SetPaintPins(true);		break;
 			case Qt::Key_Space:	if ( trackMode == TRACKMODE::OFF || GetPaintPins() || GetErasePins() || GetPaintFlood() ) return;
 								SetPaintBoard(true);	break;
-			case Qt::Key_F:		if ( trackMode != TRACKMODE::COLOR || compMode == COMPSMODE::OFF || GetPaintBoard() || GetEraseBoard() || GetPaintPins() || GetErasePins() || m_board.GetRoutingEnabled() ) return;
+			case Qt::Key_F:		if ( !bAllowPaintPins || GetPaintBoard() || GetEraseBoard() || GetPaintPins() || GetErasePins() || m_board.GetRoutingEnabled() ) return;
 								SetPaintFlood(true);	break;
 			case Qt::Key_W:		WipeTracks();	break;
 		}
@@ -1073,9 +1069,9 @@ void MainWindow::keyReleaseEvent(QKeyEvent* event)
 			case Qt::Key_F:		SetPaintFlood(false);	break;
 			case Qt::Key_Space:	SetPaintBoard(false);	break;
 			default:
-				if ( GetCurrentTextId() != BAD_TEXTID && m_bMouseClick )
+				if ( AllowCurrentTextId() && m_bMouseClick )
 					centralWidget()->setCursor(Qt::ClosedHandCursor);
-				else if ( GetCurrentCompId() != BAD_COMPID && m_bMouseClick )
+				else if ( AllowCurrentCompId() && m_bMouseClick )
 					centralWidget()->setCursor(Qt::ClosedHandCursor);
 				else
 					centralWidget()->setCursor(Qt::OpenHandCursor);
@@ -1201,6 +1197,15 @@ void MainWindow::SetPaintFlood(bool b)
 	centralWidget()->setCursor(b ? Qt::CrossCursor : Qt::OpenHandCursor);
 	UpdateControls();
 }
+void MainWindow::SetEditLayerPref(bool b)
+{
+	if ( b == GetEditLayerPref() ) return;
+	if ( b ) HideAllNonDockedDlgs();
+	m_eMouseMode = ( b ) ? MOUSE_MODE::EDIT_LAYER_PREF : MOUSE_MODE::SELECT;
+	centralWidget()->setCursor(b ? Qt::CrossCursor : Qt::OpenHandCursor);
+	UpdateControls();
+}
+
 void MainWindow::SetDefiningRect(bool b)
 {
 	if ( b == GetDefiningRect() ) return;
@@ -1212,7 +1217,7 @@ void MainWindow::SetDefiningRect(bool b)
 void MainWindow::SetResizingText(bool b)
 {
 	if ( b == GetResizingText() ) return;
-	if ( b ) { m_wireDlg->hide();	m_bomDlg->hide();	m_findDlg->hide(); }	// Mutually exclusive with 	m_textDlg
+	if ( b ) { m_wireDlg->hide();	m_bomDlg->hide();	m_findDlg->hide(); }	// Mutually exclusive with m_textDlg
 	m_eMouseMode = ( b ) ? MOUSE_MODE::RESIZE_TEXT : MOUSE_MODE::SELECT;
 	UpdateControls();
 }
