@@ -21,14 +21,16 @@
 #include "ui_pindialog.h"
 #include "mainwindow.h"
 
-PinDialog::PinDialog(MainWindow* parent)
-: QDialog(parent)
+PinDialog::PinDialog(QWidget* parent)
+: QWidget(parent)
 , ui(new Ui_PinDialog)
-, m_pMainWindow(parent)
+, m_pMainWindow(nullptr)
 {
-	ui->setupUi(this);
-
-	QObject::connect(ui->tableWidget,	SIGNAL(cellChanged(int,int)),	this,	SLOT(CellChanged(int,int)));
+	ui->setupUi( reinterpret_cast<QDialog*>(this) );
+#ifdef VEROROUTE_ANDROID
+	ui->tableWidget->verticalScrollBar()->setStyleSheet( ANDROID_VSCROLL_WIDTH );
+	ui->tableWidget->horizontalScrollBar()->setStyleSheet( ANDROID_HSCROLL_HEIGHT );
+#endif
 }
 
 PinDialog::~PinDialog()
@@ -36,6 +38,12 @@ PinDialog::~PinDialog()
 	delete ui;
 }
 
+void PinDialog::SetMainWindow(MainWindow* p)
+{
+	m_pMainWindow = p;
+	QObject::connect(ui->tableWidget,	SIGNAL(cellChanged(int,int)),	this,	SLOT(CellChanged(int,int)));
+}
+
 Component* PinDialog::GetUserComp() const
 {
 	if ( m_pMainWindow->m_board.GetGroupMgr().GetNumUserComps() != 1 ) return nullptr;	// Need single component selected
@@ -47,16 +55,25 @@ void PinDialog::CellChanged(int row, int col)
 {
 	if ( col == 0 ) return;
 
-	Component*			pComp		= GetUserComp();	assert( pComp );
+	const bool& bCompEdit = m_pMainWindow->m_board.GetCompEdit();
+
+	CompDefiner*	pDef	= bCompEdit ? &m_pMainWindow->m_board.GetCompDefiner() : nullptr;
+	Component*		pComp	= bCompEdit ? nullptr : GetUserComp();
+	assert(pDef != nullptr || pComp != nullptr);
+
 	QTableWidgetItem*	pItemLabel	= ui->tableWidget->item(row, col);
 	const size_t		iPinIndex	= static_cast<size_t>(row);
 	const std::string	strLabel	= pItemLabel->text().toStdString();
+	const int			objId		= static_cast<int>(iPinIndex) + ( pComp ? ( pComp->GetId() * 1000000 ) : 0);
 
 	if ( col == 1 )
 	{
-		if ( pComp->GetPinLabel(iPinIndex) != strLabel ) // If changed
+		const std::string& s = pDef ? pDef->GetPinLabel(iPinIndex) : pComp->GetPinLabel(iPinIndex);
+		if ( s != strLabel ) // If changed
 		{
-			pComp->SetPinLabel(iPinIndex, strLabel);
+			if ( pDef  ) pDef->SetPinLabel(iPinIndex, strLabel);
+			if ( pComp ) pComp->SetPinLabel(iPinIndex, strLabel);
+			m_pMainWindow->UpdateHistory("change pin label", objId);
 			m_pMainWindow->RepaintSkipRouting();
 		}
 	}
@@ -64,9 +81,12 @@ void PinDialog::CellChanged(int row, int col)
 	{
 		const int iAlign = ( strLabel == "L" || strLabel == "l" ) ? Qt::AlignLeft  :
 						   ( strLabel == "R" || strLabel == "r" ) ? Qt::AlignRight : Qt::AlignHCenter;
-		if ( pComp->GetPinAlign(iPinIndex) != iAlign ) // If changed
+		const int& i = pDef ? pDef->GetPinAlign(iPinIndex) : pComp->GetPinAlign(iPinIndex);
+		if ( i != iAlign ) // If changed
 		{
-			pComp->SetPinAlign(iPinIndex, iAlign);
+			if ( pDef )  pDef->SetPinAlign(iPinIndex, iAlign);
+			if ( pComp ) pComp->SetPinAlign(iPinIndex, iAlign);
+			m_pMainWindow->UpdateHistory("change pin label alignment", objId);
 			m_pMainWindow->RepaintSkipRouting();
 		}
 		if ( strLabel != "L" && strLabel != "R" && strLabel != "C" )
@@ -76,15 +96,20 @@ void PinDialog::CellChanged(int row, int col)
 
 void PinDialog::Update()
 {
-	Component*		pComp	= GetUserComp();
-	const size_t	numPins	= ( pComp ) ? pComp->GetNumPins() : 0;
+	const bool& bCompEdit = m_pMainWindow->m_board.GetCompEdit();
+
+	CompDefiner*	pDef	= bCompEdit ? &m_pMainWindow->m_board.GetCompDefiner() : nullptr;
+	Component*		pComp	= bCompEdit ? nullptr : GetUserComp();
+	const size_t	numPins	= pDef ? pDef->GetNumPins() : pComp ? pComp->GetNumPins() : 0;
+	assert(pDef != nullptr || pComp != nullptr || numPins == 0);
+
 	// Set up the table
 	ui->tableWidget->clear();
 	ui->tableWidget->setRowCount(static_cast<int>(numPins));
 	ui->tableWidget->setColumnCount(3);
 	ui->tableWidget->setColumnWidth(0,40);
 	ui->tableWidget->setColumnWidth(1,105);
-	ui->tableWidget->setColumnWidth(2,50);
+	ui->tableWidget->setColumnWidth(2,65);	// Allow for vertical scroll bar
 	m_tableHeader << "Pin" << "Label" << "Align";
 	ui->tableWidget->setHorizontalHeaderLabels(m_tableHeader);
 	ui->tableWidget->verticalHeader()->setVisible(false);
@@ -96,14 +121,14 @@ void PinDialog::Update()
 	// Populate the table with data
 	for (size_t iPinIndex = 0; iPinIndex < numPins; iPinIndex++)
 	{
-		const int& iAlign = pComp->GetPinAlign(iPinIndex);
+		const int& iAlign = pDef ? pDef->GetPinAlign(iPinIndex) : pComp->GetPinAlign(iPinIndex);
 		for (int iCol = 0; iCol < 3; iCol++)
 		{
 			std::string str;
 			switch( iCol )
 			{
 				case 0:	str = CompTypes::GetDefaultPinLabel(iPinIndex);	break;
-				case 1:	str = pComp->GetPinLabel(iPinIndex);			break;
+				case 1:	str = pDef ? pDef->GetPinLabel(iPinIndex) : pComp->GetPinLabel(iPinIndex);	break;
 				case 2:	str = ( iAlign == Qt::AlignLeft  ) ? "L" :
 							  ( iAlign == Qt::AlignRight ) ? "R" :"C";	break;
 			}
@@ -123,12 +148,21 @@ void PinDialog::Update()
 
 void PinDialog::keyPressEvent(QKeyEvent* event)
 {
+#ifndef VEROROUTE_ANDROID
 	m_pMainWindow->specialKeyPressEvent(event);
-	QDialog::keyPressEvent(event);
+#endif
+	QWidget::keyPressEvent(event);
+	event->accept();
 }
 
 void PinDialog::keyReleaseEvent(QKeyEvent* event)
 {
+#ifdef VEROROUTE_ANDROID
+	if ( event->key() == Qt::Key_Back )
+		return m_pMainWindow->keyReleaseEvent(event);	// Try Undo operation
+#else
 	m_pMainWindow->commonKeyReleaseEvent(event);
-	QDialog::keyReleaseEvent(event);
+#endif
+	QWidget::keyReleaseEvent(event);
+	event->accept();
 }
