@@ -106,7 +106,7 @@ void TemplatesDialog::Update()
 			const Component&	comp	= mgr.GetNth(bGeneric, static_cast<size_t>(i));
 			const COMP&			eType	= comp.GetType();
 
-			strType		= comp.GetFullTypeStr();			// e.g. "Film" or "DIP16"
+			strType		= bGeneric ? comp.GetTypeStr() : comp.GetFullTypeStr();	// e.g. Use "DIP" in generic list rather than "DIP16"
 			strValue	= comp.GetValueStr();
 			strFull		= CompTypes::GetFamilyStr( eType );	// e.g. "Capacitor"
 			if ( !StringHelper::IsEmptyStr(strFull) ) strFull += ": ";
@@ -175,6 +175,16 @@ void TemplatesDialog::UserDoubleClicked(int row, int)
 		m_pMainWindow->AddFromTemplate(bGeneric, mgr.GetNth(bGeneric, static_cast<size_t>(m_iRowR)));
 }
 
+const Component* TemplatesDialog::GetCurrentUserComp() const
+{
+	TemplateManager& mgr = m_pMainWindow->GetTemplateManager();
+
+	const bool bGeneric = false;
+	if ( m_iRowR >= 0 && m_iRowR < static_cast<int>(mgr.GetSize(bGeneric)) )
+		return &mgr.GetNth(bGeneric, static_cast<size_t>(m_iRowR));
+	return nullptr;
+}
+
 void TemplatesDialog::AddTemplates()
 {
 	AddTemplatesFromBoard(m_pMainWindow->m_board, false, true);	// false ==> Restrict to user-group
@@ -194,9 +204,12 @@ void TemplatesDialog::DeleteTemplate()
 									 + "(Value = " + comp.GetValueStr() + ") is about to be deleted.  There is no undo for this operation.  Continue?";
 		if ( QMessageBox::question(this, tr("Confirm Delete Template"),
 										 tr(messageStr.c_str()),
-										 QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::No ) return;	
+										 QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::No ) return;
 		if ( mgr.Remove(comp) )
+		{
 			Update();
+			m_pMainWindow->UpdateAliasDialog();
+		}
 	}
 }
 
@@ -293,16 +306,73 @@ void TemplatesDialog::AddTemplatesFromBoard(Board& board, bool bAllComps, bool b
 		QMessageBox::information(this, tr("Information"), tr("No parts are currently selected in the main view."));
 		return;
 	}
-	
+
 	const bool bGeneric = false;
 
+	bool bOverWriteAll_AlreadyExists(false);	// To handle the "Yes To All" case
+	bool bOverWriteAll_UsedImportStr(false);	// To handle the "Yes To All" case
+
+	std::list< std::string > errorStrList;
 	int nCount(0);
 	for (const auto& mapObj : compMgr.GetMapIdToComp())
 	{
 		if ( bAllComps || groupMgr.GetIsUserComp(mapObj.first) )
-			if ( mgr.Add(bGeneric, mapObj.second) ) nCount++;
+		{
+			bool bAlreadyExists(false), bUsedImportStr(false);	// Gets set true if template already exists, or import string is already in use 
+			std::string errorStr;
+			const Component& comp =  mapObj.second;
+			bool bOK = mgr.Add(bGeneric, comp, bAlreadyExists, bUsedImportStr, &errorStr);
+			if ( !bOK && ( bAlreadyExists || bUsedImportStr ) )
+			{
+				bool bFound(false);	// true ==> we've encountered the error before (i.e. a part with same type and value)
+				for (auto& str : errorStrList )
+					if ( str == errorStr ) { bFound = true;  break; }
+				if ( !bFound )
+				{
+					errorStrList.push_back(errorStr);
+
+					bool bOverWrite(false);
+					if ( bAlreadyExists )
+					{
+						if ( bOverWriteAll_AlreadyExists )
+							bOverWrite = true;
+						else
+						{
+							auto button = QMessageBox::question(this, tr("Confirm Overwrite"),
+																tr(errorStr.c_str()) + tr(" and will be overwritten.  There is no undo for this operation.  Continue?"),
+																QMessageBox::Yes | QMessageBox::YesToAll | QMessageBox::No, QMessageBox::No);
+							if ( button == QMessageBox::YesToAll )
+								bOverWriteAll_AlreadyExists = true;
+							bOverWrite = bOverWriteAll_AlreadyExists || ( button == QMessageBox::Yes );
+						}
+					}
+					else if ( bUsedImportStr )
+					{
+						if ( bOverWriteAll_UsedImportStr )
+							bOverWrite = true;
+						else
+						{
+							auto button = QMessageBox::question(this, tr("Confirm Overwrite"),
+																tr(errorStr.c_str()) + tr(".  It will be overwritten with (Value ='") + tr(comp.GetValueStr().c_str()) +
+																tr("').  There is no undo for this operation.  Continue?"),
+																QMessageBox::Yes | QMessageBox::YesToAll | QMessageBox::No, QMessageBox::No);
+							if ( button == QMessageBox::YesToAll )
+								bOverWriteAll_UsedImportStr = true;
+							bOverWrite = bOverWriteAll_UsedImportStr || ( button == QMessageBox::Yes );
+						}
+					}
+					if ( bOverWrite )
+						bOK = mgr.Add(bGeneric, comp, bAlreadyExists, bUsedImportStr, &errorStr);	// Repeat Add() with bAlreadyExists or bUsedImportStr set true to allow overwrite
+				}
+			}
+			if ( bOK )
+			{
+				nCount++;
+				Update();
+				m_pMainWindow->UpdateAliasDialog();
+			}
+		}
 	}
-	if ( nCount > 0 ) Update();
 
 	if ( bInfoMsg )
 		QMessageBox::information(this, tr("Information"), QString::number(nCount) + QString(" new templates added."));
