@@ -23,7 +23,7 @@
 
 TextDialog::TextDialog(MainWindow* parent)
 : QDialog(parent)
-, ui(new Ui::TextDialog)
+, ui(new Ui_TextDialog)
 , m_pMainWindow(parent)
 {
 	ui->setupUi(this);
@@ -77,12 +77,18 @@ void TextDialog::TextChanged()
 
 void TextDialog::keyPressEvent(QKeyEvent* event)
 {
+#ifndef VEROROUTE_ANDROID
 	m_pMainWindow->specialKeyPressEvent(event);
+#endif
 	QDialog::keyPressEvent(event);
+	event->accept();
 }
 
 void TextDialog::keyReleaseEvent(QKeyEvent* event)
 {
+#ifndef VEROROUTE_ANDROID
 	m_pMainWindow->commonKeyReleaseEvent(event);
+#endif
 	QDialog::keyReleaseEvent(event);
+	event->accept();
 }
